Package: RProtoBuf
Version: 0.4.28
Date: 2026-08-20
Authors@R: c(person("Romain", "Francois", role = "aut",
                    comment = c(ORCID = "0000-0002-2444-4226")),
             person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Murray", "Stokely", role = "aut",
                    comment = c(ORCID = "0009-0008-3390-1338")),
             person("Jeroen", "Ooms", role = "aut",
                    comment = c(ORCID = "0000-0002-4035-0289")))
Title: R Interface to the 'Protocol Buffers' 'API' (Version 2 or 3)
Description: Protocol Buffers are a way of encoding structured data in an
 efficient yet extensible format. Google uses Protocol Buffers for almost all
 of its internal 'RPC' protocols and file formats.  Additional documentation
 is available in two included vignettes one of which corresponds to our 'JSS'
 paper (2016, <doi:10.18637/jss.v071.i02>. A sufficiently recent version of
 'Protocol Buffers' library is required; currently version 3.3.0 from 2017
 is the tested minimum.
Imports: methods, utils, stats, tools, Rcpp
LinkingTo: Rcpp
Suggests: tinytest
SystemRequirements: ProtoBuf libraries and compiler version 3.3.0 or
        later; On Debian/Ubuntu these can be installed as
        libprotoc-dev, libprotobuf-dev and protobuf-compiler, while on
        Fedora/CentOS protobuf-devel and protobuf-compiler are needed.
        A modern compiler is required as well.
BugReports: https://github.com/eddelbuettel/rprotobuf/issues
URL: https://github.com/eddelbuettel/rprotobuf,
        https://dirk.eddelbuettel.com/code/rprotobuf.html
License: GPL (>= 2)
VignetteBuilder: Rcpp
NeedsCompilation: yes
Packaged: 2026-08-21 13:39:09 UTC; edd
Author: Romain Francois [aut] (ORCID: <https://orcid.org/0000-0002-2444-4226>),
  Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Murray Stokely [aut] (ORCID: <https://orcid.org/0009-0008-3390-1338>),
  Jeroen Ooms [aut] (ORCID: <https://orcid.org/0000-0002-4035-0289>)
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-08-21 22:01:25 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-21 23:16:07 UTC; unix
Archs: RProtoBuf.so.dSYM

## Package has dynamic library
useDynLib("RProtoBuf", .registration=TRUE)

importFrom("methods", "new", "show", "as", "is")
importFrom("utils", "str", "packageDescription", "packageVersion")
importFrom("stats", "update")
importFrom("tools", "file_path_as_absolute")
#importFrom("RCurl", "basicTextGatherer", "curlPerform")
importFrom("Rcpp", "evalCpp")

exportClasses(

	# descriptors
	"Descriptor",
	"FieldDescriptor", "EnumDescriptor",
	"ServiceDescriptor", "MethodDescriptor",
	"FileDescriptor", "EnumValueDescriptor",

	# message
	"Message",

	# rpc
	#"RpcHTTP",

	# virtual streams
	"ZeroCopyInputStream", "ZeroCopyOutputStream",

	# concrete implementations
	"ArrayInputStream", "ArrayOutputStream",
	"FileInputStream", "FileOutputStream",
	"ConnectionInputStream", "ConnectionOutputStream"

	)

exportMethods( "new", "[[", "[[<-", "$", "$<-", "show",
	"serialize", "as.character", "update",
	"has", "clone", "merge", "read", "readASCII", "readJSON",
	"bytesize", "length", "str", "isInitialized",
	"clear", "size", "size<-", "swap",
	"descriptor", "set", "fetch", "toString", "toJSON",
	"identical", "==", "!=", "all.equal", "add",
	"fileDescriptor", "name", "names", "flush", "close",
        "setExtension", "getExtension",

        "containing_type",
	"field_count", "nested_type_count", "enum_type_count",
	"field", "nested_type", "enum_type",

	# FieldDescriptor
	"is_extension", "number",
	"is_required", "is_repeated",
	"has_default_value", "default_value",
	"message_type", "enum_type", "type", "cpp_type",

	# EnumDescriptor
	"value_count", "value",

	# ServiceDescriptor
	"method_count", "method",

	# MethodDescriptor
	"input_type", "output_type",

	# rpc
#	"invoke",

	# streams
	"Next", "ByteCount", "BackUp", "Skip",
	"ArrayInputStream", "ArrayOutputStream", "FileInputStream", "FileOutputStream",
	"GetErrno", "SetCloseOnDelete",
	"ConnectionInputStream", "ConnectionOutputStream",
	"ReadRaw", "ReadString", "ReadVarint32",
	"ReadLittleEndian32", "ReadLittleEndian64", "ReadVarint64",
	"WriteRaw", "WriteString", "WriteLittleEndian32",
	"WriteLittleEndian64", "WriteVarint32", "WriteVarint64"


	)

export("P", "readProtoFiles", "readProtoFiles2", "resetDescriptorPool", "asMessage")

importFrom(utils, .DollarNames)
S3method(.DollarNames, Message)
S3method(.DollarNames, Descriptor)
S3method(.DollarNames, EnumDescriptor)
S3method(.DollarNames, EnumValueDescriptor)
S3method(.DollarNames, FileDescriptor)
S3method(.DollarNames, FieldDescriptor)
S3method(.DollarNames, ServiceDescriptor)
S3method(.DollarNames, MethodDescriptor)
S3method(.DollarNames, ZeroCopyInputStream)
S3method(.DollarNames, ZeroCopyOutputStream)

S3method(with, Message)
S3method(within, Message)

S3method(as.list, Message)
S3method(as.list, Descriptor)
S3method(as.list, EnumDescriptor)
S3method(as.list, FileDescriptor)
S3method(as.list, ServiceDescriptor)

S3method(all.equal, Message)

# constants
exportPattern("^TYPE_")
exportPattern("^CPPTYPE_")

# copied from RProtoBufUtils
export("serialize_pb", "unserialize_pb", "can_serialize_pb")

# export(run_unit_tests)

## RProtoBuf: R Interface to [Google Protocol Buffers](https://protobuf.dev/)

[![CI](https://github.com/eddelbuettel/rprotobuf/workflows/ci/badge.svg)](https://github.com/eddelbuettel/rprotobuf/actions?query=workflow%3Aci)
[![License](https://eddelbuettel.github.io/badges/GPL2+.svg)](https://www.r-project.org/Licenses/GPL-2)
[![CRAN](https://www.r-pkg.org/badges/version/RProtoBuf)](https://cran.r-project.org/package=RProtoBuf)
[![Dependencies](https://tinyverse.netlify.app/badge/RProtoBuf)](https://cran.r-project.org/package=RProtoBuf)
[![Downloads](https://cranlogs.r-pkg.org/badges/RProtoBuf?color=brightgreen)](https://www.r-pkg.org/pkg/RProtoBuf)
[![Last Commit](https://img.shields.io/github/last-commit/eddelbuettel/rprotobuf)](https://github.com/eddelbuettel/rprotobuf)
[![JSS](https://img.shields.io/badge/JSS-10.18637%2Fjss.v071.i02-brightgreen)](https://doi.org/10.18637/jss.v071.i02)

### What are Protocol Buffers?

A serialization format developed by [Google](https://www.google.com) and used
widely within Google, and by numerous other projects.  Quoting from the
[official website](https://protobuf.dev/):

> Protocol buffers are Google's language-neutral, platform-neutral,
extensible mechanism for serializing structured data – think XML, but
smaller, faster, and simpler. You define how you want your data to be
structured once, then you can use special generated source code to easily
write and read your structured data to and from a variety of data streams and
using a variety of languages.

[Protocol Buffers](https://protobuf.dev/) natively
support C++, C#, Go, Java and Python. Many other languages are supported via
contributed extensions. This package adds support for
[R](https://www.r-project.org).

### Installation

You can either install from source via this repo, or install
[the CRAN package](https://cran.r-project.org/package=RProtoBuf)
the usual way from [R](https://www.r-project.org) via

```r
install.packages("RProtoBuf")
```

but do make sure you have the system requirements covered when building from
source.

Under Debian/Ubuntu you may need

```bash
sudo apt-get install protobuf-compiler libprotobuf-dev libprotoc-dev
```

with similar commands on other operating systems or distributions.

### Documentation

The package contains two pdf vignettes; the second one corresponds to
[our JSS paper](https://doi.org/10.18637/jss.v071.i02).

### Authors

Romain Francois, Dirk Eddelbuettel, Murray Stokely and Jeroen Ooms

### License

GPL (>= 2)

## RProtoBuf: R Interface to [Google Protocol Buffers](https://protobuf.dev/)

[![CI](https://github.com/eddelbuettel/rprotobuf/workflows/ci/badge.svg)](https://github.com/eddelbuettel/rprotobuf/actions?query=workflow%3Aci)
[![License](https://eddelbuettel.github.io/badges/GPL2+.svg)](https://www.gnu.org/licenses/gpl-2.0.html)
[![CRAN](https://www.r-pkg.org/badges/version/RProtoBuf)](https://cran.r-project.org/package=RProtoBuf)
[![Dependencies](https://tinyverse.netlify.com/badge/RProtoBuf)](https://cran.r-project.org/package=RProtoBuf)
[![Downloads](https://cranlogs.r-pkg.org/badges/RProtoBuf?color=brightgreen)](https://www.r-pkg.org/pkg/RProtoBuf)
[![Last Commit](https://img.shields.io/github/last-commit/eddelbuettel/rprotobuf)](https://github.com/eddelbuettel/rprotobuf)
[![JSS](https://img.shields.io/badge/JSS-10.18637%2Fjss.v071.i02-brightgreen)](https://doi.org/10.18637/jss.v071.i02)

### What are Protocol Buffers?

A serialization format developed by [Google](https://www.google.com) and used
widely within Google, and by numerous other projects.  Quoting from the
[official website](https://protobuf.dev/):

> Protocol buffers are Google's language-neutral, platform-neutral,
extensible mechanism for serializing structured data – think XML, but
smaller, faster, and simpler. You define how you want your data to be
structured once, then you can use special generated source code to easily
write and read your structured data to and from a variety of data streams and
using a variety of languages.

[Protocol Buffers](https://protobuf.dev/) natively
support C++, C#, Go, Java and Python. Many other languages are supported via
contributed extensions. This package adds support for
[R](https://www.r-project.org).

### Installation

You can either install from source via this repo, or install
[the CRAN package](https://cran.r-project.org/package=RProtoBuf)
the usual way from [R](https://www.r-project.org) via

```r
install.packages("RProtoBuf")
```

but do make sure you have the system requirements covered when building from
source.

Under Debian/Ubuntu you may need

```bash
sudo apt-get install protobuf-compiler libprotobuf-dev libprotoc-dev
```

with similar commands on other operating systems or distributions.

### Documentation

The package contains two pdf vignettes; the second one corresponds to
[our JSS paper](https://doi.org/10.18637/jss.v071.i02).

### Authors

Romain Francois, Dirk Eddelbuettel, Murray Stokely and Jeroen Ooms

### License

GPL (>= 2)

Quoting from http://en.wikipedia.org/wiki/Protocol_Buffers

  Protocol Buffers is a serialization format with an interface
  description language developed by Google. The original Google implementation
  for C++, Java and Python is available under a free software, open source
  license. Various other language implementations are either available or in
  development. 
  .
  The design goals for Protocol Buffers emphasized simplicity and
  performance. In particular, it was designed to be faster than XML (no
  reproducible comparisons are publicly available to confirm this,
  however). Protocol Buffers is very similar to Facebook’s Thrift protocol,
  except it does not include a concrete RPC stack to use for defined
  services. Since Protocol Buffers was open sourced, a number of RPC stacks
  have emerged to fill this gap. 
  .
  Prior to the release as open source, Protocol Buffers had been widely used
  at Google for storing and interchanging all kinds of structured
  information. Protocol Buffers serve as a basis for a custom RPC system that
  is used for practically all inter-machine communication at Google.[1] 
  .
  Data structures and services are defined in the Proto Definition file
  (.proto) which is then compiled with protoc. This compilation generates code
  that matches the services. For example, example.proto will produce
  example.pb.cc and example.pb.h which will define C++ classes for each
  Message and Service example.proto defines. 
  .
  Protocol Buffers allow serialization into any number of formats. In the
  officially supported implementations there is a full Reflection interface
  available, making it easy to serialize protos as XML and JSON. 
  .
  Though the primary purpose of Protocol Buffers is to facilitate network
  communication, its simplicity and speed make Protocol Buffers a great
  replacement of data-centric C++ classes and structs. 

#simple R script to serialize some R objects
library(RProtoBuf)

#some vectors
serialize_pb(c(1,2,pi, NA,NaN,Inf,-Inf), "double.msg")
serialize_pb(c(1L, 2L, NA), "integer.msg")
serialize_pb(c(TRUE, FALSE, NA), "logical.msg")
serialize_pb(c("foo", "bar", NA), "character.msg")
serialize_pb(charToRaw("This is a test"), "raw.msg")

#lists
serialize_pb(list(foo=c(1,2,pi), bar=TRUE, baz="blabla", zoo=NULL), "list.msg")
serialize_pb(iris[1:3,], "dataframe.msg")

### R code from vignette source 'RProtoBuf-unitTests.Rnw'

###################################################
### code chunk number 1: RProtoBuf-unitTests.Rnw:9-13
###################################################
require(RProtoBuf)
prettyVersion <- packageDescription("RProtoBuf")$Version
prettyDate <- format(Sys.Date(), "%B %e, %Y")
library(RUnit)


###################################################
### code chunk number 2: unitTesting
###################################################
pkg <- "RProtoBuf"

if (file.exists("unitTests-results")) unlink("unitTests-results", recursive = TRUE)
dir.create("unitTests-results")
pathRcppTests <<- system.file("unitTests", package = pkg)
path <- system.file("unitTests", package=pkg)
testSuite <- defineTestSuite(name=paste(pkg, "unit testing"), dirs=path)
tests <- runTestSuite(testSuite)
err <- getErrors(tests)
if (err$nFail > 0) cat(sprintf("unit test problems: %d failures", err$nFail))
if (err$nErr > 0) cat(sprintf("unit test problems: %d errors", err$nErr))
printHTMLProtocol(tests, fileName=sprintf("unitTests-results/%s-unitTests.html", pkg))
printTextProtocol(tests, fileName=sprintf("unitTests-results/%s-unitTests.txt" , pkg))

#if (file.exists("/tmp")) {
#    invisible(sapply(c("txt", "html"), function(ext) {
#        fname <- sprintf("unitTests-results/%s-unitTests.%s", pkg, ext)
#        file.copy(fname, "/tmp", overwrite=TRUE)
#    }))
#}


###################################################
### code chunk number 3: importResults
###################################################
results <- sprintf("unitTests-results/%s-unitTests.txt", pkg)
if (file.exists(results)) {
    writeLines(readLines(results))
} else{
    writeLines("Unit test results not available")
}

require( RProtoBuf )

### read messages from files and binary connections

# read a message from a file name
f <- system.file( "examples", "addressbook.pb", package = "RProtoBuf" )
m1 <- read( tutorial.AddressBook, f )
writeLines( as.character( m1 ) )

# read the same message from a (binary) file connection
con <- file( f, open = "rb" )
m2 <- read( tutorial.AddressBook, con )
# this also works:
# m2 <- tutorial.AddressBook$read( con )
writeLines( as.character( m2 ) )
close( con )

# TODO: we need another example from a more fancy connection (pipe, ...)

stopifnot( identical( as.character(m1), as.character(m2) ) )

### write messages to files and binary connections

tf1 <- tempfile()
serialize( m1, tf1 )

tf2 <- tempfile()
con <- file( tf2, open = "wb" )
serialize( m2, con )
close( con )

# test that the two binary files are identical
stopifnot( identical(
	readBin( tf1, raw(0), 1000), readBin( tf2, raw(0), 1000)
	) )

# load the package
require( RProtoBuf )

# read all the proto files in the proto directory
# of the package "RProtoBuf"
# (this is also done in the startup of the package, so this call is 
#  actually redundant)
readProtoFiles( package = "RProtoBuf" ) 


# creating three Person messages

# create a prototype with a simple call to new on the 
# descriptor for the Person message type, and then update the message
# with fields
romain <- update( new( tutorial.Person ), 
	email = "romain@r-enthusiasts.com", 
	id = 1, 
	name = "Romain Francois", 
	phone = new( tutorial.Person.PhoneNumber , number = "+33(0)...", type = "MOBILE" )
	)

# directly supply parameters to the constructor
dirk <- new( tutorial.Person, 
	email = "edd@debian.org", 
	id = 2, 
	name = "Dirk Eddelbuettel" ) 

# update the phone repeated field with a list of PhoneNumber messages
dirk$phone <- list( 
	new( tutorial.Person.PhoneNumber , number = "+01...", type = "MOBILE" ), 
	new( tutorial.Person.PhoneNumber , number = "+01...", type = "HOME" ) )

# build the address book
book <- new( tutorial.AddressBook, person = list( romain, dirk ) )

# debug string. This is not what is transferred in the wire
writeLines( as.character( book ) )

# the serialized message
# see also the io demo to see how to serialize messages to connections 
# and files
serialize( book, NULL )

# number of bytes taken by the message on the wire
bytesize( book )
length( serialize( book, NULL ) )

# swap two persons in the address book
# (1-based indexing)
book$swap( "person", 1L , 2L )
writeLines( as.character( book ) )

# number of "person" in the book
book$size( "person" )

# clear the message
book$clear( )
writeLines( as.character( book ) )

# Jeroen Ooms
#
# HTTPS+ProtoBuf RPC POC using OpenCPU
# Script below downloads MASS::Animals using protobuf
library(RProtoBuf)
library(httr)

req <- GET ('https://public.opencpu.org/ocpu/library/MASS/data/Animals/pb')
output <- unserialize_pb(req$content)
identical(output, MASS::Animals)

# Jeroen Ooms
#
# HTTPS+ProtoBuf RPC POC using OpenCPU
# The call below maps to: do.call(stats::rnorm, list(n=42, mean=100))

# !! This requires httr (>= 0.2.99). Version 0.2 has a bug.
# library(devtools)
# install_github("httr")

# Actual code
library(RProtoBuf)
library(httr)

args <- list(n=42, mean=100)
payload <- serialize_pb(args, NULL)

req <- POST (
  url = "https://public.opencpu.org/ocpu/library/stats/R/rnorm/pb",
  body = payload,
  add_headers(
  	"Content-Type" = "application/x-protobuf"
  )
)

#This is the output of stats::rnorm(n=42, mean=100)
output <- unserialize_pb(req$content)
print(length(output))

setGeneric( "method_count", function(object ){
	standardGeneric( "method_count" )
} )
setMethod( "method_count", "ServiceDescriptor", function(object){
	.Call("ServiceDescriptor_method_count", object@pointer)
} )


setGeneric( "method", function(object, index, name ){
	standardGeneric( "method" )
} )
setMethod( "method", "ServiceDescriptor", function(object, index, name){

	has_index  <- !missing(index)
	has_name   <- !missing(name)

	if( as.integer(has_index) + as.integer(has_name) != 1L ){
		stop( "need exactly one of `index` or `name`" )
	}

	if( has_index ){
		stop( "No routine for getting a method from a ServiceDescriptor by index." )
	}

	if( has_name ){
		stop( "No routine for getting a method from a ServiceDescriptor by name." )
	}

} )

setGeneric( "read", function( descriptor, input ){
	standardGeneric( "read" )
} )

setMethod( "read", c( descriptor = "Descriptor" , input = "character" ),
function(descriptor, input ){
	file <- file_path_as_absolute( input )
	.Call(Descriptor__readMessageFromFile, descriptor@pointer, file)
} )

setMethod( "read", c( descriptor = "Descriptor", input = "raw" ), function(descriptor, input ){
	.Call(Descriptor__readMessageFromRawVector, descriptor@pointer, input)
} )

setMethod( "read", c( descriptor = "Descriptor" ),
function( descriptor, input ){
	if( !inherits( input, "connection" ) ){
		stop( "can only read from connections" )
	}
	wasopen <- identical( summary(input)[["opened"]], "opened" )
	if( !wasopen ) open( input, "rb")
        stopifnot(summary(input)[["text"]] == "binary")
	message <- .Call(Descriptor__readMessageFromConnection, descriptor@pointer, input)
	if( !wasopen ) close( input )
	message
} )




setGeneric( "readASCII", function( descriptor, input, partial=FALSE ){
	standardGeneric( "readASCII" )
} )

setMethod( "readASCII", c( descriptor = "Descriptor" , input = "character" ),
function(descriptor, input, partial=FALSE){
        stopifnot(is.logical(partial), length(partial) == 1)
	.Call(Descriptor__readASCIIFromString, descriptor@pointer, input, partial)
} )

setMethod( "readASCII", c( descriptor = "Descriptor" ),
function( descriptor, input, partial=FALSE){
        stopifnot(is.logical(partial), length(partial) == 1)
	if( !inherits( input, "connection" ) ){
		stop( "can only read from connections" )
	}
	wasopen <- identical( summary(input)[["opened"]], "opened" )
	if( !wasopen ) open( input, "rb" )
        stopifnot(summary(input)[["text"]] == "binary")
	message <- .Call(Descriptor__readASCIIFromConnection, descriptor@pointer, input, partial)
    close(input)
	message
} )




setGeneric( "readJSON", function( descriptor, input ){
	standardGeneric( "readJSON" )
} )

setMethod( "readJSON", c( descriptor = "Descriptor" , input = "character" ),
function(descriptor, input){
	.Call(Descriptor__readJSONFromString, descriptor@pointer, input)
} )

setMethod( "readJSON", c( descriptor = "Descriptor" ),
function( descriptor, input){
	if( !inherits( input, "connection" ) ){
		stop( "can only read from connections" )
	}
	wasopen <- identical( summary(input)[["opened"]], "opened" )
	if( !wasopen ) open( input, "rb" )
        stopifnot(summary(input)[["text"]] == "binary")
	message <- .Call(Descriptor__readJSONFromConnection, descriptor@pointer, input)
	close(input)
	message
} )

# attaches the descriptor pool to the search path at the given
# position

OTABLE <- NULL
NAMESPACE <- environment()

attachDescriptorPool <- function(pos = 2) {
    if (is.null(OTABLE)) {
        ##unlockBinding( "OTABLE", NAMESPACE )
        otable <- .Call(newProtocolBufferLookup, pos)
        #attach(otable, pos=pos, name="RProtoBuf:DescriptorPool")
        assign("OTABLE", otable, envir = NAMESPACE, inherits = FALSE)
        lockBinding("OTABLE", NAMESPACE )
    }

}

#' Serialize R object to Protocol Buffer Message.
#'
#' Serializes R objects to a general purpose protobuf message using the same
#' \code{rexp.proto} descriptor and mapping between R objects and protobuf
#' mesages as RHIPE.
#'
#' Clients need both the message and the \code{rexp.proto} descriptor to parse
#' serialized R objects. The latter is included in the the package installation
#' \code{proto} directory: \code{system.file(package="RProtoBuf", "proto/rexp.proto")}
#'
#' The following storage types are natively supported by the descriptor:
#' \code{character}, \code{raw}, \code{double}, \code{complex}, \code{integer},
#' \code{list}, and \code{NULL}. Objects with other storage types, such as
#' functions, environments, S4 classes, etc, are serialized using base R
#' \code{\link{serialize}} and stored in the proto \code{native} type.
#' Missing values, attributes and numeric precision will be preserved.
#'
#' @param object R object to serialize
#' @param connection passed on to  \code{\link{serialize}}
#' @param ... additional arguments passed on to  \code{\link{serialize}}
#' @aliases unserialize_pb can_serialize_pb
#' @export unserialize_pb
#' @export can_serialize_pb
#' @export
#' @examples msg <- tempfile();
#' serialize_pb(iris, msg);
#' obj <- unserialize_pb(msg);
#' identical(iris, obj);
#'
serialize_pb <- function(object, connection, ...){

  #convert object to protobuf message
  msg <- rexp_obj(object);

  #serialize the message
  serialize(msg, connection = connection, ...);
}

unserialize_pb <- function(connection){

  #convert object to protobuf message
  unrexp(read(pb(rexp.REXP), connection));
}

## Jeroen Ooms and Dirk Eddelbuettel

library(RProtoBuf)

#test.serialize_pb <- function() {
##verify that rexp.proto is loaded
RProtoBuf:::pb(rexp.REXP)

##serialize a nested list
x <- list(foo=cars, bar=Titanic)
expect_equal(unserialize_pb(serialize_pb(x, NULL)), x)

##a bit of everything, copied from jsonlite package
set.seed('123')
myobject <- list(
    mynull = NULL,
    mycomplex = lapply(eigen(matrix(-rnorm(9),3)), round, 3),
    mymatrix = round(matrix(rnorm(9), 3),3),
    myint = as.integer(c(1,2,3)),
    mydf = cars,
    mylist = list(foo='bar', 123, NA, NULL, list('test')),
    mylogical = c(TRUE,FALSE,NA),
    mychar = c('foo', NA, 'bar'),
    myemptychar = character(0),
    somemissings = c(1,2,NA,NaN,5, Inf, 7 -Inf, 9, NA),
    myrawvec = charToRaw('This is a test'),
    myS4 = asS4("test")
)

expect_equal(unserialize_pb(serialize_pb(myobject, NULL)), myobject)
#}

#test.serialize_pb.alldatasets <- function() {
datasets <- as.data.frame(data(package="datasets")$results)
datasets$name <- sub("\\s+.*$", "", datasets$Item)

encoded.datasets <- sapply(datasets$name,
                           function(x) serialize_pb(get(x), NULL))

unserialized.datasets <- sapply(encoded.datasets, unserialize_pb)

expect_true(all(sapply(names(unserialized.datasets),
                     function(name) identical(get(name),
                                              unserialized.datasets[[name]]))))
#}


## this was issue #37 reported and fixed by Jeffrey Shen
#test.serialize.sublist <- function() {
x <- packageVersion("RProtoBuf")
expect_identical(x, unserialize_pb(serialize_pb(x, NULL)), info="checking sublists")
#}

t#est.serialize.posixlt <- function() {
x <- as.POSIXlt("1970-01-01T00:00:00Z")
expect_identical(x, unserialize_pb(serialize_pb(x, NULL)), info="checking posixlt")
#}

setGeneric( "identical" )
setMethod( "identical", c(x="Message", y = "Message" ), function( x, y, num.eq = TRUE, single.NA = TRUE, attrib.as.set = TRUE){
	.Call(identical_messages, x@pointer, y@pointer)
} )

setMethod( "==", c( e1 = "Message", e2 = "Message" ), function(e1, e2 ){
	.Call(identical_messages, e1@pointer, e2@pointer)
} )
setMethod( "!=", c( e1 = "Message", e2 = "Message" ), function(e1, e2 ){
	! .Call(identical_messages, e1@pointer, e2@pointer)
} )

setGeneric( "all.equal" )
setMethod( "all.equal", c( target = "Message", current = "Message" ),
	function(target, current, tolerance = .Machine$double.eps^0.5, ...){
        all.equal.Message(target, current, tolerance, ...)
} )

all.equal.Message <- function(target, current, tolerance = .Machine$double.eps^0.5, ...){
	.Call(all_equal_messages, target@pointer, current@pointer, tolerance)
}

setGeneric( "descriptor", function(object, ...){
	standardGeneric( "descriptor" )
} )
setMethod( "descriptor", "Message", function(object, ...){
	.Call(Message__descriptor, object@pointer)
} )

setGeneric( "fileDescriptor", function(object, ...){
	standardGeneric( "fileDescriptor" )
} )
setMethod( "fileDescriptor", "Message", function(object, ...){
	.Call(Message__fileDescriptor, object@pointer)
} )
setMethod( "fileDescriptor", "Descriptor", function(object, ...){
	.Call(Descriptor__fileDescriptor, object@pointer)
} )
setMethod( "fileDescriptor", "EnumDescriptor", function(object, ...){
	.Call(EnumDescriptor__fileDescriptor, object@pointer)
} )
setMethod( "fileDescriptor", "FieldDescriptor", function(object, ...){
	.Call(FieldDescriptor__fileDescriptor, object@pointer)
} )
setMethod( "fileDescriptor", "ServiceDescriptor", function(object, ...){
	.Call(ServiceDescriptor__fileDescriptor, object@pointer)
} )
setMethod( "fileDescriptor", "MethodDescriptor", function(object, ...){
	.Call(MethodDescriptor__fileDescriptor, object@pointer)
} )

.onLoad <- function(libname, pkgname ){
    ##minversion <- packageDescription(pkgname, lib.loc=libname)$MinimumLibProtoVersion
    ##minversion <- as.integer( gsub( "[[:space:]]+", "", minversion ) )
    ##.Call(check_libprotobuf_version, minversion)
    readProtoFiles2( protoPath=system.file( "proto", package=pkgname, lib.loc=libname ) )
    attachDescriptorPool( pos = length(search()) )
    options("RProtoBuf.int64AsString" = FALSE)
    #if( exists( ".httpd.handlers.env", asNamespace( "tools" ) ) ){
    #    e <- tools:::.httpd.handlers.env
    #    e[["RProtoBuf"]] <- RProtoBuf.http.handler
    #}
    .RProtoBuf_libname <<- libname
}

.onAttach <- function(libname, pkgName) {
    if (interactive()) {
        packageStartupMessage("RProtoBuf ", packageVersion("RProtoBuf"),
                              " using Protocol Buffers library version ",
                              format(getProtobufLibVersion(TRUE)), ".")
    }
}

setGeneric( "isInitialized", function( object, ... ){
	standardGeneric( "isInitialized" )
} )
setMethod( "isInitialized", "Message", function(object, ... ){
	.Call( Message__is_initialized, object@pointer )
} )

# S3 dispatch does not work for some reason

generateActiveBindings <- function(data){
	env <- new.env( parent = environment() )
	xp <- data@pointer
	names <- .Call(Message__fieldNames, xp)

	if( !is.null(names) && length(names) ){
		lapply( names, function(x ){
  		  makeActiveBinding( x, function(v){
  		    if( missing(v) ){
  		    	# get
  		      .Call(getMessageField, xp, x)
  		    } else {
  		    	# set
  		      .Call(setMessageField, xp, x, v)
  		    }
  		  }, env )
  		} )
  	}
  	env
}

with.Message <- function( data, expr, ...){
	envir <- generateActiveBindings(data)
  	eval( substitute( expr ), envir = envir )
}

within.Message <- function(data, expr, ... ){
	envir <- generateActiveBindings(data)
  	eval( substitute( expr ), envir = envir )
  	data
}

# :tabSize=4:indentSize=4:noTabs=false:folding=explicit:collapseFolds=1:

# {{{ generics
setGeneric( "flush" )
setGeneric( "close" )
setGeneric( "GetErrno", function(object){
	standardGeneric( "GetErrno" )
} )
setGeneric( "SetCloseOnDelete", function(object, close=FALSE){
	standardGeneric( "SetCloseOnDelete" )
} )
# }}}

# {{{ methods
setGeneric( "Next", function(object, payload){
	standardGeneric( "Next" )
} )
setGeneric( "BackUp", function(object, count){
	standardGeneric( "BackUp" )
} )
setGeneric( "ByteCount", function(object){
	standardGeneric( "ByteCount" )
} )
setGeneric( "Skip", function(object, count){
	standardGeneric( "Skip" )
} )
# }}}

# {{{ zero input stream
setMethod( "Next", c( object = "ZeroCopyInputStream", payload = "missing"), function(object, payload){
	.Call(ZeroCopyInputStream_Next, object@pointer)
} )

setMethod( "BackUp", "ZeroCopyInputStream", function(object, count){
	invisible(.Call(ZeroCopyInputStream_BackUp, object@pointer, count))
} )

setMethod( "ByteCount", "ZeroCopyInputStream", function(object){
	.Call(ZeroCopyInputStream_ByteCount, object@pointer)
} )

setMethod( "Skip", "ZeroCopyInputStream", function(object, count){
	invisible( .Call(ZeroCopyInputStream_Skip, object@pointer, count) )
} )
# }}}

# {{{ zero copy output stream
setMethod( "Next", c( object = "ZeroCopyOutputStream", payload = "raw" ), function(object, payload){
	.Call(ZeroCopyOutputStream_Next, object@pointer, payload)
} )

setMethod( "BackUp", "ZeroCopyOutputStream", function(object, count){
	invisible( .Call(ZeroCopyOutputStream_BackUp, object@pointer, count) )
} )

setMethod( "ByteCount", "ZeroCopyOutputStream", function(object){
	.Call(ZeroCopyOutputStream_ByteCount, object@pointer)
} )
# }}}

# {{{ ArrayInputStream
setGeneric( "ArrayInputStream", function(payload, block_size){
	standardGeneric( "ArrayInputStream" )
} )
setMethod( "ArrayInputStream", c( payload = "raw", block_size = "missing" ) ,
function(payload, block_size){
	.Call(ArrayInputStream__new, payload, -1L)
} )
setMethod( "ArrayInputStream", c( payload = "raw", block_size = "integer" ) ,
function(payload, block_size){
	.Call(ArrayInputStream__new, payload, block_size)
} )
setMethod( "ArrayInputStream", c( payload = "raw", block_size = "numeric" ) ,
function(payload, block_size){
	.Call(ArrayInputStream__new, payload, as.integer(block_size))
} )
# }}}

# {{{ ArrayOutputStream constructor function
setGeneric( "ArrayOutputStream", function(size, block_size){
	standardGeneric( "ArrayOutputStream" )
} )
setMethod( "ArrayOutputStream", signature( size = "integer", block_size = "missing" ), function(size, block_size){
	.Call(ArrayOutputStream__new, size, -1L)
} )
setMethod( "ArrayOutputStream", signature( size = "integer", block_size = "integer" ), function(size, block_size){
	.Call(ArrayOutputStream__new, size, block_size)
} )
setMethod( "ArrayOutputStream", signature( size = "integer", block_size = "numeric" ), function(size, block_size){
	.Call(ArrayOutputStream__new, size, as.integer(block_size) )
} )
setMethod( "ArrayOutputStream", signature( size = "numeric", block_size = "missing" ), function(size, block_size){
	.Call(ArrayOutputStream__new, as.integer(size), -1L)
} )
setMethod( "ArrayOutputStream", signature( size = "numeric", block_size = "integer" ),function(size, block_size){
	.Call(ArrayOutputStream__new, as.integer(size), block_size)
} )
setMethod( "ArrayOutputStream", signature( size = "numeric", block_size = "numeric" ), function(size, block_size){
	.Call(ArrayOutputStream__new, as.integer(size), as.integer(block_size) )
} )
# }}}

# {{{ FileInputStream
setGeneric( "FileInputStream", function(filename, block_size = -1L, close.on.delete = FALSE ){
	standardGeneric( "FileInputStream" )
} )
setMethod( "FileInputStream", signature( filename = "character", block_size = "integer", close.on.delete = "logical" ), function(filename, block_size = -1L, close.on.delete = FALSE){
	full_filename <- file_path_as_absolute(filename)
	.Call(FileInputStream_new, filename, block_size, close.on.delete)
} )
setMethod( "close", "FileInputStream", function(con, ...){
	.Call(FileInputStream_Close, con@pointer)
} )
setMethod( "GetErrno", "FileInputStream", function(object){
	.Call(FileInputStream_GetErrno, object@pointer)
} )
setMethod( "SetCloseOnDelete", "FileInputStream", function(object, close=FALSE){
	invisible( .Call(FileInputStream_SetCloseOnDelete, object@pointer, isTRUE(close)) )
} )
# }}}

# {{{ FileOutputStream
setGeneric( "FileOutputStream", function(filename, block_size = -1L, close.on.delete = FALSE ){
	standardGeneric( "FileOutputStream" )
} )
setMethod( "FileOutputStream", signature( filename = "character", block_size = "integer", close.on.delete = "logical" ), function(filename, block_size = -1L, close.on.delete = FALSE){
	if( !file.exists( filename ) ){
		if( !file.exists( dirname(filename) ) ){
			stop( "directory does not exist" )
		}
		filename <- normalizePath(filename, mustWork=FALSE)
	} else{
		filename <- file_path_as_absolute(filename)
	}

	.Call(FileOutputStream_new, filename, block_size, close.on.delete)
} )
setMethod( "flush", "FileOutputStream", function(con){
	.Call(FileOutputStream_Flush, con@pointer)
} )
setMethod( "close", "FileOutputStream", function(con, ...){
	.Call(FileOutputStream_Close, con@pointer)
} )
setMethod( "GetErrno", "FileOutputStream", function(object){
	.Call(FileOutputStream_GetErrno, object@pointer)
} )
setMethod( "SetCloseOnDelete", "FileOutputStream", function(object, close=FALSE){
	invisible( .Call(FileOutputStream_SetCloseOnDelete, object@pointer, isTRUE(close)) )
} )
# }}}

# {{{ ConnectionInputStream
setGeneric( "ConnectionInputStream", function(object, ...){
	standardGeneric( "ConnectionInputStream" )
} )
setMethod( "ConnectionInputStream", "connection", function(object, ...){
	sc <- summary( object )
	if( ! identical( sc[["can read"]], "yes") ){
		stop( "cannot read from connection" )
	}
	if( ! identical( sc[["text"]], "binary" ) ){
		stop( "not a binary connection" )
	}
	was_open <- isOpen( object )
	if( ! was_open ){
		open( object )
	}
	.Call(ConnectionInputStream_new, object, was_open)
} )
# }}}

# {{{ ConnectionOutputStream
setGeneric( "ConnectionOutputStream", function(object, ...){
	standardGeneric( "ConnectionOutputStream" )
} )
setMethod( "ConnectionOutputStream", "connection", function(object, ...){
	sc <- summary( object )
	if( ! identical( sc[["can write"]], "yes") ){
		stop( "cannot write to connection" )
	}
	if( ! identical( sc[["text"]], "binary" ) ){
		stop( "not a binary connection" )
	}
	was_open <- isOpen( object )
	if( ! was_open ){
		open( object )
	}
	.Call(ConnectionOutputStream_new, object, was_open)
} )
# }}}

setGeneric( "input_type", function(object){
	standardGeneric( "input_type" )
} )
setGeneric( "output_type", function(object){
	standardGeneric( "output_type" )
} )

setMethod( "input_type", "MethodDescriptor", function(object){
	.Call(MethodDescriptor__input_type, object@pointer )
} )

setMethod( "output_type", "MethodDescriptor", function(object){
	.Call(MethodDescriptor__output_type, object@pointer )
} )

setMethod( "enum_type", c( object = "EnumValueDescriptor", index = "missing", name = "missing"), function(object, index, name){
	.Call(EnumValueDescriptor__enum_type, object@pointer)
} )

setGeneric( "number", function(object){
	standardGeneric( "number" )
} )
setMethod( "number", "EnumValueDescriptor", function(object){
	.Call(EnumValueDescriptor__number, object@pointer)
} )

setGeneric( "clear", function(object, field, ...){
	standardGeneric( "clear" )
} )
setMethod( "clear", signature( "Message", "missing" ), function(object, field, ...){
	.Call( Message__clear, object@pointer )
	invisible( object )
} )

setMethod( "clear", signature( "Message", "character" ), function(object, field, ...){
	.Call( Message__clear_field, object@pointer, field )
	invisible( object )
} )

setMethod( "clear", signature( "Message", "integer" ), function(object, field, ...){
	.Call( Message__clear_field, object@pointer, field )
	invisible( object )
} )

setMethod( "clear", signature( "Message", "numeric" ), function(object, field, ...){
	.Call( Message__clear_field, object@pointer, as.integer(field) )
	invisible( object )
} )

setMethod( "clear", signature( "Message", "raw" ), function(object, field, ...){
	.Call( Message__clear_field, object@pointer, as.integer(field) )
	invisible( object )
} )

setGeneric( "WriteRaw", function(object, payload){
	standardGeneric("WriteRaw")
} )
setMethod( "WriteRaw", c(object="ZeroCopyOutputStream", payload = "raw" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteRaw, object@pointer, payload)
} )

setGeneric( "WriteString", function(object, payload){
	standardGeneric("WriteString")
} )
setMethod( "WriteString", c(object="ZeroCopyOutputStream", payload = "character" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteString, object@pointer, payload)
} )

setGeneric( "WriteLittleEndian32", function(object, payload){
	standardGeneric("WriteLittleEndian32")
} )
setMethod( "WriteLittleEndian32", c(object="ZeroCopyOutputStream", payload = "integer" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteLittleEndian32, object@pointer, payload)
} )
setMethod( "WriteLittleEndian32", c(object="ZeroCopyOutputStream", payload = "numeric" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteLittleEndian32, object@pointer, payload)
} )
setMethod( "WriteLittleEndian32", c(object="ZeroCopyOutputStream", payload = "raw" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteLittleEndian32, object@pointer, payload)
} )

setGeneric( "WriteLittleEndian64", function(object, payload){
	standardGeneric("WriteLittleEndian64")
} )
setMethod( "WriteLittleEndian64", c(object="ZeroCopyOutputStream", payload = "integer" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteLittleEndian64, object@pointer, payload)
} )
setMethod( "WriteLittleEndian64", c(object="ZeroCopyOutputStream", payload = "numeric" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteLittleEndian64, object@pointer, payload)
} )
setMethod( "WriteLittleEndian64", c(object="ZeroCopyOutputStream", payload = "raw" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteLittleEndian64, object@pointer, payload)
} )


setGeneric( "WriteVarint32", function(object, payload){
	standardGeneric("WriteVarint32")
} )
setMethod( "WriteVarint32", c(object="ZeroCopyOutputStream", payload = "integer" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteVarint32, object@pointer, payload)
} )
setMethod( "WriteVarint32", c(object="ZeroCopyOutputStream", payload = "numeric" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteVarint32, object@pointer, payload)
} )
setMethod( "WriteVarint32", c(object="ZeroCopyOutputStream", payload = "raw" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteVarint32, object@pointer, payload)
} )


setGeneric( "WriteVarint64", function(object, payload){
	standardGeneric("WriteVarint64")
} )
setMethod( "WriteVarint64", c(object="ZeroCopyOutputStream", payload = "integer" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteVarint64, object@pointer, payload)
} )
setMethod( "WriteVarint64", c(object="ZeroCopyOutputStream", payload = "numeric" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteVarint64, object@pointer, payload)
} )
setMethod( "WriteVarint64", c(object="ZeroCopyOutputStream", payload = "raw" ), function(object, payload){
	.Call(ZeroCopyOutputStream_WriteVarint64, object@pointer, payload)
} )

setGeneric( "is_extension", function(object){
	standardGeneric("is_extension")
} )
setMethod( "is_extension", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__is_extension, object@pointer)
})

setMethod( "number", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__number, object@pointer)
} )

TYPE_DOUBLE   <- 1L
TYPE_FLOAT    <- 2L
TYPE_INT64    <- 3L
TYPE_UINT64   <- 4L
TYPE_INT32    <- 5L
TYPE_FIXED64  <- 6L
TYPE_FIXED32  <- 7L
TYPE_BOOL     <- 8L
TYPE_STRING   <- 9L
TYPE_GROUP    <- 10L
TYPE_MESSAGE  <- 11L
TYPE_BYTES    <- 12L
TYPE_UINT32   <- 13L
TYPE_ENUM     <- 14L
TYPE_SFIXED32 <- 15L
TYPE_SFIXED64 <- 16L
TYPE_SINT32   <- 17L
TYPE_SINT64   <- 18L

.TYPES <- sapply(ls( pattern="^TYPE_" ), function(x) get(x))

setGeneric( "type", function(object, as.string = FALSE){
	standardGeneric( "type" )
} )
setMethod( "type", "FieldDescriptor", function(object, as.string = FALSE){
	type <- .Call(FieldDescriptor__type, object@pointer)
	if( as.string ) {
                names(which(.TYPES == type))
        } else {
                type
        }
} )


CPPTYPE_INT32 <- 1L
CPPTYPE_INT64 <- 2L
CPPTYPE_UINT32 <- 3L
CPPTYPE_UINT64 <- 4L
CPPTYPE_DOUBLE <- 5L
CPPTYPE_FLOAT <- 6L
CPPTYPE_BOOL <- 7L
CPPTYPE_ENUM <- 8L
CPPTYPE_STRING <- 9L
CPPTYPE_MESSAGE <- 10L

.CPPTYPES <- sapply(ls( pattern="^CPPTYPE_" ), function(x) get(x))
setGeneric( "cpp_type", function(object, as.string = FALSE ){
	standardGeneric( "cpp_type" )
} )
setMethod( "cpp_type", "FieldDescriptor", function(object, as.string = FALSE){
	cpptype <- .Call(FieldDescriptor__cpp_type, object@pointer)
	if( as.string ) {
                names(which(.CPPTYPES == cpptype))
        } else {
                cpptype
        }
} )

LABEL_OPTIONAL <- 1L
LABEL_REQUIRED <- 2L
LABEL_REPEATED <- 3L
.LABELS <- sapply(ls( pattern="^LABEL_" ), function(x) get(x))

setGeneric( "label", function(object, as.string = FALSE ){
	standardGeneric( "label" )
} )
setMethod( "label", "FieldDescriptor", function(object, as.string = FALSE){
	lab <- .Call(FieldDescriptor__label, object@pointer)
	if( as.string ) {
                names(which(.LABELS == lab))
        } else {
                lab
        }
} )

setGeneric( "is_repeated", function(object ){
	standardGeneric( "is_repeated" )
} )
setMethod( "is_repeated", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__is_repeated, object@pointer)
} )

setGeneric( "is_optional", function(object){
	standardGeneric( "is_optional" )
} )
setMethod( "is_optional", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__is_optional, object@pointer)
} )

setGeneric( "is_required", function(object ){
	standardGeneric( "is_required" )
} )
setMethod( "is_required", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__is_required, object@pointer)
} )

setGeneric( "has_default_value", function(object ){
	standardGeneric( "has_default_value" )
} )
setMethod( "has_default_value", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__has_default_value, object@pointer)
} )

setGeneric( "default_value", function(object ){
	standardGeneric( "default_value" )
} )
setMethod( "default_value", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__default_value, object@pointer)
} )


setGeneric( "message_type", function(object ){
	standardGeneric( "message_type" )
} )
setMethod( "message_type", "FieldDescriptor", function(object){
	.Call(FieldDescriptor__message_type, object@pointer)
} )

setMethod( "enum_type", c( object = "FieldDescriptor", index = "missing", name = "missing"), function(object){
	.Call(FieldDescriptor__enum_type, object@pointer)
} )

setGeneric( "add", function(object, field, values){
	standardGeneric( "add" )
} )
setMethod( "add", "Message", function( object, field, values ){

	if( is( values, "Message" ) ){
		values <- list( values )
	}

	.Call( Message__add_values, object@pointer, field, values )
	invisible( object )
} )

setGeneric( "merge" )
setMethod( "merge",
	c( x = "Message", y = "Message" ),
	function( x, y , ... ){
		if( !identical( x@type, y@type ) ){
                        stop(sprintf("incompatible message types, cannot merge '%s' and '%s'", x@type, y@type))
		}

		message <- .Call( Message__merge, x@pointer, y@pointer)
		message
} )

._toString_Message <- function(x, debug = getOption("RProtoBuf.toString.debug", TRUE), ...){
	if (isTRUE(debug)) {
		.Call( Message__as_character, x@pointer )
	} else {
		.Call( Message__print_text_format, x@pointer)
	}
}
._toString_Descriptor <- function(x, ...){
	.Call( Descriptor__as_character, x@pointer )
}
._toString_EnumDescriptor <- function(x, ...){
	.Call( EnumDescriptor__as_character, x@pointer )
}
._toString_FieldDescriptor <- function(x, ...){
	.Call( FieldDescriptor__as_character, x@pointer )
}
._toString_ServiceDescriptor <- function(x, ...){
	.Call( ServiceDescriptor__as_character, x@pointer )
}
._toString_MethodDescriptor <- function(x, ...){
	.Call( MethodDescriptor__as_character, x@pointer )
}
._toString_FileDescriptor <- function(x, ...){
	.Call( FileDescriptor__as_character, x@pointer )
}
._toString_EnumValueDescriptor <- function(x, ...){
	.Call( EnumValueDescriptor__as_character, x@pointer )
}


setMethod( "as.character", "Message", ._toString_Message  )
setMethod( "as.character", "Descriptor", ._toString_Descriptor  )
setMethod( "as.character", "EnumDescriptor", ._toString_EnumDescriptor )
setMethod( "as.character", "FieldDescriptor", ._toString_FieldDescriptor )
setMethod( "as.character", "ServiceDescriptor", ._toString_ServiceDescriptor )
setMethod( "as.character", "MethodDescriptor", ._toString_MethodDescriptor )
setMethod( "as.character", "FileDescriptor", ._toString_FileDescriptor )
setMethod( "as.character", "EnumValueDescriptor", ._toString_EnumValueDescriptor )

setGeneric( "toString" )
setMethod( "toString", "Message", ._toString_Message  )
setMethod( "toString", "Descriptor", ._toString_Descriptor  )
setMethod( "toString", "EnumDescriptor", ._toString_EnumDescriptor )
setMethod( "toString", "FieldDescriptor", ._toString_FieldDescriptor )
setMethod( "toString", "ServiceDescriptor", ._toString_ServiceDescriptor )
setMethod( "toString", "MethodDescriptor", ._toString_MethodDescriptor )
setMethod( "toString", "FileDescriptor", ._toString_FileDescriptor )
setMethod( "toString", "EnumValueDescriptor", ._toString_EnumValueDescriptor )

setGeneric( "toJSON", function( x, ... ) {
    standardGeneric( "toJSON" )
} )
setMethod( "toJSON", c( x = "Message"),
function(x, preserve_proto_field_names = FALSE, always_print_primitive_fields = FALSE) {
    .Call( Message__as_json, x@pointer, preserve_proto_field_names, always_print_primitive_fields)
} )

setGeneric( "toDebugString", function( x ) {
  standardGeneric( "toDebugString" )
} )
setMethod( "toDebugString", c( x = "Message"),
  function(x) {
    .Call( Message__as_character, x@pointer)
  } )

setGeneric( "toTextFormat", function( x ) {
    standardGeneric( "toTextFormat" )
} )
setMethod( "toTextFormat", c( x = "Message"),
function(x) {
    .Call( Message__print_text_format, x@pointer )
} )

setGeneric( "field_count", function(object){
	standardGeneric( "field_count" )
} )
setMethod( "field_count", "Descriptor", function(object){
	.Call( Descriptor__field_count, object@pointer )
} )

setGeneric( "nested_type_count", function(object){
	standardGeneric( "nested_type_count" )
} )
setMethod( "nested_type_count", "Descriptor", function(object){
	.Call( Descriptor__nested_type_count, object@pointer )
} )


setGeneric( "enum_type_count", function(object){
	standardGeneric( "enum_type_count" )
} )
setMethod( "enum_type_count", "Descriptor", function(object){
	.Call( Descriptor__enum_type_count, object@pointer )
} )


setGeneric( "field", function(object, index, number, name){
	standardGeneric( "field" )
} )
setMethod( "field", "Descriptor", function( object, index, number, name){
	has_index  <- !missing(index)
	has_number <- !missing(number)
	has_name   <- !missing(name)

	if( as.integer(has_index) + as.integer(has_number) + as.integer(has_name) != 1L ){
		stop( "need exactly one of `index`, `number` or `name`" )
	}

	if( has_index ){
		return( .Call( Descriptor__field, object@pointer, as.integer(index)-1L ) )
	}

	if( has_number ){
		return( .Call( Descriptor__FindFieldByNumber, object@pointer, as.integer(number) ) )
	}

	if( has_name ){
		return( .Call( Descriptor__FindFieldByName, object@pointer, as.character(name) ) )
	}

} )

setGeneric( "nested_type", function(object, index, name){
	standardGeneric( "nested_type" )
} )
setMethod( "nested_type", "Descriptor", function(object, index, name ){

	has_index  <- !missing(index)
	has_name   <- !missing(name)
	if( !identical( as.integer(has_index) + as.integer(has_name), 1L ) ){
		stop( "need exactly one of `index` or `name`" )
	}

	if( has_index ){
		return( .Call( Descriptor__nested_type, object@pointer, as.integer(index)-1L ) )
	}

	if( has_name ){
		return( .Call( Descriptor__FindNestedTypeByName, object@pointer, as.character(name) ) )
	}

} )

setMethod( "enum_type", "Descriptor", function(object, index, name){
	has_index  <- !missing(index)
	has_name   <- !missing(name)
	if( !identical( as.integer(has_index) + as.integer(has_name), 1L ) ){
		stop( "need exactly one of `index` or `name`" )
	}
	if( has_index ){
		return( .Call( Descriptor__enum_type, object@pointer, as.integer(index)-1L ) )
	}

	if( has_name ){
		stop( "No routine for finding enum type of a Descriptor by name. ")
	}

})

# serialization of protobuf messages
# at the moment, we grab the payload as a raw vector and send this
# to the connection, but in the future, we will directly stream out
# the payload to the connection

setGeneric( "serialize" )
setMethod( "serialize", c( object = "Message" ) ,
	function( object, connection, ascii = FALSE, refhook = NULL){
                stopifnot(object$isInitialized())
		iscon <- inherits(connection, "connection")
		isnull <- is.null( connection )

		if( is.character( connection ) ){
			# pretend it is a file name
			if( !file.exists(connection) ){
				if( !file.exists( dirname(connection) ) ){
					stop( "directory does not exist" )
				}
				file <- normalizePath(connection, mustWork=FALSE)
			} else{
				file <- file_path_as_absolute(connection)
			}
			.Call( Message__serialize_to_file, object@pointer, file )
			invisible( NULL)
		} else if( iscon || isnull ) {

			# first grab the payload as a raw vector,
			payload <- .Call( Message__get_payload, object@pointer )
			if( isnull ){
				# just return it if the connection is NULL
				payload
			} else{
				# otherwise write the payload to the connection
				# FIXME: we might want to be more clever about this so that
				#        we don't have to store the full payload in memory
				#        but for now it will do just fine
				writeBin( payload, connection )
				invisible( NULL )
			}

		}
	}
)

as.list.Message <- function( x, ...){
	.Call(Message__as_list, x@pointer)
}
as.list.Descriptor <- function(x, ...){
    # Fields, then nested types, then enum types defined in the message
    # are returned in a list.
	.Call(Descriptor__as_list, x@pointer)
}
as.list.EnumDescriptor <- function( x, ...){
	.Call(EnumDescriptor__as_list, x@pointer)
}
as.list.FileDescriptor <- function( x, ...){
	.Call(FileDescriptor__as_list, x@pointer)
}
as.list.ServiceDescriptor <- function( x, ...){
	.Call(ServiceDescriptor__as_list, x@pointer)
}

# set of functions not supposed to be called by the end user
# these are thin wrappers around C functions, to help with
# the error trapping on R side

#' thin wrapper around the C function of the same name
#'
#' @param files list of proto files to import
#' @param dir directory in which to look for proto files (ignored if files is given)
#' @param package if specified, proto files are imported from the "proto" directory
#'                of the package
#' @return invisible(NULL)
readProtoFiles <- function(files,
                           dir,
                           package = "RProtoBuf",
                           pattern = "\\.proto$",
                           lib.loc = NULL
	){

	if( missing( files ) ){
		if( missing( dir ) ){
			if( missing(package) ){
				dir <- getwd()
			} else {
				dir <- system.file( "proto", package = package, lib.loc = lib.loc )
				if( !file.exists( dir ) ){
					stop( sprintf("package '%s' does not have a 'proto' directory", package) )
				}
			}
		}
		files <- list.files( dir, pattern = pattern, full.names = TRUE )
	} else {
		if( !is.character( files )  ){
			stop( "'file' should be a character vector" )
		}

		ex <- sapply( files, file.exists )
		if( ! any( ex ) ){
			stop( "none of the files exist" )
		}

		# TODO: we need to pass the full path of the file
		#       or create a mapping from the current working directory
		#       in the DiskSourceTree
		files <- sapply(files[ex], function(x) file_path_as_absolute(x) )
	}
	directories <- unique( c( getwd(), dirname(files) ) )
	.Call( readProtoFiles_cpp, files, directories )
	invisible(NULL)
}

readProtoFiles2 <- function(files,
                            dir = ".",
                            pattern = "\\.proto$",
                            recursive = FALSE,
                            protoPath = getwd()
	){

	if( !is.character( protoPath ) || length( protoPath ) < 1 ){
		stop( "'protoPath' should be a character vector with at least one element" )
	}
	protoPath <- unique( normalizePath( protoPath ) )

	# Search for the base directory in the search path that contains the given filepath.
	file_search <- function(filepaths) {
		helper <- function(filepath) {
			if( file.exists( filepath ) && file_path_as_absolute( filepath ) == filepath ){
				return( TRUE )
			}
			for( baseDir in protoPath ){
				if( file.exists( file.path( baseDir, filepath ) ) ){
					return( baseDir )
				}
			}
			return( NA_character_ )
		}
		sapply( filepaths, helper )
	}

	if( missing( files ) ) {
		if( !is.character( dir ) || length( dir ) < 1 ){
			stop( "'dir' argument must be non-empty when 'files' is missing" )
		}

		files <- c()
		# search for dir relative to the search path.
		baseDirs <- file_search( dir )
		for( i in seq_along( dir ) ) {
			absPaths <-
				list.files( file.path( baseDirs[i], dir[i] ), pattern = pattern,
                    recursive = recursive, full.names = TRUE )
			files <- c( files, substr( absPaths, nchar( baseDirs[i] ) + 2, 10000L ) )
			files <- sub( "^\\./", "", files )
		}
	}
	missing_files <- files[ is.na( file_search( files ) ) ]
	if( length( missing_files) > 0 ){
		warning( "Missing files: ", paste(missing_files, collapse = ", " ) )
	}

	.Call( readProtoFiles_cpp, files, protoPath )
	invisible(NULL)
}

resetDescriptorPool <- function(){
	.Call( resetDescriptorPool_cpp )
	readProtoFiles2( protoPath=system.file( "proto", package="RProtoBuf", lib.loc=.RProtoBuf_libname) )
	invisible(NULL)
}

getProtobufLibVersion <- function(format = FALSE){
	version <- .Call(get_protobuf_library_version)

	if (format) {
		major <- version %/% 10^6
		minor <- ( tmp <- (version - major * 10^6) ) %/% 10^3
		revision <- (tmp - minor * 10^3 )
		package_version(sprintf("%d.%d.%d", major, minor, revision))
	} else {
		version
	}

}

# Copyright 2011 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#.setUp <- function() {
    resetDescriptorPool()
#}

#test.import.reset <- function() {
## Verify that the common setup is resetting the descriptor pool.
## Only the messages defined in the proto directory should be present.
expect_true(all(grepl("^(rexp|tutorial|rprotobuf)\\.", objects("RProtoBuf:DescriptorPool"))))
#}

#test.import.error <- function() {
## Verify that we get a graceful error rather than a segfault.
expect_error(readProtoFiles(system.file("DESCRIPTION", package="RProtoBuf"), syntax="proto3"))
#}

#test.import <- function() {
#unit.test.data.dir <- system.file("unitTests", "data", package="RProtoBuf")
unit.test.data.dir <- file.path("data", "cyclical")
expect_true(!exists("protobuf_unittest_import.ImportMessage", "RProtoBuf:DescriptorPool"))

resetDescriptorPool()
readProtoFiles2(recursive = FALSE, protoPath=unit.test.data.dir)
#expect_true(exists("protobuf_unittest_import.ImportMessage", "RProtoBuf:DescriptorPool"))
# The following is imported by unittest.proto and hence, should have been imported implicitly.
#expect_true(exists("protobuf_unittest_subdir.SubdirMessage", "RProtoBuf:DescriptorPool"))
# The following is not imported by any of the top level files.
expect_true(!exists("protobuf_unittest_recursive_subdir.SubdirMessage", "RProtoBuf:DescriptorPool"))


#test.import.subdir <- function() {
unit.test.data.dir <- system.file("tinytest", "data", package="RProtoBuf")

# Any of these two will work.
readProtoFiles2(file="subdir/subdir_message.proto", protoPath=unit.test.data.dir)
readProtoFiles2(dir="subdir", protoPath=unit.test.data.dir)
expect_true(exists("protobuf_unittest_subdir.SubdirMessage", "RProtoBuf:DescriptorPool"))

readProtoFiles2(dir="recursive", recursive=TRUE, protoPath=unit.test.data.dir)
expect_true(exists("protobuf_unittest_recursive_subdir.SubdirMessage", "RProtoBuf:DescriptorPool"))
#}

#test.import.cyclical <- function() {
unit.test.cyclical.dir <- system.file("tinytest", "data", "cyclical", package="RProtoBuf")
proj1.dir <- file.path(unit.test.cyclical.dir, "proj1")
proj2.dir <- file.path(unit.test.cyclical.dir, "proj2")

# The following files should be loaded in order as C depends on B depends on A.

readProtoFiles2(file="proto/a.proto", protoPath=proj1.dir)
expect_true(exists("A", "RProtoBuf:DescriptorPool"))
expect_true(!exists("C", "RProtoBuf:DescriptorPool"))

readProtoFiles2(file="proto/b.proto", protoPath=proj2.dir)
expect_true(exists("B", "RProtoBuf:DescriptorPool"))

readProtoFiles2(file="proto/c.proto", protoPath=proj1.dir)
expect_true(exists("C", "RProtoBuf:DescriptorPool"))
#}

#test.assign.in.global <- function() {
xx.undef <<- 3
expect_equal(xx.undef, 3)
#}

## rprotobuf.h: R/C++ interface class library
##
## Copyright (C) 2017 - 2020  Dirk Eddelbuettel
##
## This file is part of RProtoBuf.
##
## RProtoBuf is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 2 of the License, or
## (at your option) any later version.
##
## RProtoBuf is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RProtoBuf.  If not, see <http://www.gnu.org/licenses/>.

library(RProtoBuf)

## The Travis CI tests use both proto2 and proto3, so we need to condition against
## running this one with proto2 where it cannot pass we impose a proto3 file
isProto3 <- (RProtoBuf:::getProtobufLibVersion() >= 3000000)

if (!isProto3) exit_file("Need Proto3 for this test.")

#.setUp <- function(){
if( !exists("SearchRequest", "RProtoBuf:DescriptorPool")) {
    unitest.proto.file <- system.file("tinytest", "data", "proto3.proto", package="RProtoBuf")
    readProtoFiles(file = unitest.proto.file)
}
#}

#test.proto3 <- function() {
q <- new(SearchRequest, query="abc", page_number=42L, result_per_page=77L)
expect_equal(q$query, "abc", msg="proto3 string")
expect_equal(q$page_number, 42L, msg="proto3 int")
expect_equal(q$result_per_page, 77L, msg="proto3 int again")
#}

setGeneric("swap", function( object, field, left, right) {
	standardGeneric( "swap" )
} )
setMethod( "swap", "Message", function( object, field, left, right ){
	left <- as.integer( left ) - 1L
	right <- as.integer( right ) - 1L

	if( length(left) != length(right) ){
		stop( "left and right should have the same length" )
	}
	.Call( Message__swap, object@pointer, field, left, right )
	invisible( object )
} )

# :tabSize=4:indentSize=4:noTabs=false:folding=explicit:collapseFolds=1:

# implementation of completion for proto messages
# through the .DollarNames method (R>= 2.10.0)

# S4 dispatch does not work for .DollarNames, so we'll use S3
# {{{ bring .DollarNames from the future if necessary
if( !exists( ".DollarNames", envir = asNamespace("utils") ) ){
	.DollarNames <- function(x, pattern)
    	UseMethod(".DollarNames")
}
# }}}

# {{{ Message
.DollarNames.Message <- function(x, pattern = "" ){

	names <- c(
		.Call( Message__fieldNames, x@pointer ) ,
		"has(", "clone()", "clone(", "isInitialized()", "serialize(",
		"clear()", "clear(", "size(", "bytesize()", "fetch(",
		"swap(", "str()", "as.character()", "update(", "as.list()",
                "setExtension(", "getExtension(",
                "descriptor()", "set(", "toString(", "toJSON(", "add(", "fileDescriptor()" )
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ Descriptor
.DollarNames.Descriptor <- function(x, pattern = "" ){

	names <- c(
		.Call( Descriptor__getMemberNames, x@pointer ),
		"new(", "read(", "readASCII(", "readJSON(", "fileDescriptor()", "name(",
                "toString()", "as.character()", "asMessage()",
		"containing_type()", "field_count()", "nested_type_count()", "enum_type_count",
		"field(", "nested_type(", "enum_type(" )
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ EnumDescriptor
.DollarNames.EnumDescriptor <- function(x, pattern = "" ){

	names <- c(
		.Call( EnumDescriptor__getConstantNames, x@pointer ),
		"name(", "fileDescriptor()", "as.character()", "toString()",
		"containing_type()", "length()", "value_count()", "value(",
		"has(", "asMessage()")
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ EnumValueDescriptor
.DollarNames.EnumValueDescriptor <- function(x, pattern = "" ){
	names <- c("number()", "name()", "enum_type()",
		   "as.character()", "toString()", "asMessage()")
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ FieldDescriptor
.DollarNames.FieldDescriptor <- function(x, pattern = "" ){
	names <- c("as.character()", "toString()", "name(",
		"fileDescriptor()", "containing_type()",
		"is_extension()", "number()", "type(", "cpp_type(", "label(",
		"is_repeated()", "is_required()", "is_optional()",
		"message_type()", "enum_type()", "asMessage()",
		"has_default_value()", "default_value("
	)
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ ServiceDescriptor
.DollarNames.ServiceDescriptor <- function(x, pattern = "" ){
	names <- c(
		.Call( ServiceDescriptor__getMethodNames, x@pointer ),
		"as.character()", "toString()", "name(", "fileDescriptor()",
		"method_count()", "method(name=", "method(index="  )
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ MethodDescriptor
.DollarNames.MethodDescriptor <- function(x, pattern = "" ){
	names <- c("as.character()", "toString()", "name(", "fileDescriptor()",
	"input_type()", "output_type()" )
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ FileDescriptor
.DollarNames.FileDescriptor <- function(x, pattern = "" ){
	names <- c(
		.Call( FileDescriptor__getMemberNames, x@pointer ),
		"as.character()", "toString()", "name(", "as.list()",
                   "asMessage()", "package()")
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ ZeroCopyInputStream
.DollarNames.ZeroCopyInputStream <- function(x, pattern = "" ){
	names <- c("Next()", "ByteCount()", "Skip(",
		"BackUp(", "ReadRaw(", "ReadString()",
		"ReadVarint32()", "ReadVarint64()",
		"ReadLittleEndian32()", "ReadLittleEndian64()" )
	grep( pattern, names, value = TRUE )
}
# }}}

# {{{ ZeroCopyOutputStream
.DollarNames.ZeroCopyOutputStream <- function(x, pattern = "" ){
	names <- c("Next(", "ByteCount()", "BackUp(",
		"WriteRaw(",  "WriteString(",  "WriteLittleEndian32(",
		"WriteLittleEndian64(", "WriteVarint32(", "WriteVarint64(" )
	grep( pattern, names, value = TRUE )
}
# }}}

setGeneric( "ReadRaw", function(object, size ){
	standardGeneric( "ReadRaw" )
} )
setMethod( "ReadRaw", c( object="ZeroCopyInputStream", size = "integer" ), function(object, size){
	.Call(ZeroCopyInputStream_ReadRaw, object@pointer, size)
} )
setMethod("ReadRaw", c( object="ZeroCopyInputStream", size = "numeric" ),
          function(object, size) {
        if (size %% 1 == 0) {
                .Call(ZeroCopyInputStream_ReadRaw, object@pointer, as.integer(size))
        } else {
                stop("Size must be a whole number.")
        }
} )

setGeneric( "ReadString", function(object, size ){
	standardGeneric( "ReadString" )
} )
setMethod( "ReadString", c( object="ZeroCopyInputStream", size = "integer" ), function(object, size){
	.Call(ZeroCopyInputStream_ReadString, object@pointer, size)
} )
setMethod("ReadString", c( object="ZeroCopyInputStream", size = "numeric" ),
          function(object, size) {
        if (size %% 1 == 0) {
                .Call(ZeroCopyInputStream_ReadString, object@pointer, as.integer(size))
        } else {
                stop("Size must be a whole number.")
        }
} )

setGeneric( "ReadVarint32", function(object){
	standardGeneric( "ReadVarint32" )
} )
setMethod( "ReadVarint32", c( object="ZeroCopyInputStream"), function(object){
	.Call(ZeroCopyInputStream_ReadVarint32, object@pointer)
} )

setGeneric( "ReadLittleEndian32", function(object){
	standardGeneric( "ReadLittleEndian32" )
} )
setMethod( "ReadLittleEndian32", c( object="ZeroCopyInputStream"), function(object){
	.Call(ZeroCopyInputStream_ReadLittleEndian32, object@pointer)
} )

setGeneric( "ReadLittleEndian64", function(object){
	standardGeneric( "ReadLittleEndian64" )
} )
setMethod( "ReadLittleEndian64", c( object="ZeroCopyInputStream"), function(object){
	.Call(ZeroCopyInputStream_ReadLittleEndian64, object@pointer)
} )

setGeneric( "ReadVarint64", function(object){
	standardGeneric( "ReadVarint64" )
} )
setMethod( "ReadVarint64", c( object="ZeroCopyInputStream"), function(object){
	.Call(ZeroCopyInputStream_ReadVarint64, object@pointer)
} )

## TODO(mstokely): Set and fetch are undocumented in Message-class.Rd
## And untested.  Are they usful?

setGeneric( "set", function(object, field, index, values ){
	standardGeneric( "set" )
} )
setMethod( "set", "Message", function(object, field, index, values ){

	if( !is.numeric( index ) ){
		stop( "index should be numbers" )
	}

	if( inherits( values, "Message" ) ){
		values <- list( values )
	}
	# TODO: we need to handle R type = RAWSXP and cpp type == string or bytes
	if( !identical( length( values ), length(index) ) ){
		# TODO: we might want to be more flexible about that later
		#       recycling, ...
		stop( "`index` should have the same length as `values`" )
	}

	fsize <- size( object, field )
	if( any( index > fsize ) || any( index < 1) ){
		stop( sprintf( "index should only contain values between 1 and %d", fsize )  )
	}

	.Call( Message__set_field_values, object@pointer, field, index - 1L , values )

	# we work by reference
	invisible( NULL )
} )

setGeneric( "fetch", function(object, field, index ){
	standardGeneric( "fetch" )
} )
setMethod( "fetch", "Message", function(object, field, index ){

	if( !is.numeric( index ) ){
		stop( "index should be numbers" )
	}
	fsize <- size( object, field )
	if( any( index > fsize ) || any( index < 1) ){
		stop( sprintf( "index should only contain values between 1 and %d", fsize )  )
	}
	.Call( Message__get_field_values, object@pointer, field, index - 1L  )
} )

setGeneric( "clone", function( object, ... ){
	standardGeneric( "clone" )
} )
._clone.message <- function( object, ... ){
	message <- .Call(Message__clone, object@pointer)
	update( message, ... )
	message
}
setMethod( "clone", "Message", ._clone.message )

# Functions to convert an arbitrary R object into a protocol buffer
# using the universal rexp.proto descriptor.
#
# Written by Jeroen Ooms
# Modified 2014 by Murray Stokely to support language and environment types

rexp_obj <- function(obj){
  if(isS4(obj)) {
    # Some S4 objects do not return "S4" from typeof.  e.g. lubridate interval
    # These must be natively encoded or we will lose the S4 flag.
    return(rexp_native(obj))
  }
  sm <- typeof(obj);
  msg <- switch(sm,
    "character" = rexp_string(obj),
    "raw" = rexp_raw(obj),
    "double" = rexp_double(obj),
    "complex" = rexp_complex(obj),
    "integer" = rexp_integer(obj),
    "list" = rexp_list(obj),
    "logical" = rexp_logical(obj),
    "NULL" = rexp_null(),
    return(rexp_native(obj))
  );

  attrib <- attributes(obj)
  msg$attrName <- names(attrib)
  msg$attrValue <- lapply(attrib, rexp_obj)
  msg
}

rexp_string <- function(obj){
  xvalue <- lapply(as.list(obj), function(x){
    new(pb(rexp.STRING), strval=x, isNA=is.na(x))
  })
  new(pb(rexp.REXP), rclass = 0, stringValue=xvalue)
}

# For objects that only make sense in R, we just fall back
# to R's default serialization.

rexp_native <- function(obj){
  new(pb(rexp.REXP), rclass= 8, nativeValue = base::serialize(obj, NULL))
}

rexp_raw <- function(obj){
  new(pb(rexp.REXP), rclass= 1, rawValue = obj)
}

rexp_double <- function(obj){
  new(pb(rexp.REXP), rclass=2, realValue = obj)
}

rexp_complex <- function(obj){
  xvalue <- lapply(as.list(obj), function(x){
    new(pb(rexp.CMPLX), real=Re(x), imag=Im(x))
  })
  new(pb(rexp.REXP), rclass=3, complexValue = xvalue)
}

rexp_integer <- function(obj){
  new(pb(rexp.REXP), rclass=4, intValue = obj)
}

rexp_list <- function(obj){
  if (is(obj, "POSIXlt")) {
      xobj <- lapply(unclass(obj), rexp_obj)
  } else if (length(obj) > 0 && identical(obj, obj[[1]])) {
      # Avoid infinite recursion
      # some R objects return themselves when subindexed
      xobj <- rexp_obj(unlist(obj))
  } else {
      xobj <- lapply(obj, rexp_obj)
  }

  new(pb(rexp.REXP), rclass=5, rexpValue = xobj)
}

rexp_logical <- function(obj){
  xobj <- as.integer(obj)
  xobj[is.na(obj)] <- 2
  new(pb(rexp.REXP), rclass=6, booleanValue = xobj)
}

rexp_null <- function(){
  new(pb(rexp.REXP), rclass=7)
}

unrexp <- function(msg){
  stopifnot(is(msg, "Message"))
  stopifnot(msg@type == "rexp.REXP")

  myrexp <- as.list(msg)
  xobj <- switch(as.character(myrexp$rclass),
     "0" = unrexp_string(myrexp),
     "1" = unrexp_raw(myrexp),
     "2" = unrexp_double(myrexp),
     "3" = unrexp_complex(myrexp),
     "4" = unrexp_integer(myrexp),
     "5" = unrexp_list(myrexp),
     "6" = unrexp_logical(myrexp),
     "7" = unrexp_null(),
     "8" = unrexp_native(myrexp),
     stop("Unsupported rclass:", myrexp$rclass)
  )

  if(length(myrexp$attrValue)){
    attrib <- lapply(myrexp$attrValue, unrexp)
    names(attrib) <- myrexp$attrName
    tryCatch(attributes(xobj) <- attrib, error=function(cond) {
        #Try not setting the class in case the values are invalid
        attributes(xobj) <- attrib[names(attrib)!="class"]
        warning("Unable to set class, ", cond$message)
    })
  }

  xobj
}

unrexp_string <- function(myrexp){
  mystring <- unlist(lapply(myrexp$stringValue, "[[", "strval"))
  isNA <- unlist(lapply(myrexp$stringValue, "[[", "isNA"))
  mystring[isNA] <- NA
  as.character(mystring)
}

unrexp_raw <- function(myrexp){
  myrexp$rawValue
}

unrexp_double <- function(myrexp){
  myrexp$realValue
}

unrexp_complex <- function(myrexp){
  xvalue <- lapply(myrexp$complexValue, function(x){
    complex(real=x$real, imaginary=x$imag)
  })
  as.complex(unlist(xvalue))
}

unrexp_integer <- function(myrexp){
  myrexp$intValue
}

unrexp_list <- function(myrexp){
  lapply(myrexp$rexpValue, unrexp)
}

unrexp_logical <- function(myrexp){
  xvalue <- myrexp$booleanValue
  xvalue[xvalue==2] <- NA
  as.logical(xvalue)
}

unrexp_null <- function(){
  NULL
}

unrexp_native <- function(myrexp){
  buf <- myrexp$nativeValue
  if(!length(buf))
    return(NULL)
  unserialize(buf)
}

#Helper function to lookup a PB descriptor
pb <- function(name){
  descriptor <- deparse(substitute(name))
  if(!exists(descriptor, "RProtoBuf:DescriptorPool")){
    stop("No ProtoBuf Descriptor for: ", descriptor)
  }
  get(descriptor, "RProtoBuf:DescriptorPool")
}

#Checks if object can be serialized
can_serialize_pb <- rexp_valid <- function(obj) {
# We can now serialize everything.  just call back to R serialization
  return(TRUE)
}

# Copyright 2012 Google Inc. All Rights Reserved.
# Author: mstokely@google.com (Murray Stokely)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

setGeneric( "setExtension", function(object, field, values){
        standardGeneric( "setExtension" )
} )

setMethod( "setExtension", "Message", function( object, field, values ){
	if (!inherits(field, "FieldDescriptor")) {
		stop("setExtension requires a FieldDescriptor")
	}
	if (!is_extension(field)) {
		stop(name(field), " is not an extension FieldDescriptor.")
	}
	.Call(setMessageField, object@pointer, field, values)
	invisible( object )
} )


setGeneric( "getExtension", function(object, field){
	standardGeneric( "getExtension" )
} )
setMethod( "getExtension", "Message", function( object, field){
	if (!inherits(field, "FieldDescriptor")) {
		stop("getExtension requires a FieldDescriptor")
	}
	if (!is_extension(field)) {
		stop(name(field), " is not an extension FieldDescriptor.")
	}
	# This check causes a CHECK failure in the C++ code, so give
	# a more user-friendly error here.
	if (containing_type(field)@type != object@type) {
		stop("Field", name(field), " does not match message type ",
             "(", containing_type(field)@type, "!=", object@type, ")")
	}
    .Call(getExtension_cpp, object@pointer, field)
} )

# Build against protobuf libs compiled with Rtools
VERSION <- commandArgs(TRUE)
if (!file.exists(sprintf("../windows/protobuf-%s/include/google/protobuf/descriptor.h", VERSION))) {
    if (getRversion() < "3.3.0") setInternet2()
    download.file(sprintf("https://github.com/rwinlib/protobuf/archive/v%s.zip", VERSION), "lib.zip", quiet = TRUE)
    dir.create("../windows", showWarnings = FALSE)
    unzip("lib.zip", exdir = "../windows")
    unlink("lib.zip")
}

setGeneric( "containing_type", function(object){
	standardGeneric( "containing_type" )
} )

setMethod( "containing_type", "Descriptor", function(object){
	retval <- .Call( Descriptor__containing_type, object@pointer )
	if (length(retval@type) == 0) {
		# Descriptors do not always have containing types.
		# In such cases NULL is better return value than malformed Descriptor.
		return(NULL)
	} else {
                return(retval)
	}
} )
setMethod( "containing_type", "EnumDescriptor", function(object){
	retval <- .Call( EnumDescriptor__containing_type, object@pointer )
	if (length(name(retval)) == 0) {
		# If this enum type is nested in a message type, this
		# is that message type.	 Otherwise, NULL.
		return(NULL)
	} else {
                return(retval)
	}
} )
setMethod( "containing_type", "FieldDescriptor", function(object){
    # Never NULL
	.Call( FieldDescriptor__containing_type, object@pointer )
} )

setGeneric( "bytesize", function(object, ...){
	standardGeneric( "bytesize" )
} )
setMethod( "bytesize", "Message", function(object, ...){
	.Call(Message__bytesize, object@pointer)
} )

setGeneric( "size", function( object, field, ... ){
	standardGeneric( "size" )
} )
setMethod( "size", "Message", function(object, field, ...){

	if( is.character( field ) || is.numeric( field ) ){
		.Call(Message__field_size, object@pointer, field)
	} else{
		stop( "field should be a character or a number" )
	}

} )


setGeneric( "size<-", function( object, field, ..., value ){
	standardGeneric( "size<-" )
} )
setMethod( "size<-", "Message", function(object, field, ..., value){

	if( !is.numeric( value ) ){
		stop("value should be a number")
	}
	value <- as.integer( value )[1]

	if( is.character( field ) || is.numeric( field ) ){
		.Call(Message__set_field_size, object@pointer, field, value)
	} else{
		stop( "field should be a character or a number" )
	}
	object
} )

setGeneric( "value_count", function(object ){
	standardGeneric( "value_count" )
} )
setMethod( "value_count", "EnumDescriptor", function(object){
	.Call(EnumDescriptor__value_count, object@pointer)
} )


setGeneric( "value", function(object, index, name, number ){
	standardGeneric( "value" )
} )
setMethod( "value", "EnumDescriptor", function(object, index, name, number){

	has_index  <- !missing(index)
	has_number <- !missing(number)
	has_name   <- !missing(name)

	if( as.integer(has_index) + as.integer(has_number) + as.integer(has_name) != 1L ){
		stop( "need exactly one of `index`, `number` or `name`" )
	}

	if( has_index ){
		stopifnot(is.numeric(index))
		return( .Call(EnumDescriptor__getValueByIndex, object@pointer, as.integer(index)-1L) )
	}

	if( has_number ){
		stopifnot(is.numeric(number))
		return( .Call(EnumDescriptor__getValueByNumber, object@pointer, as.integer(number)) )
	}

	if( has_name ){
		stopifnot(is.character(name))
		return( .Call(EnumDescriptor__getValueByName, object@pointer, as.character(name)) )
	}

} )

# :tabSize=4:indentSize=4:noTabs=false:folding=explicit:collapseFolds=1:

# this has to be
IMPLEMENTATIONS <- new.env( parent = emptyenv() )

# invisible version of .Call
#.icall <- function(...) invisible(.Call(...))

# {{{ class definitions
# we need to formalize connection so that the S4 methods
# ConnectionInputStream and ConnectionOutputStream can
# dispatch connections
setOldClass( "connection" )

# FIXME: there is probably another way
# TODO: include other subclasses of connections
setOldClass( c("file", "connection" ) )
setOldClass( c("url", "connection" ) )

setClass( "Descriptor", representation(
	pointer = "externalptr",   # pointer to a google::protobuf::Descriptor c++ object
	type = "character"         # message type
), prototype = list( pointer = NULL, type = character(0) ) )

setClass( "FieldDescriptor", representation(
	pointer = "externalptr" ,   # pointer to a google::protobuf::FieldDescriptor c++ object
	name    = "character",
	full_name  = "character",
	type    = "character"
), prototype = list( pointer = NULL, name = character(0),
	full_name = character(0), type = character(0) ) )

setClass( "EnumDescriptor", representation(
	pointer = "externalptr" ,  # pointer to a google::protobuf::EnumDescriptor c++ object
	name    = "character",
	full_name  = "character",
	type    = "character"   # TODO(mstokely): enums don't really have another type, remove?
), prototype = list( pointer = NULL, name = character(0),
	full_name = character(0), type = character(0) ) )

setClass( "ServiceDescriptor", representation(
	pointer = "externalptr",   # pointer to a google::protobuf::ServiceDescriptor c++ object
	name = "character"         # full name of the service
), prototype = list( pointer = NULL, name = character(0) ) )

setClass( "MethodDescriptor", representation(
	pointer = "externalptr",   # pointer to a google::protobuf::ServiceDescriptor c++ object
	name = "character"     ,   # full name of the service
	service = "character"      # full name of the service that defines this method
), prototype = list( pointer = NULL, name = character(0), service = character(0) ) )

setClass( "FileDescriptor", representation(
	pointer = "externalptr",  # pointer to a google::protobuf::FileDescriptor c++ object
	filename = "character",	  # filename
	package = "character"     # the package
), prototype = list( pointer = NULL, filename = character(0), package = character(0) ) )

setClass( "EnumValueDescriptor", representation(
	pointer = "externalptr",  # pointer to a google::protobuf::EnumValueDescriptor c++ object
	name    = "character",
	full_name  = "character"
), prototype = list( pointer = NULL, name = character(0), full_name = character(0) ) )

# actual objects

setClass( "Message",  representation(
   pointer = "externalptr",    # pointer to sa google::protobuf::Message object
   type    = "character"       # message type (fully qualified, with package path)
), prototype = list( pointer = NULL, type = character(0) ) )

# rpc

#setClass( "RpcHTTP", representation(
#	host = "character", port = "integer", root = "character"
#), prototype = list( host = "127.0.0.1", port = 4444L, root = "" ) )

# streams
setClass( "ZeroCopyInputStream", representation(
	pointer = "externalptr"
), prototype = list( pointer = NULL ), contains = "VIRTUAL" )
setClass( "ArrayInputStream", contains = "ZeroCopyInputStream" )
setClass( "FileInputStream", contains = "ZeroCopyInputStream" )
setClass( "ConnectionInputStream", contains = "ZeroCopyInputStream" )

setClass( "ZeroCopyOutputStream", representation(
	pointer = "externalptr"
), prototype = list( pointer = NULL ), contains = "VIRTUAL" )
setClass( "ArrayOutputStream", contains = "ZeroCopyInputStream" )
setClass( "FileOutputStream", contains = "ZeroCopyInputStream" )
setClass( "ConnectionOutputStream", contains = "ZeroCopyOutputStream" )
# }}}

# {{{ new
newProto <- function( descriptor, ... ){
	message <- .Call(newProtoMessage, descriptor)
	update( message, ... )
	message
}
setGeneric("new")
setMethod("new", signature(Class="Descriptor"), function(Class, ...) newProto(Class, ...))
# }}}

# {{{ P
P <- function( type, file ){

	if( !missing(file) ){
		readProtoFiles( file )
	}
	if( missing( type ) ){
		stop( "'type' is required" )
	}
	if( !is.character(type) ){
		stop( "'type' is not a character vector" )
	}
	if( length(type) != 1L){
		stop( "'type' should have exactly one element" )
	}

	desc <- .Call(getProtobufDescriptor, type)
	if( is.null( desc ) ){
		# See if it is an extension
		desc <- .Call(getExtensionDescriptor, type)
		if (is.null(desc)) {
			# See if it is an enum
			desc <- .Call(getEnumDescriptor, type)
			if (is.null(desc)) {
				stop( sprintf( "could not find descriptor for message type '%s' ", type ) )
			}
		}
	}
	desc
}
# }}}

# {{{ show
setMethod( "show", c( "Message" ), function(object){
  tmp <- sprintf( "message of type '%s' with %d field%s set", object@type,
                 length(object), if (length(object) == 1) "" else "s" )
  nexts <- .Call(Message__num_extensions, object@pointer)
  if (nexts > 0) {
    tmp <- paste(tmp, sprintf("and %d extension%s", nexts, if (nexts == 1) "" else "s"))
  }
  cat(tmp, fill=TRUE)
} )
setMethod( "show", c( "Descriptor" ), function(object){
	cat( sprintf( "descriptor for type '%s' ", object@type ) , fill=TRUE)
} )
setMethod( "show", c( "FieldDescriptor" ), function(object){
	cat( sprintf( "descriptor for field '%s' of type '%s' ",
                     object@name, object@type ), fill=TRUE)
} )
setMethod( "show", c( "EnumDescriptor" ), function(object){
	cat( sprintf( "descriptor for enum '%s' with %d values", object@name,
                      value_count(object) ) , fill=TRUE)
} )
setMethod( "show", c( "ServiceDescriptor" ), function(object){
	cat( sprintf( "service descriptor <%s>", object@name ) , fill=TRUE)
} )
setMethod( "show", c( "FileDescriptor" ), function(object){
	cat( sprintf( "file descriptor for package %s (%s)", object@package,
                      object@filename) , fill=TRUE)
} )
setMethod( "show", c( "EnumValueDescriptor" ), function(object){
	cat( sprintf( "enum value descriptor %s", object@full_name), fill=TRUE)
} )

# }}}

# {{{ dollar extractors
setMethod("$", "Message", function(x, name) {

	switch( name,
		"has" = function( ... ) has(x, ...),
		"clone" = function( ... )    .Call(Message__clone         , x@pointer),
		"isInitialized" = function() .Call(Message__is_initialized, x@pointer),
		"descriptor" = function()    .Call(Message__descriptor    , x@pointer),

		"size"  = function(field, ...) size(x, field, ... ),
		"bytesize" = function() bytesize(x),
		"swap" = function(...) swap(x,...),
		"update" = function(...) update(x, ...),
		"str" = function() str(x),
		"as.character" = function() as.character(x),
		"as.list" = function() as.list(x),
		"asMessage" = function() asMessage(x),
		"setExtension" = function(field, values, ...) setExtension(x, field, values, ...),
		"getExtension" = function(field, ...) getExtension(x, field, ...),
		"set" = function(...) set( x, ... ),
		"fetch" = function(...) fetch(x, ... ),
		"toString" = function(...) toString( x, ... ),
                "toJSON" = function(preserve_proto_field_names = FALSE, always_print_primitive_fields = FALSE, ...)
            		toJSON(x, preserve_proto_field_names = preserve_proto_field_names,
                           always_print_primitive_fields = always_print_primitive_fields, ... ),
		"toTextFormat" = function() toTextFormat(x),
		"toDebugString" = function() toDebugString(x),
		"add" = function(...) add( x, ...),

		"serialize" = function(...) serialize( x, ... ),
		"clear" = function(...) clear( x, ... ),

		"fileDescriptor" = function() fileDescriptor(x ),

		# default
		.Call(getMessageField, x@pointer, name)
		)
	} )
setMethod("$<-", "Message", function(x, name, value) {
	.Call(setMessageField, x@pointer, name, value)
	x
} )

setMethod("$", "Descriptor", function(x, name) {
	switch( name,
		"new" = function( ... ) newProto( x, ... ) ,
		"read" = function( input ) read( x, input ) ,
		"readASCII" = function(input, ...) readASCII(x, input, ...),
		"readJSON" = function(input, ...) readJSON(x, input, ...),
		"toString" = function(...) toString(x, ...) ,
		"as.character" = function(...) as.character(x, ...) ,
		"as.list" = function(...) as.list(x, ...) ,
		"asMessage" = function() asMessage(x),
		"fileDescriptor" = function() fileDescriptor(x ),
		"name" = function(...) name(x, ... ),

		"containing_type" = function() containing_type(x),
		"field_count" = function() field_count(x),
		"nested_type_count" = function() nested_type_count(x),
		"enum_type_count" = function() enum_type_count(x),
		"field" = function(...) field( x, ... ),
		"nested_type" = function(...) nested_type( x, ...),
		"enum_type" = function(...) enum_type( x, ...),


		# default
		.Call(do_dollar_Descriptor, x@pointer, name)
	)
} )
setMethod( "$", "EnumDescriptor", function(x, name ){
	switch( name,
		"as.character" = function() as.character(x),
		"as.list"= function() as.list(x) ,
		"asMessage" = function() asMessage(x),
		"toString" = function(...) toString(x, ...) ,
		"name" = function(...) name(x, ...),
		"fileDescriptor" = function() fileDescriptor(x ),
		"containing_type" = function() containing_type(x),
		"length" = function() length(x),
		"value_count" = function() value_count(x),
		"value" = function(...) value(x, ...),
                "has" = function(name, ...) has(x, name, ...),
		# default
		.Call(get_value_of_enum, x@pointer, name)
		)
} )
setMethod( "$", "FieldDescriptor", function(x, name ){
	switch( name,
		"as.character" = function() as.character(x),
		"asMessage" = function() asMessage(x),
		"toString" = function(...) toString(x, ...) ,
		"name" = function(...) name(x, ...),
		"fileDescriptor" = function() fileDescriptor(x ),
		"containing_type" = function() containing_type(x),

		"is_extension" = function() is_extension(x),
		"number" = function() number(x),
		"type" = function(...) type(x, ...),
		"cpp_type" = function(...) cpp_type(x, ...),
		"label" = function(...) label(x, ...),
		"is_repeated" = function() is_repeated(x),
		"is_required" = function() is_required(x),
		"is_optional" = function() is_optional(x),
		"has_default_value" = function() has_default_value(x),
		"default_value" = function() default_value(x),
		"enum_type" = function() enum_type(x),
		"message_type" = function() message_type(x),

		invisible(NULL)
		)
} )

setMethod( "$", "ServiceDescriptor", function(x, name ){
	switch( name,
		"as.character" = function() as.character(x),
		"asMessage" = function() asMessage(x),
		"toString" = function(...) toString(x, ...) ,
		"name" = function(...) name(x, ...),
		"fileDescriptor" = function() fileDescriptor(x ),
		"method_count" = function() method_count(x),
		"method" = function(...) method(x, ... ),

		stop( "No '", name, "' method for ServiceDescriptor objects is implemented." )
		)
} )

setMethod( "$", "FileDescriptor", function(x, name ){

	switch( name,
		"as.character" = function() as.character(x),
		"toString" = function(...) toString(x, ...) ,
		"asMessage" = function() asMessage(x),
		"as.list" = function() as.list(x),
      		"name" = function(...) name(x, ... ),
       		"package" = function() x@package,
                as.list(x)[[name]]
		)
})

setMethod( "$", "EnumValueDescriptor", function(x, name ){

	switch( name,
		"as.character" = function() as.character(x),
		"toString" = function(...) toString(x, ...) ,
		"asMessage" = function() asMessage(x),
		"name" = function(...) name(x, ... ),
		"number" = function() number(x),
		"enum_type" = function(...) enum_type( x, ...),
               invisible(NULL)
		)

})


setMethod( "$", "MethodDescriptor", function(x, name ){
	switch( name,
		#"invoke" = function(...) invoke(x, ...),
		"implementation" = if( x@name %in% names(IMPLEMENTATIONS) ){
			get( x@name, IMPLEMENTATIONS )
		},
		"name" = function(...) name(x, ...),
		"toString"  = function() toString(x) ,
		"as.character" = function() as.character(x),
		"fileDescriptor" = function() fileDescriptor(x ),
		"input_type" = function() input_type(x),
		"output_type" = function() output_type(x),

		invisible( NULL)
	)
} )

#setMethod( "$<-", "MethodDescriptor", function(x, name, value ){
#
#	if( identical( name, "implementation" ) ){
#		check_valid_implementation( x, value )
#		assign( x@name, value, envir = IMPLEMENTATIONS )
#	}
#	x
#} )

setMethod( "$", "ZeroCopyInputStream", function(x, name ){
	switch( name,
		# ZeroCopyInputStream C++ methods
		"Next" = function(...) Next(x, ...),
		"ByteCount" = function(...) ByteCount(x, ...),
		"Skip" = function(...) Skip(x, ...),
		"BackUp" = function(...) BackUp(x, ...),

		# CodedInputStream related
		"ReadRaw" = function(...) ReadRaw(x, ...),
		"ReadString" = function(...) ReadString(x, ...),
		"ReadVarint32"= function() ReadVarint32(x),
		"ReadVarint64" = function() ReadVarint64(x),
		"ReadLittleEndian32" = function() ReadLittleEndian32(x),
		"ReadLittleEndian64" = function() ReadLittleEndian64(x),

		# default
		invisible(NULL)
		)
} )

setMethod( "$", "ZeroCopyOutputStream", function(x, name ){
	switch( name,
		# ZeroCopyInputStream C++ methods
		"Next" = function(...) Next(x, ...),
		"ByteCount" = function(...) ByteCount(x, ...),
		"BackUp" = function(...) BackUp(x, ...),

		# CodedOutputStream related
		"WriteRaw" = function(...) WriteRaw(x, ...),
		"WriteString" = function(...) WriteString(x, ...),
		"WriteLittleEndian32" = function(...) WriteLittleEndian32(x,...),
		"WriteLittleEndian64" = function(...) WriteLittleEndian64(x,...),
		"WriteVarint32" = function(...) WriteVarint32(x, ...),
		"WriteVarint64" = function(...) WriteVarint64(x, ...),

		# default
		invisible(NULL)
		)
} )


# }}}

# {{{ [[
setMethod( "[[", "Message", function(x, i, j, ..., exact = TRUE){
	if( missing( i ) ){
		stop( "`i` is required" )
	}
	if( !missing(j) ){
		warning( "`j` is ignored" )
	}

        ## This works correctly by number or name. e.g. p[[1]] or p[["name"]]
	if( is.character( i ) || is.numeric( i ) ){
		.Call(getMessageField, x@pointer, i)
	} else {
		stop( "wrong type, `i` should be a character or a number" )
	}

} )

setMethod( "[[", "Descriptor", function(x, i, j, ..., exact = TRUE){
	if( missing( i ) ){
          stop( "`i` is required" )
	}
	if( !missing(j) ){
          warning( "`j` is ignored" )
	}

	if( is.character( i ) ) {
          # gets a named field, nested type, or enum.
          .Call(Descriptor_getField, x@pointer, i)
        } else if (is.numeric( i ) ) {
          return(as.list(x)[[i]])
	} else {
          stop( "wrong type, `i` should be a character or a number" )
	}
} )

setMethod( "[[", "EnumDescriptor", function(x, i, j, ..., exact = TRUE){
	if( missing( i ) ){
                stop( "`i` is required" )
	}
	if( !missing(j) ){
                warning( "`j` is ignored" )
	}
        if (is.character(i) || is.numeric(i)) {
          return(as.list(x)[[i]])
        } else {
          stop( "wrong type, `i` should be a character or a number" )
	}
} )

setMethod("[[", "ServiceDescriptor", function(x, i, j, ..., exact = TRUE){
	if( missing( i ) ){
		stop( "`i` is required" )
	}
	if( !missing(j) ){
		warning( "`j` is ignored" )
	}
	if( is.character( i ) || is.numeric( i ) ){
		stop( "No '", name, "' method for ServiceDescriptor objects is implemented." )
	} else{
		stop( "wrong type, `i` should be a character or a number" )
	}
} )

# }}}

# {{{ [[<-
setReplaceMethod( "[[", "Message",
function(x, i, j, ..., exact = TRUE, value ){

	# TODO: we might want to relax this later, i.e
	# allow to mutate repeated fields like this:
	# x[[ "field", 1:2 ]] <- 1:2

	if( missing( i ) ){
		stop( "`i` is required" )
	}
	if( !missing(j) ){
		warning( "`j` is ignored" )
	}

	if( is.character( i ) || is.numeric( i ) ){
		.Call( setMessageField, x@pointer, i, value )
	} else {
		stop( "wrong type, `i` should be a character or a number" )
	}
	x

} )

# }}}

# {{{ update
setGeneric( "update" )
setMethod( "update", "Message", function( object, ... ){

	dots <- list( ... )
	if( !length( dots ) ){
		return( object )
	}
	names <- names( dots )
	named <- dots[ names != "" ]
	if( !length( named ) ){
		return( object )
	}
	.Call( update_message, object@pointer, named)
	object

} )
# }}}

# {{{ length
setGeneric( "length" )
setMethod( "length", "Message", function( x ){
	.Call( Message__length, x@pointer )
} )
# Returns number of fields, enums, types in message descriptor.
# May be more than field_count which is only fields.
# e.g. length(tutorial.Person) > field_count(tutorial.Person)
setMethod( "length", "Descriptor", function( x ){
        length(as.list(x))
} )
setMethod( "length", "EnumDescriptor", function( x ){
	.Call( EnumDescriptor__length, x@pointer )
} )
setMethod( "length", "ServiceDescriptor", function( x ){
	.Call("ServiceDescriptor_method_count", x@pointer )
} )
# }}}

# {{{ str
# we need to do this otherwise it gets messed up by the length method
setGeneric( "str" )
setMethod( "str", "Message", function( object, ...){
txt <- sprintf( '	Formal class \'Message\' [package "RProtoBuf"] with 2 slots
  ..@ pointer:<externalptr>
  ..@ type   : chr "%s"
', object@type )
writeLines( txt )
} )
# }}}

# {{{ name
setGeneric( "name", function(object, full = FALSE){
	standardGeneric( "name" )
})
setMethod( "name", c( object = "Descriptor" ) ,
function(object, full = FALSE){
	.Call( Descriptor__name, object@pointer, full )
})
setMethod( "name", c( object = "FieldDescriptor" ) ,
function(object, full = FALSE){
	.Call( FieldDescriptor__name, object@pointer, full )
})
setMethod( "name", c( object = "EnumDescriptor" ) ,
function(object, full = FALSE){
	.Call( EnumDescriptor__name, object@pointer, full )
})
setMethod( "name", c( object = "EnumValueDescriptor" ) ,
function(object, full = FALSE){
	.Call( EnumValueDescriptor__name, object@pointer, full )
})
setMethod( "name", c( object = "ServiceDescriptor" ) ,
function(object, full = FALSE){
	.Call( ServiceDescriptor__name, object@pointer, full )
})
setMethod( "name", c( object = "MethodDescriptor" ) ,
function(object, full = FALSE){
	.Call( MethodDescriptor__name, object@pointer, full )
})
setMethod( "name", c( object = "FileDescriptor" ) ,
function(object, full = FALSE){
	filename <- .Call( FileDescriptor__name, object@pointer )
	if( full ) filename else basename( filename )
})
# }}}

# {{{ names
# as.list() and names() don't make as much sense for FieldDescriptors,
# EnumValueDescriptors, etc.
setMethod( "names", c( x = "Message" ) ,
function(x){
        names(as.list(x))
})
setMethod( "names", c( x = "Descriptor" ) ,
function(x){
        names(as.list(x))
})
setMethod( "names", c( x = "EnumDescriptor" ) ,
function(x){
        names(as.list(x))
})
# }}}

# {{{ as
setAs("Descriptor", "Message", function(from){
	.Call( Descriptor__as_Message, from@pointer )
})
setAs("FieldDescriptor", "Message", function(from){
	.Call( FieldDescriptor__as_Message, from@pointer )
})
setAs("EnumDescriptor", "Message", function(from){
	.Call( EnumDescriptor__as_Message, from@pointer )
})
setAs("ServiceDescriptor", "Message", function(from){
	.Call( ServiceDescriptor__as_Message, from@pointer )
})
setAs("MethodDescriptor", "Message", function(from){
	.Call( MethodDescriptor__as_Message, from@pointer )
})
setAs("FileDescriptor", "Message", function(from){
	.Call( FileDescriptor__as_Message, from@pointer )
})
setAs("EnumValueDescriptor", "Message", function(from){
	.Call( EnumValueDescriptor__as_Message, from@pointer )
})
asMessage <- function( x, ... ){
	as( x, "Message", ... )
}
# }}}

# {{{ enum_type
setGeneric( "enum_type", function( object, index, name ){
	standardGeneric( "enum_type" )
})
# }}}

# Copyright 2013 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.extension <- function() {
if (!exists("protobuf_unittest.TestAllTypes", "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

expect_true(inherits(protobuf_unittest.optional_uint32_extension,
                   "FieldDescriptor"))

## Verify we can pull in other extensions with P()
expect_true(inherits(P("protobuf_unittest.optional_uint32_extension"),
                   "FieldDescriptor"))

## Test setting and getting singular extensions.
test <- new(protobuf_unittest.TestAllExtensions)
test$setExtension(protobuf_unittest.optional_int32_extension,
                  as.integer(1))
expect_equal(test$getExtension(protobuf_unittest.optional_int32_extension),
            as.integer(1))
## Unset extensions should return NULL.
expect_equal(test$getExtension(protobuf_unittest.optional_double_extension),
            NULL)


## Test setting and getting repeated extensions.
expect_equal(test$getExtension(protobuf_unittest.repeated_int32_extension),
            NULL)
test$setExtension(protobuf_unittest.repeated_int32_extension, 1:10)
expect_equal(test$getExtension(protobuf_unittest.repeated_int32_extension),
            1:10)

## Test nested extensions.
expect_equal(test$getExtension(protobuf_unittest.TestNestedExtension.test),
            NULL)
test$setExtension(protobuf_unittest.TestNestedExtension.test, "foo")
expect_equal(test$getExtension(protobuf_unittest.TestNestedExtension.test),
            "foo")

## Test setting and getting enums.
test$setExtension(protobuf_unittest.optional_nested_enum_extension,
                  protobuf_unittest.TestAllTypes.NestedEnum$BAR)

## Test foreign message extensions
foo <- new(protobuf_unittest.ForeignMessage)
foo$c <- 3
test$setExtension(protobuf_unittest.optional_foreign_message_extension,
                  foo)
expect_equal(test$getExtension(protobuf_unittest.optional_foreign_message_extension)$c,
            3)

# This causes an Rcpp exception, but not an R stop error as of my
# version of Rcpp, so we can't checkError unfortunately, but we
# can at least make sure it doesn't crash R.
# TODO(edd): Commented out now
# test$setExtension(protobuf_unittest.optional_nested_enum_extension, 9)

## Test nested message extensions.
tmp <- new( protobuf_unittest.TestAllTypes.NestedMessage )
tmp$bb <- 3
test$setExtension(protobuf_unittest.optional_nested_message_extension, tmp)
expect_equal(test$getExtension(protobuf_unittest.optional_nested_message_extension)$bb, 3)

## Check that we do something sensible if invalid field descriptors are passed
expect_error(test$getExtension(protobuf_unittest.TestAllExtensions))
expect_error(test$setExtension(protobuf_unittest.TestAllExtensions, 3))

## Check that we don't CHECK fail in C++, but instead give a reasonable
## error message if incorrect extensions types are provided.
expect_error(test$getExtension(protobuf_unittest.my_extension_string))
#}

library(RProtoBuf)

#.setUp <- function(){
if( ! exists( "NestedOuter", "RProtoBuf:DescriptorPool" ) ) {
    ## None of the messages in unittest.proto is exactly right to tickle this bug.
    unitest.proto.file <- system.file( "tinytest", "data", "nested.proto", package = "RProtoBuf" )
    readProtoFiles(file = unitest.proto.file)
}
#}

# RProtoBuf before v0.2.5 had a bug whereby uninitialized memory was read and used
# to determine whether or not to update a nested message field in new() / update().
#test.nested <- function() {
for (i in 1:1000) {
    q <- new(NestedOuter, b=new(NestedInner, x=1))
    expect_true(q$isInitialized())
}
#}

# Copyright 2013 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.message <- function() {
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}
a <- new(protobuf_unittest.TestAllTypes)
a$optionalgroup$a <- 3
expect_equal(a$optionalgroup$a, 3)

a$repeated_nested_message <- list(
    new(protobuf_unittest.TestAllTypes.NestedMessage, bb=3),
    new(protobuf_unittest.TestAllTypes.NestedMessage, bb=4))
expect_equal(a$repeated_nested_message[[1]]$bb, 3)
expect_equal(a$repeated_nested_message[[2]]$bb, 4)

expect_error(a$repeated_nested_message <- list(
    new(protobuf_unittest.ForeignMessage, c=1),
    new(protobuf_unittest.TestAllTypes.NestedMessage, bb=4)))

expect_error(a$repeated_nested_message <- list(
    new(protobuf_unittest.TestAllTypes.NestedMessage, bb=4),
    new(protobuf_unittest.ForeignMessage, c=1)))

expect_error(a$repeated_nested_message <- list(
    new(protobuf_unittest.TestAllTypes.NestedMessage, bb=4),
    3))

expect_error(a$repeated_nested_message <- list(
    new(protobuf_unittest.TestAllTypes.NestedMessage, bb=4),
    "foo"))

expect_equal(a$toTextFormat(), "OptionalGroup {\n  a: 3\n}\nrepeated_nested_message {\n  bb: 3\n}\nrepeated_nested_message {\n  bb: 4\n}\n")
expect_equal(a$toDebugString(), a$toString())
expect_equal(a$toString(debug = FALSE), a$toTextFormat())
expect_equal(a$toString(debug = TRUE), a$toDebugString())
#}

# Copyright 2013 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.int32 <- function() {
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

a <- new(protobuf_unittest.TestAllTypes)
a$repeated_int32 <- 1
# Verify we can set character strings
a$repeated_int32 <- c("9007", "9008")
expect_equal(length(a$repeated_int32), 2)
# Verify we can't set any garbage string to a repeated int32.
expect_error(a$repeated_int32 <-c("invalid", "invalid"))
expect_error(a$repeated_int32 <-c("33-"))

a$optional_int32 <- 1
a$optional_int32 <- "2"
expect_equal(a$optional_int32, 2)
# Verify we can't set any garbage string to an optional int32.
expect_error(a$optional_int32 <- "invalid")

a$optional_uint32 <- 10000
a$optional_uint32 <- "200000"
expect_equal(a$optional_uint32, 200000)
# Verify we can't set any garbage string to an optional uint32.
expect_error(a$optional_uint32 <- "invalid")

a$repeated_uint32 <- c("9007", "9008")
expect_equal(length(a$repeated_uint32), 2)
#}

library(RProtoBuf)

#.setUp <- function(){
if( ! exists( "TestBytes", "RProtoBuf:DescriptorPool" ) ){
    unitest.proto.file <- system.file( "tinytest", "data", "bytes.proto", package = "RProtoBuf" )
    readProtoFiles(file = unitest.proto.file)
}
#}

#test.all <- function() {
test <- new(TestBytes, req = "abc", rep = list(charToRaw("def"), raw(10)))
expect_equal(rawToChar(test$req), "abc")
test$req <- charToRaw("abc")
expect_equal(rawToChar(test$req), "abc")
expect_equal(rawToChar(test$opt), "hello world")
expect_equal(test$rep, list(charToRaw("def"), raw(10)))
test$rep[[3]]=charToRaw("ghi")
expect_equal(test$rep, list(charToRaw("def"), raw(10), charToRaw("ghi")))
ser <- serialize(test, NULL)
test1 <- read(TestBytes, ser)
expect_equal(rawToChar(test1$req), "abc")
expect_equal(rawToChar(test1$opt), "hello world")
expect_equal(test1$rep, list(charToRaw("def"), raw(10), charToRaw("ghi")))

## Test raw(10) can be set to a single req field.
test$req <- raw(10)
expect_equal(length(test$req), 10)
expect_true(all(test$req == raw(10)))
#}

# Copyright 2013 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.bool <- function() {
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto", package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

a <- new(protobuf_unittest.TestAllTypes)
a$optional_bool <- TRUE
a$optional_bool <- FALSE
## Verify we can not set a protocol buffer bool to NA.
expect_error(a$optional_bool <- NA)

## Verify we can set character strings
a$repeated_bool <- c(TRUE, FALSE, TRUE)
expect_equal(length(unique(a$repeated_bool)), 2)

## Verify we can't set any garbage string to a bool.
expect_error(a$optional_bool <- "100")
expect_error(a$optional_bool <- "invalid")

## Verify we can't set any garbage string to a repeated bool.
expect_error(a$repeated_bool <-c("invalid", "invalid"))
expect_error(a$repeated_bool <-c("33-"))

## Verify we can set NA
expect_error(a$repeated_bool <- c(TRUE, FALSE, TRUE, NA))
#}

# Copyright 2011 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.enums <- function() {
ProtoFormat <- P("tutorial.Person")

## value(..) returns an EnumValueDescriptor object

expect_equal(name(value(ProtoFormat$PhoneType, index=1)), "MOBILE")
expect_equal(name(value(ProtoFormat$PhoneType, index=1), TRUE),
            "tutorial.Person.MOBILE")
expect_equal(number(value(ProtoFormat$PhoneType, index=1)), 0)
expect_true(inherits(enum_type(value(ProtoFormat$PhoneType, index=1)),
                   "EnumDescriptor"))
expect_true(inherits(asMessage(value(ProtoFormat$PhoneType, index=1)),
                   "Message"))

## Now check the '$' interfaces
expect_equal(name(value(ProtoFormat$PhoneType, index=1)),
            value(ProtoFormat$PhoneType, index=1)$name())
expect_equal(number(value(ProtoFormat$PhoneType, index=1)),
            value(ProtoFormat$PhoneType, index=1)$number())

expect_equal(name(value(ProtoFormat$PhoneType, index=2)), "HOME")


expect_equal(length(ProtoFormat$PhoneType), 3)
expect_true(has(ProtoFormat$PhoneType, "WORK"))
expect_true(!has(ProtoFormat$PhoneType, "NONEXISTANT"))

## Verify we can subset the EnumDescriptor class
expect_equal(ProtoFormat$PhoneType[["WORK"]], 2)
expect_equal(ProtoFormat$PhoneType[["MOBILE"]], 0)

## Verify that invalid indices are returned as NULL.
expect_true(is.null(value(ProtoFormat$PhoneType, index=900)))

## Verify that we import top-level enums from .proto files.
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto", package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}
expect_true(inherits(P("protobuf_unittest.ForeignEnum"), "EnumDescriptor"))
#}

# Copyright 2013 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.uint32 <- function() {
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

foo <- new(protobuf_unittest.TestAllTypes)
foo$optional_uint32 <- 2^32 - 1
foo$repeated_uint32 <- c(foo$optional_uint32, foo$optional_uint32)
expect_equal(as.character(foo$optional_uint32),
            "4294967295")
expect_equal(foo$optional_uint32,
            foo$repeated_uint32[[1]])
foo$add("repeated_uint32", c(2^32 - 1, 2^32 - 1))
expect_equal(length(unique(foo$repeated_uint32)), 1)

## fixed32 are a more efficient representation of uint32
foo$optional_fixed32 <- 2^32 - 1
foo$repeated_fixed32 <- c(foo$optional_fixed32, foo$optional_fixed32)
expect_equal(as.character(foo$optional_fixed32),
            "4294967295")
expect_equal(foo$optional_fixed32,
            foo$repeated_fixed32[[1]])
#}

# Copyright 2026 Troy Hernandez
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

readProtoFiles2("helloworld.proto",
                protoPath = system.file("proto", package = "RProtoBuf"))

fd <- fileDescriptor(P("rprotobuf.HelloWorldRequest"))
svc <- fd$HelloWorldService
expect_true(inherits(svc, "ServiceDescriptor"))

# method_count and length (issue #116: the .Call target was misnamed)
expect_equal(method_count(svc), 1L)
expect_equal(length(svc), 1L)

# method by index, 1-based as elsewhere (issue #116: routine compiled
# but unregistered)
m <- method(svc, index = 1L)
expect_true(inherits(m, "MethodDescriptor"))
expect_equal(name(m), "HelloWorld")

# method by name
m2 <- method(svc, name = "HelloWorld")
expect_true(inherits(m2, "MethodDescriptor"))
expect_equal(name(m2, full = TRUE), "rprotobuf.HelloWorldService.HelloWorld")

# downstream MethodDescriptor accessors work from either path
expect_equal(name(input_type(m)), "HelloWorldRequest")
expect_equal(name(output_type(m)), "HelloWorldResponse")

# out of range / unknown name give NULL, as the EnumDescriptor
# accessors do
expect_null(method(svc, index = 2L))
expect_null(method(svc, index = 0L))
expect_null(method(svc, name = "NoSuchMethod"))

# exactly one of index or name
expect_error(method(svc))
expect_error(method(svc, index = 1L, name = "HelloWorld"))

# Copyright 2013 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.int64 <- function() {
## Preserve option.
old.optval <- options("RProtoBuf.int64AsString")
on.exit(options(old.optval))

if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

if (.Machine$sizeof.longlong < 8) {
    exit_file("Can't test 64-bit int type on platform with sizeof(long long) < 8")
}
if (is.na(Rcpp:::capabilities()["long long"]) || Rcpp:::capabilities()["long long"] != TRUE) {
    exit_file("Can't test 64-bit int type without RCPP_LONG_LONG support.")
}

a <- new(protobuf_unittest.TestAllTypes)
a$repeated_int64 <- 1
# Now just test that we can use add to set int64 fields.
a$add("repeated_int64", 2:10)
expect_equal(length(a$repeated_int64), 10)

# Verify we can set character strings of large 64-bit ints
a$repeated_int64 <- c("9007199254740992", "9007199254740993")
expect_equal(length(a$repeated_int64), 2)

# Verify we can't set any garbage string to a repeated int64.
expect_error(a$repeated_int64 <-c("invalid", "invalid"))

a$optional_int64 <- 1
a$optional_int64 <- "2"
expect_equal(a$optional_int64, 2)
# Verify we can't set any garbage string to an optional int64.
expect_error(a$optional_int64 <- "invalid")

a <- protobuf_unittest.TestAllTypes$readASCII(file(system.file("tinytest", "data", "int64.ascii",
                                                               package="RProtoBuf")))
# And can read them in OK from an ASCII file.
expect_equal(length(a$repeated_int64), 2)

# By default, when they are read as numerics, only 1 unique value
expect_equal(length(unique(a$repeated_int64)), 1)

options("RProtoBuf.int64AsString" = TRUE)
# But we can see they are different if we treat them as strings.
expect_equal(length(unique(a$repeated_int64)), 2)
#}

## -*- indent-tabs-mode: nil; tab-width: 4; show-trailing-whitespace: t; c-indent-level: 4; c-basic-offset: 4; -*-
##  Copyright 2012 Google Inc. All Rights Reserved.
##  Author: Murray Stokely
##
##  This program is free software; you can redistribute it and/or
##  modify it under the terms of the GNU General Public License
##  as published by the Free Software Foundation; either version 2
##  of the License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

# this is executed before each test function
#.setUp <- function() {
if (!exists("tutorial.Person", "RProtoBuf:DescriptorPool")) {
    protofile <- system.file("proto", "addressbook.proto", package = "RProtoBuf")
    readProtoFiles(file = protofile)
}
#}

#test.arrayinputstream <- function() {
# Note: This class is experimental, and some parts of the design
# may need to change, but this works now.
stream <- ArrayInputStream(as.raw(0:10))
expect_equal(stream$ReadRaw(5), as.raw(0:4))

stringstream <- ArrayInputStream(as.raw(c(0x74, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x67)))
expect_equal(stringstream$ReadString(as.integer(7)), "testing")

intstream <- ArrayInputStream(as.raw(c(0x9e, 0xa7, 0x05)))
expect_equal(intstream$ReadVarint32(), 86942)
#}

#test.encoding <- function() {
if (!exists("protobuf_encoding_test.Test1",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("tinytest", "data", "encoding.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

## Encoding examples from https://protobuf.dev/docs/encoding
test1 <- new(protobuf_encoding_test.Test1)
test1$a <- 150
expect_identical(test1$serialize(NULL), as.raw(c(0x08,0x96,0x01)))

test2 <- new(protobuf_encoding_test.Test2)
test2$b <- "testing"
expect_identical(test2$serialize(NULL),
               as.raw(c(0x12, 0x07, 0x74, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x67)))

test3 <- new(protobuf_encoding_test.Test3)
test3$c$a <- 150
expect_identical(test3$serialize(NULL),
               as.raw(c(0x1a, 0x03, 0x08, 0x96, 0x01)))

test4 <- new(protobuf_encoding_test.Test4)
test4$d <- c(3, 270, 86942)
expect_identical(test4$serialize(NULL),
               as.raw(c(0x22, 0x06, 0x03, 0x8e, 0x02, 0x9e, 0xa7, 0x05)))
#}

#test.serialize <- function() {
person <- new(tutorial.Person)

expect_true(!person$isInitialized())
expect_error(serialize(person, NULL))
             #"Uninitialized object should fail to serialize.")
person$id <- 1
expect_error(serialize(person, NULL))
             #"Uninitialized object should fail to serialize.")
person$name <- "Murray"

expect_true(person$isInitialized())
expect_true(length(serialize(person, NULL)) > 1)
#}

library(RProtoBuf)

# this is executed before each test function
#.setUp <- function() {
if (! exists("protobuf_unittest_import.TestAllTypes", "RProtoBuf:DescriptorPool")) {
    unitest.proto.file <- system.file("tinytest", "data", "unittest.proto", package = "RProtoBuf")
    readProtoFiles(file = unitest.proto.file)
}
#}

#.tearDown <- function() {}

#test.import <- function() {
expect_true(exists("protobuf_unittest_import.ImportMessage", "RProtoBuf:DescriptorPool"),
            info = "exists( protobuf_unittest_import.ImportMessage ) ")
expect_true(exists( "protobuf_unittest_import.ImportEnum", "RProtoBuf:DescriptorPool" ),
            info = "exists( protobuf_unittest_import.ImportEnum ) ")
expect_equal(names(as.list( protobuf_unittest_import.ImportMessage)), "d",
             info = "names( protobuf_unittest_import.ImportMessage ) == 'd'")
import_enum <- as.list(protobuf_unittest_import.ImportEnum )
expect_true(all( c("IMPORT_FOO", "IMPORT_BAR", "IMPORT_BAZ") %in% names(import_enum)),
            info = "expected names for 'protobuf_unittest_import.ImportEnum'")
expect_equal(unlist(unname(import_enum)), 7:9,
             info = "expected values for 'protobuf_unittest_import.ImportEnum' ")
#}

#test.readProtoFile <- function() {
expect_true(exists( "protobuf_unittest.TestAllTypes", "RProtoBuf:DescriptorPool" ),
            info = "exists( protobuf_unittest_import.TestAllTypes ) ")
expect_true(exists( "protobuf_unittest.TestAllTypes.NestedMessage", "RProtoBuf:DescriptorPool" ),
            info = "exists( protobuf_unittest_import.TestAllTypes.NestedMessage ) " )
expect_true(exists( "protobuf_unittest.TestAllTypes.NestedEnum", "RProtoBuf:DescriptorPool" ),
            info = "exists( protobuf_unittest_import.TestAllTypes.NestedEnum ) " )
expect_true(exists( "protobuf_unittest.TestAllTypes.OptionalGroup", "RProtoBuf:DescriptorPool" ),
          info = "exists( protobuf_unittest.TestAllTypes.OptionalGroup ) " )
types <- c("int32", "int64", "uint32", "uint64", "sint32", "sint64",
           "fixed32", "fixed64", "sfixed32", "sfixed64", "float", "double",
           "bool", "string", "bytes" )
fieldnames <- names( as.list( protobuf_unittest.TestAllTypes ) )
prefixes <- c("optional", "default", "repeated" )
for (prefix in prefixes ) {
    for (type in types ) {
        expect_true(sprintf( "%s_%s", prefix, type ) %in% fieldnames,
                  info = sprintf( "%s_%s in field names", prefix, type))
    }
    expect_true(sprintf("%s_foreign_enum"  , prefix ) %in% fieldnames,
              info = sprintf( "%s_foreign_enum in field names"   , prefix ) )
    expect_true(sprintf("%s_import_enum"   , prefix ) %in% fieldnames,
              info = sprintf( "%s_import_enum in field names"    , prefix ) )
}
expect_true(exists( "protobuf_unittest.ForeignMessage", "RProtoBuf:DescriptorPool"),
          info = "exists( protobuf_unittest.ForeignMessage ) " )
expect_equal(names(as.list(protobuf_unittest.ForeignMessage)), "c")
expect_true(exists( "protobuf_unittest.ForeignEnum", "RProtoBuf:DescriptorPool"),
          info = "exists( protobuf_unittest.ForeignEnum ) ")
foreign_enum <- as.list( protobuf_unittest.ForeignEnum )
expect_equal(length(foreign_enum), 3L,
            info = "length( protobuf_unittest.ForeignEnum ) == 3" )
expect_true(all(c("FOREIGN_FOO", "FOREIGN_BAR", "FOREIGN_BAZ") %in% names( foreign_enum )),
          info = "expected names for enum `protobuf_unittest.ForeignEnum`" )
expect_equal( unlist(unname(as.list(protobuf_unittest.ForeignEnum))), 4:6,
            info = "expected values for enum `protobuf_unittest.ForeignEnum`" )
#}

## Early versions of RProtoBuf did not support repeated messages properly.
#test.repeatedFields <- function() {
test <- new(protobuf_unittest.TestAllTypes)
test$add("repeated_int32", c(1:5))
expect_equal(test$repeated_int32, c(1:5))

test$repeated_int32 <- 1
expect_equal(test$repeated_int32, 1)

## Prior to RProtoBuf v0.2.5, this was not handled properly.
test.2 <- new(protobuf_unittest.TestAllTypes,
              repeated_string=c("foo", "bar"))
expect_equal(test.2$repeated_string, c("foo", "bar"))

## Versions of RProtoBuf <= 0.2.5 had non-deterministic behavior due to a
## memory management bug when setting a repeated field to a
## non-vector type (e.g. a Message).
test$repeated_foreign_message <- list(new(protobuf_unittest.ForeignMessage,
                                          c = 1),
                                      new(protobuf_unittest.ForeignMessage,
                                          c = 2))
expect_equal(length(test$repeated_foreign_message), 2)
test$repeated_foreign_message <- new(protobuf_unittest.ForeignMessage,
                                     c = 3)
expect_equal(length(test$repeated_foreign_message), 1)
#}

#test.repeated.bools <- function() {
test <- new(protobuf_unittest.TestAllTypes)
test$add("repeated_bool", c(TRUE, FALSE))
expect_equal(test$repeated_bool, c(TRUE, FALSE))

test$add("repeated_bool", as.integer(c(TRUE, FALSE)))
test$add("repeated_bool", as.numeric(c(TRUE, FALSE)))

expect_equal(test$repeated_bool, rep(c(TRUE, FALSE), 3))

## Verify that we don't silently cast NA into TRUE or FALSE.
expect_error(test$add("repeated_bool", c(TRUE, FALSE, NA)))
#}

## Versions of RProtoBuf <= 0.2.5 would terminate the R instance with unhandled Rcpp exceptions.
#test.invalidAssignments <- function() {
test <- new(protobuf_unittest.TestAllTypes)
expect_error(test$optional_int32 <- 1:10)
#}

## Versions of RProtoBuf <= 0.2.5 did not distinguish between non-existant
## and not-set fields with has().
#test.has <- function() {
test <- new(protobuf_unittest.TestAllTypes)
test$add("repeated_int32", c(1:5))
expect_true( has(test, "repeated_int32"))
expect_true( test$has("repeated_int32"))
expect_true( is.null(test$has("nonexistant")))
expect_true( !test$has("optional_int32"))
#}

# Copyright 2012 Google Inc.
# Author: Murray Stokely
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

# this is executed before each test function
#.setUp <- function(){
if( !exists("tutorial.Person", "RProtoBuf:DescriptorPool")) {
    protofile <- system.file("proto", "addressbook.proto", package = "RProtoBuf" )
    readProtoFiles(file = protofile)
}
#}


#test.FieldDescriptor.class <- function() {

Person <- P( "tutorial.Person")

## field descriptor object
expect_true(!is.null(Person$email), info="non-null email")

## debug string
expect_true(nchar(as.character( Person$email )) > 1, info="non-empty email")

## default values
expect_true(!has_default_value(Person$id), info="no default for id")
expect_true(has_default_value(Person$PhoneNumber$type), info="default for phone")
expect_equal(default_value(Person$PhoneNumber$type), 1, info="check default for phone type")
expect_equal(default_value(Person$id), 0, info="check default for person id")

## Get the types of field descriptors
expect_equal(type(Person$id), TYPE_INT32, info="type int32")
expect_equal(type(Person$id, TRUE), "TYPE_INT32", info="type int32 as char")
expect_equal(cpp_type(Person$email), CPPTYPE_STRING, info="cpptype string")
expect_equal(cpp_type(Person$email, TRUE), "CPPTYPE_STRING", info="cpptype string as char")

## Test if a field is optional
expect_true(is_required(Person$id), info="id is required")
expect_true(!is_repeated(Person$id), info="id is not repeated")

expect_true(!is_required(Person$email), info="email is not required")
expect_true(!is_repeated(Person$email), info="email is not repeated")

## Return the class of a message field
expect_true(inherits(message_type(Person$phone), "Descriptor"), info="inherits Descriptor")

## Containing type of a field is the message descriptor
expect_true(inherits(Person$id$containing_type(), "Descriptor"),
            info="inherits containing type Descriptor")

## No containing type for the top-level message descriptor.
expect_true(is.null(Person$containing_type()), info="no containing typw")

#}

library(RProtoBuf)

## created as
##   demo(addressbook)  ## creates 'book'
##   serialize(book, "/tmp/addressbook.pb")

# this is executed before each test function -- but somehow does not work
#.setUp <- function() {
file <- system.file( "examples", "addressbook.pb", package = "RProtoBuf" )
book <- read( tutorial.AddressBook, file )
assign("book", book, globalenv())
#}

#test.size <- function() {
expect_equal(book$size("person"),        2,   info="Number of persons")
expect_equal(bytesize(book),             125, info="Bytes in book")
expect_equal(bytesize(book$person[[1]]), 60,  info="Bytes of first person message")
#}

#test.fileDescriptor <- function() {
expect_equal(name(book$fileDescriptor()), "addressbook.proto")
#}

#test.personOne <- function() {
expect_equal(book$person[[1]]$name,              "Romain Francois",   info="First person name")
expect_equal(book$person[[2]]$name,              "Dirk Eddelbuettel", info="First person name")
expect_equal(book$person[[1]]$id,                1,   		 info="First person id")
expect_equal(book$person[[2]]$id,                2,			 info="First person id")
expect_equal(book$person[[1]]$email,             "romain@r-enthusiasts.com",   info="First person email")
expect_equal(book$person[[2]]$email,             "edd@debian.org",    info="Second person email")
expect_equal(length(book$person[[1]]$phone),     1,                   info="First person phones")
expect_equal(length(book$person[[2]]$phone),     2,                   info="Second person phones")
expect_equal(book$person[[1]]$phone[[1]]$number, "+33(0)...",         info="First person phone number")
expect_equal(book$person[[2]]$phone[[1]]$number, "+01...",            info="Second person phone number")
expect_equal(book$person[[2]]$phone[[1]]$type,   0,                   info="Second person phone number")
#}

#test.ascii <- function() {
## Output in text format to a temporary file
out.file <- tempfile()
writeLines(book$toTextFormat(), file(out.file))

## Verify that we can read back in the message from a text file.
book2 <- readASCII( tutorial.AddressBook, file(out.file, "rb"))
expect_equal(as.character(book), as.character(book2) )

## Verify that we can read in messages from unopened connections.
book3 <- readASCII( tutorial.AddressBook, file(out.file))
expect_equal(as.character(book), as.character(book3) )

## Verify that we get an exception if we try to read from a text connection.
## (better than silently getting an empty proto.)
book4 <- expect_error( readASCII( tutorial.AddressBook, file(out.file, "rt")))

## # Test does not work on windows because of chmod
## if(!grepl("mingw", R.Version()$platform)){
##     # Verify that we get an exception if the file is not readable.
##     old.mode <- file.info(out.file)[["mode"]]
##     Sys.chmod(out.file, "0000")
##     book5 <- checkException( readASCII( tutorial.AddressBook, file(out.file, "rb")))
##     # Set the permissions back to ensure the file is cleaned up properly.
##     Sys.chmod(out.file, old.mode)
## }

## Verify that we get an exception if the file is not parseable.
out.file2 <- tempfile()
writeLines("jibberish", file(out.file2))
book6 <- expect_error( readASCII( tutorial.AddressBook, file(out.file2)))

## Verify that we get an exception if we forget the file() and thus treat the
## path as a protobuf string.
expect_error( readASCII( tutorial.AddressBook, out.file2))

incomplete.msg <- new(tutorial.Person, name="Murray", email="murray@stokely.org")
tmp.file <- tempfile()
writeLines(incomplete.msg$toTextFormat(), file(tmp.file))

expect_true(!incomplete.msg$isInitialized())
## Verify we normally get an exception if we try to read an incomplete ASCII protocol buffer
expect_error(tutorial.Person$readASCII(file(tmp.file)))

## Verify we can however read it if we set partial=TRUE.
new.msg <- tutorial.Person$readASCII(file(tmp.file), TRUE)
expect_equal(incomplete.msg$name, new.msg$name)
#}

## Ensure we do not try to run JSON tests on older ProtoBuf versions
## Support starts with version 3.

isProto2 <- (RProtoBuf:::getProtobufLibVersion() < 3000000)
if (isProto2) exit_file("Skipping JSON tests with ProtoBuf2")

# These tests are similar to the group of tests covered with test.ascii above.
#test.json <- function() {
## Output in JSON format to a temporary file
out.file <- tempfile()
writeLines( book$toJSON(), file(out.file))

## Verify that we can read back in the message from a text file.
book2 <- readJSON( tutorial.AddressBook, file(out.file, "rb"))
expect_equal(as.character(book), as.character(book2) )

## Verify that we can read in messages from unopened connections.
book3 <- readJSON( tutorial.AddressBook, file(out.file))
expect_equal(as.character(book), as.character(book3) )

## Verify that we get an exception if we try to read from a text connection.
## (better than silently getting an empty proto.)
book4 <- expect_error( readJSON( tutorial.AddressBook, file(out.file, "rt")))

## Verify that we get an exception if the file is not parseable.
out.file2 <- tempfile()
writeLines("jibberish", file(out.file2))
book6 <- expect_error( readJSON( tutorial.AddressBook, file(out.file2)))

## Verify that we get an exception if we forget the file() and thus treat the
## path as a protobuf string.
expect_error( readJSON( tutorial.AddressBook, out.file2))
#}

library(RProtoBuf)

## created as
##   demo(addressbook)  ## creates 'book'
##   serialize(book, "/tmp/addressbook.pb")

# this is executed before each test function -- but somehow does not work
#.setUp <- function() {
file <- system.file( "examples", "addressbook.pb", package = "RProtoBuf" )
book <- read( tutorial.AddressBook, file )
assign("book", book, globalenv())
#}

#test.size <- function() {
expect_equal(book$size("person"),        2,   info="Number of persons")
expect_equal(bytesize(book),             125, info="Bytes in book")
expect_equal(bytesize(book$person[[1]]), 60,  info="Bytes of first person message")
#}

#test.fileDescriptor <- function() {
expect_equal(name(book$fileDescriptor()), "addressbook.proto")
#}

#test.personOne <- function() {
expect_equal(book$person[[1]]$name,              "Romain Francois",   info="First person name")
expect_equal(book$person[[2]]$name,              "Dirk Eddelbuettel", info="First person name")
expect_equal(book$person[[1]]$id,                1,   		 info="First person id")
expect_equal(book$person[[2]]$id,                2,			 info="First person id")
expect_equal(book$person[[1]]$email,             "romain@r-enthusiasts.com",   info="First person email")
expect_equal(book$person[[2]]$email,             "edd@debian.org",    info="Second person email")
expect_equal(length(book$person[[1]]$phone),     1,                   info="First person phones")
expect_equal(length(book$person[[2]]$phone),     2,                   info="Second person phones")
expect_equal(book$person[[1]]$phone[[1]]$number, "+33(0)...",         info="First person phone number")
expect_equal(book$person[[2]]$phone[[1]]$number, "+01...",            info="Second person phone number")
expect_equal(book$person[[2]]$phone[[1]]$type,   0,                   info="Second person phone number")
#}

#test.ascii <- function() {
## Output in text format to a temporary file
out.file <- tempfile()
writeLines( as.character(book), file(out.file))

## Verify that we can read back in the message from a text file.
book2 <- readASCII( tutorial.AddressBook, file(out.file, "rb"))
expect_equal(as.character(book), as.character(book2) )

## Verify that we can read in messages from unopened connections.
book3 <- readASCII( tutorial.AddressBook, file(out.file))
expect_equal(as.character(book), as.character(book3) )

## Verify that we get an exception if we try to read from a text connection.
## (better than silently getting an empty proto.)
book4 <- expect_error( readASCII( tutorial.AddressBook, file(out.file, "rt")))

## # Test does not work on windows because of chmod
## if(!grepl("mingw", R.Version()$platform)){
##     # Verify that we get an exception if the file is not readable.
##     old.mode <- file.info(out.file)[["mode"]]
##     Sys.chmod(out.file, "0000")
##     book5 <- checkException( readASCII( tutorial.AddressBook, file(out.file, "rb")))
##     # Set the permissions back to ensure the file is cleaned up properly.
##     Sys.chmod(out.file, old.mode)
## }

## Verify that we get an exception if the file is not parseable.
out.file2 <- tempfile()
writeLines("jibberish", file(out.file2))
book6 <- expect_error( readASCII( tutorial.AddressBook, file(out.file2)))

## Verify that we get an exception if we forget the file() and thus treat the
## path as a protobuf string.
expect_error( readASCII( tutorial.AddressBook, out.file2))

incomplete.msg <- new(tutorial.Person, name="Murray", email="murray@stokely.org")
tmp.file <- tempfile()
writeLines(as.character(incomplete.msg), file(tmp.file))

expect_true(!incomplete.msg$isInitialized())
## Verify we normally get an exception if we try to read an incomplete ASCII protocol buffer
expect_error(tutorial.Person$readASCII(file(tmp.file)))

## Verify we can however read it if we set partial=TRUE.
new.msg <- tutorial.Person$readASCII(file(tmp.file), TRUE)
expect_equal(incomplete.msg$name, new.msg$name)
#}

## Ensure we do not try to run JSON tests on older ProtoBuf versions
## Support starts with version 3.

isProto2 <- (RProtoBuf:::getProtobufLibVersion() < 3000000)
if (isProto2) exit_file("Skipping JSON tests with ProtoBuf2")

# These tests are similar to the group of tests covered with test.ascii above.
#test.json <- function() {
## Output in JSON format to a temporary file
out.file <- tempfile()
writeLines( book$toJSON(), file(out.file))

## Verify that we can read back in the message from a text file.
book2 <- readJSON( tutorial.AddressBook, file(out.file, "rb"))
expect_equal(as.character(book), as.character(book2) )

## Verify that we can read in messages from unopened connections.
book3 <- readJSON( tutorial.AddressBook, file(out.file))
expect_equal(as.character(book), as.character(book3) )

## Verify that we get an exception if we try to read from a text connection.
## (better than silently getting an empty proto.)
book4 <- expect_error( readJSON( tutorial.AddressBook, file(out.file, "rt")))

## Verify that we get an exception if the file is not parseable.
out.file2 <- tempfile()
writeLines("jibberish", file(out.file2))
book6 <- expect_error( readJSON( tutorial.AddressBook, file(out.file2)))

## Verify that we get an exception if we forget the file() and thus treat the
## path as a protobuf string.
expect_error( readJSON( tutorial.AddressBook, out.file2))
#}

# Copyright 2012 Google Inc.
# Author: Murray Stokely
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

# this is executed before each test function
#.setUp <- function(){
if( !exists("tutorial.Person", "RProtoBuf:DescriptorPool")) {
    protofile <- system.file("proto", "addressbook.proto", package = "RProtoBuf" )
    readProtoFiles(file = protofile)
}
#}


#test.FieldDescriptor.class <- function() {

Person <- P( "tutorial.Person")

## field descriptor object
expect_true(!is.null(Person$email), info="non-null email")

## debug string
expect_true(nchar(as.character( Person$email )) > 1, info="non-empty email")

## default values
expect_true(!has_default_value(Person$id), info="no default for id")
expect_true(has_default_value(Person$PhoneNumber$type), info="default for phone")
expect_equal(default_value(Person$PhoneNumber$type), 1, info="check default for phone type")
expect_equal(default_value(Person$id), 0, info="check default for person id")

## Get the types of field descriptors
expect_equal(type(Person$id), TYPE_INT32, info="type int32")
expect_equal(type(Person$id, TRUE), "TYPE_INT32", info="type int32 as char")
expect_equal(cpp_type(Person$email), CPPTYPE_STRING, info="cpptype string")
expect_equal(cpp_type(Person$email, TRUE), "CPPTYPE_STRING", info="cpptype string as char")

## Get the label of a field descriptor
expect_equal(label(Person$id), LABEL_REQUIRED, info="label required")
expect_equal(label(Person$id, TRUE), "LABEL_REQUIRED", info="label required as char")
expect_equal(label(Person$email), LABEL_OPTIONAL, info="label optional")
expect_equal(label(Person$email, TRUE), "LABEL_OPTIONAL", info="label optional as char")

## Test if a field is optional
expect_true(is_required(Person$id), info="id is required")
expect_true(!is_optional(Person$id), info="id is not optional")
expect_true(!is_repeated(Person$id), info="id is not repeated")

expect_true(!is_required(Person$email), info="email is not required")
expect_true(is_optional(Person$email), info="email is optional")
expect_true(!is_repeated(Person$email), info="email is not repeated")

## Return the class of a message field
expect_true(inherits(message_type(Person$phone), "Descriptor"), info="inherits Descriptor")

## Containing type of a field is the message descriptor
expect_true(inherits(Person$id$containing_type(), "Descriptor"),
            info="inherits containing type Descriptor")

## No containing type for the top-level message descriptor.
expect_true(is.null(Person$containing_type()), info="no containing typw")

#}

setGeneric( "has", function( object, name, ... ){
	standardGeneric( "has" )
} )
._has_message <- function( object, name, ...){
	if (!.Call(Message__field_exists, object@pointer, name)) {
		return(NULL)
	} else {
		return(.Call(Message__has_field, object@pointer, name))
	}
}
._has_enum_name <- function( object, name, ...){
    return(.Call(has_enum_name, object@pointer, name))
}

setMethod( "has", "Message", ._has_message )
setMethod( "has", "EnumDescriptor", ._has_enum_name)

if (requireNamespace("tinytest", quietly=TRUE) &&
    utils::packageVersion("tinytest") >= "1.0.0") {

    ## Set a seed to make the test deterministic
    set.seed(42)

    ## R makes us to this
    Sys.setenv("R_TESTS"="")

    ## there are several more granular ways to test files in a tinytest directory,
    ## see its package vignette; tests can also run once the package is installed
    ## using the same command `test_package(pkgName)`, or by director or file
    tinytest::test_package("RProtoBuf")
}

# Copyright 2014 Google Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

library(RProtoBuf)

#test.descriptor <- function() {
expect_true(inherits(tutorial.Person$email, "FieldDescriptor"))
expect_true(inherits(tutorial.Person[["email"]], "FieldDescriptor"))
expect_true(inherits(tutorial.Person[[1]], "FieldDescriptor"))
## Currently, the $ extractor for fields returns NULL for invalid reference
## stop() probably better.
expect_equal(tutorial.Person$nonexistantfoobarbaz, NULL)
## But we raise an error with invalid indices with '[['.
expect_error(tutorial.Person[[909]])
expect_error(tutorial.Person[["nonexistantfoobarbaz"]])
#}

#!/bin/sh

#write some PB data with R
Rscript writemsg.R

#compile proto file
cp -f ../proto/rexp.proto .
protoc rexp.proto --python_out=.
rm rexp.proto

#read with python and print
python readmsg.py

#cleanup tmp files
rm rexp_pb2.py rexp_pb2.pyc
rm *.msg

\documentclass[11pt]{article}
%\VignetteIndexEntry{RProtoBuf-intro}
%\VignetteEngine{highlight::highlight}
%\VignetteKeywords{RProtoBuf, Protocol Buffers, package}
%\VignetteDepends{RProtoBuf}

\usepackage{url}
\usepackage{color}
\usepackage[colorlinks]{hyperref}
\definecolor{link}{rgb}{0,0,0.3}
\hypersetup{
    colorlinks,%
    citecolor=link,%
    filecolor=link,%
    linkcolor=link,%
    urlcolor=link
}
\setlength{\oddsidemargin}{0pt}
\setlength{\textwidth}{17cm} % uh-oh, I use letter :)
\setcounter{tocdepth}{2}

\usepackage{microtype}                     %% cf http://www.khirevich.com/latex/microtype/
\usepackage[T1]{fontenc}		   %% cf http://www.khirevich.com/latex/font/
\usepackage[bitstream-charter]{mathdesign} %% cf http://www.khirevich.com/latex/font/

<<echo=FALSE,print=FALSE>>=
library("RProtoBuf")
options("width"=65)
rpb.version <- packageDescription("RProtoBuf")$Version
prettyDate <- format(Sys.Date(), "%B %e, %Y")
@
% closing $ needed here


\author{Romain Fran\c{c}ois \and Dirk Eddelbuettel \and Murray Stokely}
\title{RProtoBuf: An R API for Protocol Buffers}
\date{Version \Sexpr{rpb.version} as of \Sexpr{prettyDate}}
\begin{document}
\maketitle

\abstract{
  \noindent
  \textsl{Protocol Buffers} is a software project by Google that is used
  extensively internally and also released under an Open Source license.
  It provides a way of encoding structured data in an efficient yet
  extensible format. Google formally supports APIs for C++, Java and Python.

  This vignette describes version \Sexpr{rpb.version} of the
  \texttt{RProtoBuf} package which brings support for protocol buffer
  messages to R.

}


\tableofcontents

\section{Protocol Buffers}

Protocol buffers are a language-neutral, platform-neutral, extensible
way of serializing structured data for use in communications
protocols, data storage, and more.

Protocol Buffers offer key features such as an efficient data interchange
format that is both language- and operating system-agnostic yet uses a
lightweight and highly performant encoding, object serialization and
de-serialization as well data and configuration management. Protocol
buffers are also forward compatible: updates to the \texttt{proto}
files do not break programs built against the previous specification.

While benchmarks are not available, Google states on the project page that in
comparison to XML, protocol buffers are at the same time \textsl{simpler},
between three to ten times \textsl{smaller}, between twenty and one hundred
times \textsl{faster}, as well as less ambiguous and easier to program.

The protocol buffers code is released under an open-source (BSD) license. The
protocol buffer project (\url{http://code.google.com/p/protobuf/})
contains a C++ library and a set of runtime libraries and compilers for
C++, Java and Python.

With these languages, the workflow follows standard practice of so-called
Interface Description Languages (IDL)
(c.f. \href{http://en.wikipedia.org/wiki/Interface_description_language}{Wikipedia
  on IDL}).  This consists of compiling a protocol buffer description file
(ending in \texttt{.proto}) into language specific classes that can be used
to create, read, write and manipulate protocol buffer messages. In other
words, given the 'proto' description file, code is automatically generated
for the chosen target language(s). The project page contains a tutorial for
each of these officially supported languages:
\url{http://code.google.com/apis/protocolbuffers/docs/tutorials.html}

Besides the officially supported C++, Java and Python implementations, several projects have been
created to support protocol buffers for many languages. The list of known
languages to support protocol buffers is compiled as part of the
project page: \url{http://code.google.com/p/protobuf/wiki/ThirdPartyAddOns}

The protocol buffer project page contains a comprehensive
description of the language: \url{http://code.google.com/apis/protocolbuffers/docs/proto.html}

\section{Basic use: Protocol Buffers and R}

This section describes how to use the R API to create and manipulate
protocol buffer messages in R, and how to read and write the
binary \emph{payload} of the messages to files and arbitrary binary
R connections.

\subsection{Importing proto files dynamically}

In contrast to the other languages (Java, C++, Python) that are officially
supported by Google, the implementation used by the \texttt{RProtoBuf}
package does not rely on the \texttt{protoc} compiler (with the exception of
the two functions discussed in the previous section). This means that no
initial step of statically compiling the proto file into C++ code that is
then accessed by R code is necessary. Instead, \texttt{proto} files are
parsed and processed \textsl{at runtime} by the protobuf C++ library---which
is much more appropriate for a dynamic language.

The \texttt{readProtoFiles} function allows importing \texttt{proto}
files in several ways.

<<>>=
args( readProtoFiles )
@

Using the \texttt{file} argument, one can specify one or several file
paths that ought to be proto files.

<<>>=
proto.dir <- system.file( "proto", package = "RProtoBuf" )
proto.file <- file.path( proto.dir, "addressbook.proto" )
<<eval=FALSE>>=
readProtoFiles( proto.file )
@

With the \texttt{dir} argument, which is
ignored if the \texttt{file} is supplied, all files matching the
\texttt{.proto} extension will be imported.

<<>>=
dir( proto.dir, pattern = "\\.proto$", full.names = TRUE )
<<eval=FALSE>>=
readProtoFiles( dir = proto.dir )
@

Finally, with the
\texttt{package} argument (ignored if \texttt{file} or
\texttt{dir} is supplied), the function will import all \texttt{.proto}
files that are located in the \texttt{proto} sub-directory of the given
package. A typical use for this argument is in the \texttt{.onLoad}
function of a package.

<<eval=FALSE>>=
readProtoFiles( package = "RProtoBuf" )
@

Once the proto files are imported, all message descriptors are
are available in the R search path in the \texttt{RProtoBuf:DescriptorPool}
special environment. The underlying mechanism used here is
described in more detail in section~\ref{sec-lookup}.

<<>>=
ls( "RProtoBuf:DescriptorPool" )
@


\subsection{Creating a message}

The objects contained in the special environment are
descriptors for their associated message types. Descriptors will be
discussed in detail in another part of this document, but for the
purpose of this section, descriptors are just used with the \texttt{new}
function to create messages.

<<>>=
p <- new( tutorial.Person, name = "Romain", id = 1 )
@

\subsection{Access and modify fields of a message}

Once the message is created, its fields can be queried
and modified using the dollar operator of R, making protocol
buffer messages seem like lists.

<<>>=
p$name
p$id
p$email <- "francoisromain@free.fr"
@

However, as opposed to R lists, no partial matching is performed
and the name must be given entirely.

The \verb|[[| operator can also be used to query and set fields
of a mesages, supplying either their name or their tag number :

<<>>=
p[["name"]] <- "Romain Francois"
p[[ 2 ]] <- 3
p[[ "email" ]]
@

Protocol buffers include a 64-bit integer type, but R lacks native
64-bit integer support.  A workaround is available and described in
Section~\ref{sec:int64} for working with large integer values.

% TODO(mstokely): Document extensions here.
% There are none in addressbook.proto though.

\subsection{Display messages}

Protocol buffer messages and descriptors implement \texttt{show}
methods that provide basic information about the message :

<<>>=
p
@

For additional information, such as for debugging purposes,
the \texttt{as.character} method provides a more complete ASCII
representation of the contents of a message.

<<>>=
writeLines( as.character( p ) )
@

\subsection{Serializing messages}

However, the main focus of protocol buffer messages is
efficiency. Therefore, messages are transported as a sequence
of bytes. The \texttt{serialize} method is implemented for
protocol buffer messages to serialize a message into the sequence of
bytes (raw vector in R speech) that represents the message.

<<>>=
serialize( p, NULL )
@

The same method can also be used to serialize messages to files :

<<>>=
tf1 <- tempfile()
tf1
serialize( p, tf1 )
readBin( tf1, raw(0), 500 )
@

Or to arbitrary binary connections:

<<>>=
tf2 <- tempfile()
con <- file( tf2, open = "wb" )
serialize( p, con )
close( con )
readBin( tf2, raw(0), 500 )
@

\texttt{serialize} can also be used in a more traditionnal
object oriented fashion using the dollar operator :

<<>>=
# serialize to a file
p$serialize( tf1 )
# serialize to a binary connection
con <- file( tf2, open = "wb" )
p$serialize( con )
close( con )
@


\subsection{Parsing messages}

The \texttt{RProtoBuf} package defines the \texttt{read}
function to read messages from files, raw vector (the message payload)
and arbitrary binary connections.

<<>>=
args( read )
@


The binary representation of the message (often called the payload)
does not contain information that can be used to dynamically
infer the message type, so we have to provide this information
to the \texttt{read} function in the form of a descriptor :

<<>>=
message <- read( tutorial.Person, tf1 )
writeLines( as.character( message ) )
@

The \texttt{input} argument of \texttt{read} can also be a binary
readable R connection, such as a binary file connection:

<<>>=
con <- file( tf2, open = "rb" )
message <- read( tutorial.Person, con )
close( con )
writeLines( as.character( message ) )
@

Finally, the payload of the message can be used :

<<>>=
# reading the raw vector payload of the message
payload <- readBin( tf1, raw(0), 5000 )
message <- read( tutorial.Person, payload )
@


\texttt{read} can also be used as a pseudo method of the descriptor
object :

<<>>=
# reading from a file
message <- tutorial.Person$read( tf1 )
# reading from a binary connection
con <- file( tf2, open = "rb" )
message <- tutorial.Person$read( con )
close( con )
# read from the payload
message <- tutorial.Person$read( payload )
@

\section{Classes, Methods and Pseudo Methods}

The \texttt{RProtoBuf} package uses the S4 system to store
information about descriptors and messages, but the information stored
in the R object is very minimal and mainly consists of an external
pointer to a C++ variable that is managed by the \texttt{proto} C++
library.

<<>>=
str( p )
@

Using the S4 system allows the \texttt{RProtoBuf} package to dispatch
methods that are not generic in the S3 sense, such as \texttt{new}
and \texttt{serialize}.

The \texttt{RProtoBuf} package combines the \emph{R typical} dispatch
of the form \verb|method( object, arguments)| and the more traditional
object oriented notation \verb|object$method(arguments)|.

\subsection{messages}

Messages are represented in R using the \texttt{Message}
S4 class. The class contains the slots \texttt{pointer} and \texttt{type} as
described on the Table~\ref{Message-class-table}.

\begin{table}[h]
\centering
\begin{tabular}{|cp{10cm}|}
\hline
\textbf{slot} & \textbf{description} \\
\hline
\texttt{pointer} & external pointer to the \texttt{Message} object of the C++ proto library. Documentation for the
\texttt{Message} class is available from the protocol buffer project page:
\url{http://code.google.com/apis/protocolbuffers/docs/reference/cpp/google.protobuf.message.html#Message} \\
\hline
\texttt{type} & fully qualified path of the message. For example a \texttt{Person} message
has its \texttt{type} slot set to \texttt{tutorial.Person} \\
\hline
\end{tabular}
\caption{\label{Message-class-table}Description of slots for the \texttt{Message} S4 class}
\end{table}

Although the \texttt{RProtoBuf} package uses
the S4 system, the \verb|@| operator is very rarely used. Fields of the
message are retrieved or modified using the \verb|$| or \verb|[[| operators
as seen on the previous section, and pseudo-methods can also
be called using the \verb|$| operator. Table~\ref{Message-methods-table}
describes the methods defined for the \texttt{Message} class :

\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|ccp{8cm}|}
\hline
\textbf{method} & \textbf{section} & \textbf{description} \\
\hline
\hline
\texttt{has} & \ref{Message-method-has} & Indicates if a message has a given field.   \\
\texttt{clone} &  \ref{Message-method-clone} & Creates a clone of the message \\
\texttt{isInitialized} & \ref{Message-method-isInitialized} & Indicates if a message has all its required fields set\\
\texttt{serialize} & \ref{Message-method-serialize} & serialize a message to a file or a binary connection or retrieve the message payload as a raw vector\\
\texttt{clear} & \ref{Message-method-clear} & Clear one or several fields of a message, or the entire message\\
\texttt{size} & \ref{Message-method-size} & The number of elements in a message field\\
\texttt{bytesize} & \ref{Message-method-bytesize} & The number of bytes the message would take once serialized\\
\hline
\texttt{swap} & \ref{Message-method-swap} & swap elements of a repeated field of a message\\
\texttt{set} & \ref{Message-method-set} & set elements of a repeated field\\
\texttt{fetch} & \ref{Message-method-fetch} & fetch elements of a repeated field\\
\texttt{setExtension} & \ref{Message-method-setExtension} & set an extension of a message\\
\texttt{getExtension} & \ref{Message-method-getExtension} & get the value of an extension of a message\\
\texttt{add} & \ref{Message-method-add} & add elements to a repeated field \\
\hline
\texttt{str} & \ref{Message-method-str} & the R structure of the message\\
\texttt{as.character} & \ref{Message-method-ascharacter} & character representation of a message\\
\texttt{toString} & \ref{Message-method-toString} & character representation of a message (same as \texttt{as.character}) \\
\texttt{as.list} & \ref{Message-method-aslist} & converts message to a
named R list\\
\texttt{update} & \ref{Message-method-update} & updates several fields of a message at once\\
\texttt{descriptor} & \ref{Message-method-descriptor} & get the descriptor of the message type of this message\\
\texttt{fileDescriptor} & \ref{Message-method-fileDescriptor} & get the file descriptor
of this message's descriptor\\
\hline
\end{tabular}
\end{small}
\caption{\label{Message-methods-table}Description of methods for the \texttt{Message} S4 class}
\end{table}

\subsubsection{Retrieve fields}
\label{Message-method-getfield}

The \verb|$| and \verb|[[| operators allow extraction of a field data.

<<keep.source=T>>=
message <- new( tutorial.Person,
	name = "foo", email = "foo@bar.com", id = 2,
	phone = list(
		new( tutorial.Person.PhoneNumber, number = "+33(0)...", type = "HOME" ),
		new( tutorial.Person.PhoneNumber, number = "+33(0)###", type = "MOBILE" )
	) )
message$name
message$email
message[[ "phone" ]]
# using the tag number
message[[ 2 ]] # id
@

Neither \verb|$| nor \verb|[[| support partial matching of names. The
\verb|$| is also used to call methods on the message, and the
\verb|[[| operator can use the tag number of the field.

Table~\ref{table-get-types} details correspondance between
the field type and the type of data that is retrieved by \verb|$| and
\verb|[[|.

\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|c|p{5cm}p{5cm}|}
\hline
field type & R type (non repeated) & R type (repeated) \\
\hline
\hline
double	& \texttt{double} vector & \texttt{double} vector \\
float	& \texttt{double} vector & \texttt{double} vector \\
\hline
uint32	  & \texttt{double} vector & \texttt{double} vector \\
fixed32	  & \texttt{double} vector & \texttt{double} vector \\
\hline
int32	  & \texttt{integer} vector & \texttt{integer} vector \\
sint32	  & \texttt{integer} vector & \texttt{integer} vector \\
sfixed32  & \texttt{integer} vector & \texttt{integer} vector \\
\hline
int64	  & \texttt{integer} or \texttt{character}
vector \footnotemark & \texttt{integer} or \texttt{character} vector \\
uint64	  & \texttt{integer} or \texttt{character} vector & \texttt{integer} or \texttt{character} vector \\
sint64	  & \texttt{integer} or \texttt{character} vector & \texttt{integer} or \texttt{character} vector \\
fixed64	  & \texttt{integer} or \texttt{character} vector & \texttt{integer} or \texttt{character} vector \\
sfixed64  & \texttt{integer} or \texttt{character} vector & \texttt{integer} or \texttt{character} vector \\
\hline
bool	& \texttt{logical} vector & \texttt{logical} vector \\
\hline
string	& \texttt{character} vector & \texttt{character} vector \\
bytes	& \texttt{character} vector & \texttt{character} vector \\
\hline
enum & \texttt{integer} vector & \texttt{integer} vector \\
\hline
message & \texttt{S4} object of class \texttt{Message} & \texttt{list} of \texttt{S4} objects of class \texttt{Message} \\
\hline
\end{tabular}
\end{small}
\caption{\label{table-get-types}Correspondance between field type and
  R type retrieved by the extractors. \footnotesize{1. R lacks native
  64-bit integers, so the \texttt{RProtoBuf.int64AsString} option is
  available to return large integers as characters to avoid losing
  precision.  This option is described in Section~\ref{sec:int64}}.  R
  also lacks an unsigned integer type.}
\end{table}

\subsubsection{Modify fields}
\label{Message-method-setfield}

The \verb|$<-| and \verb|[[<-| operators are implemented for
\texttt{Message} objects to set the value of a field. The
R data is coerced to match the type of the message field.

<<>>=
message <- new( tutorial.Person,
	name = "foo", id = 2 )
message$email <- "foo@bar.com"
message[[ "id" ]] <- 2
message[[ 1 ]] <- "foobar"
writeLines( message$as.character() )
@

Table~\ref{table-message-field-setters} describes the R types that
are allowed in the right hand side depending on the target type of the
field.

\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|p{5cm}|p{7cm}|}
\hline
internal type & allowed R types \\
\hline
\hline
\texttt{double}, \texttt{float}	& \texttt{integer}, \texttt{raw}, \texttt{double}, \texttt{logical} \\
\hline
\texttt{int32},  \texttt{int64},  \texttt{uint32},  \texttt{uint64},  \texttt{sint32},
\texttt{sint64},  \texttt{fixed32},  \texttt{fixed64},  \texttt{sfixed32},  \texttt{sfixed64}
&
	\texttt{integer}, \texttt{raw}, \texttt{double},
        \texttt{logical}, \texttt{character} \\
\hline
\texttt{bool} & \texttt{integer}, \texttt{raw}, \texttt{double}, \texttt{logical} \\
\hline
\texttt{bytes}, \texttt{string} & \texttt{character} \\
\hline
\texttt{enum} & \texttt{integer}, \texttt{double}, \texttt{raw}, \texttt{character} \\
\hline
\texttt{message}, \texttt{group} & \texttt{S4}, of class \texttt{Message} of the appropriate message type, or a \texttt{list} of
\texttt{S4} objects of class \texttt{Message} of the appropriate message type.\\
\hline
\end{tabular}
\end{small}
\caption{\label{table-message-field-setters}Allowed R types depending on internal field types. }
\end{table}


\subsubsection{Message\$has method}
\label{Message-method-has}

The \texttt{has} method indicates if a field of a message is set. For
repeated fields, the field is considered set if there is at least
on object in the array. For non-repeated fields, the field is considered
set if it has been initialized.

The \texttt{has} method is a thin wrapper around the \texttt{HasField}
and \texttt{FieldSize} methods of the \texttt{google::protobuf::Reflection} C++ class.

<<>>=
message <- new( tutorial.Person, name = "foo" )
message$has( "name" )
message$has( "id" )
message$has( "phone" )
@

\subsubsection{Message\$clone method}
\label{Message-method-clone}

The \texttt{clone} function creates a new message that is a clone
of the message. This function is a wrapper around the methods \texttt{New}
and \texttt{CopyFrom} of the \texttt{google::protobuf::Message} C++ class.

<<>>=
m1 <- new( tutorial.Person, name = "foo" )
m2 <- m1$clone( )
m2$email <- "foo@bar.com"
writeLines( as.character( m1 ) )
writeLines( as.character( m2 ) )
@

\subsubsection{Message\$isInitialized method}
\label{Message-method-isInitialized}

The \texttt{isInitialized} method quickly checks
if all required fields have values set. This is a thin wrapper
around the \texttt{IsInitialized} method of the
\texttt{google::protobuf::Message} C++ class.

<<>>=
message <- new( tutorial.Person, name = "foo" )
message$isInitialized()
message$id <- 2
message$isInitialized()
@

\subsubsection{Message\$serialize method}
\label{Message-method-serialize}

The \texttt{serialize} method can be used to serialize
the message as a sequence of bytes into a file or a binary connection.

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
tf1 <- tempfile( )
tf1
message$serialize( tf1 )

tf2 <- tempfile( )
tf2
con <- file( tf2, open = "wb" )
message$serialize( con )
close( con )
@

The files \Sexpr{basename(tf1)} and \Sexpr{basename(tf2)} both
contain the message payload as a sequence of bytes. The \texttt{readBin}
function can be used to read the files as a raw vector in R:

<<>>=
readBin( tf1, raw(0), 500 )
readBin( tf2, raw(0), 500 )
@

The \texttt{serialize} method can also be used to directly retrieve the
payload of the message as a raw vector:

<<>>=
message$serialize(NULL)
@

\subsubsection{Message\$clear method}
\label{Message-method-clear}

The \texttt{clear} method can be used to clear all fields of a
message when used with no argument, or a given field.

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
writeLines( as.character( message ) )
message$clear()
writeLines( as.character( message ) )

message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
message$clear( "id" )
writeLines( as.character( message ) )
@

The \texttt{clear} method is a thin wrapper around the
\texttt{Clear} method of the \texttt{google::protobuf::Message} C++ class.


\subsubsection{Message\$size method}
\label{Message-method-size}

The \texttt{size} method is used to query the number of objects
in a repeated field of a message :

<<keep.source=T>>=
message <- new( tutorial.Person, name = "foo",
	phone = list(
		new( tutorial.Person.PhoneNumber, number = "+33(0)...", type = "HOME"  ),
		new( tutorial.Person.PhoneNumber, number = "+33(0)###", type = "MOBILE"  )
		) )
message$size( "phone" )
size( message, "phone" )
@

The \texttt{size} method is a thin wrapper around the \texttt{FieldSize}
method of the \texttt{google::protobuf::Reflection} C++ class.


\subsubsection{Message\$bytesize method}
\label{Message-method-bytesize}

The \texttt{bytesize} method retrieves the number of bytes the message
would take once serialized. This is a thin wrapper around the \texttt{ByteSize}
method of the \texttt{google::protobuf::Message} C++ class.

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
message$bytesize()
bytesize( message )
length( message$serialize( NULL ) )
@

\subsubsection{Message\$swap method}
\label{Message-method-swap}

The \texttt{swap} method can be used to swap elements
of a repeated field.

<<keep.source=T>>=
message <- new( tutorial.Person, name = "foo",
	phone = list(
		new( tutorial.Person.PhoneNumber, number = "+33(0)...", type = "HOME"  ),
		new( tutorial.Person.PhoneNumber, number = "+33(0)###", type = "MOBILE"  )
		) )
message$swap( "phone", 1, 2 )
writeLines( as.character( message$phone[[1]] ) )
writeLines( as.character( message$phone[[2]] ) )

swap( message, "phone", 1, 2 )
writeLines( as.character( message$phone[[1]] ) )
writeLines( as.character( message$phone[[2]] ) )
@


\subsubsection{Message\$set method}
\label{Message-method-set}

The \texttt{set} method can be used to set values
of a repeated field.

<<keep.source=T>>=
message <- new( tutorial.Person, name = "foo",
	phone = list(
		new( tutorial.Person.PhoneNumber, number = "+33(0)...", type = "HOME"  ),
		new( tutorial.Person.PhoneNumber, number = "+33(0)###", type = "MOBILE"  )
		) )
number <- new( tutorial.Person.PhoneNumber,
		number = "+33(0)---", type = "WORK"  )
message$set( "phone", 1, number )
writeLines( as.character( message ) )
@

\subsubsection{Message\$fetch method}
\label{Message-method-fetch}

The \texttt{fetch} method can be used to get values
of a repeated field.

<<keep.source=T>>=
message <- new( tutorial.Person, name = "foo",
	phone = list(
		new( tutorial.Person.PhoneNumber, number = "+33(0)...", type = "HOME"  ),
		new( tutorial.Person.PhoneNumber, number = "+33(0)###", type = "MOBILE"  )
		) )
message$fetch( "phone", 1 )
@

\subsubsection{Message\$setExtension method}
\label{Message-method-setExtension}

The \texttt{setExtension} method can be used to set an extension field of the
Message.

<<keep.source=T>>=
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("unitTests", "data",
                                       "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}

## Test setting a singular extensions.
test <- new(protobuf_unittest.TestAllExtensions)
test$setExtension(protobuf_unittest.optional_int32_extension,
                  as.integer(1))
@

\subsubsection{Message\$getExtension method}
\label{Message-method-getExtension}

The \texttt{getExtension} method can be used to get values
of an extension.

<<keep.source=T>>=
test$getExtension(protobuf_unittest.optional_int32_extension)
@

\subsubsection{Message\$add method}
\label{Message-method-add}

The \texttt{add} method can be used to add values to a repeated field.

<<keep.source=T>>=
message <- new( tutorial.Person, name = "foo")
phone <- new( tutorial.Person.PhoneNumber,
	number = "+33(0)...", type = "HOME"  )
message$add( "phone", phone )
writeLines( message$toString() )
@

\subsubsection{Message\$str method}
\label{Message-method-str}

The \texttt{str} method gives the R structure of the message. This is rarely
useful.

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
message$str()
str( message )
@

\subsubsection{Message\$as.character method}
\label{Message-method-ascharacter}

The \texttt{as.character} method gives the debug string of the message.

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
writeLines( message$as.character() )
writeLines( as.character( message ) )
@

\subsubsection{Message\$toString method}
\label{Message-method-toString}

\texttt{toString} currently is an alias to the \texttt{as.character} function.

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
writeLines( message$toString() )
writeLines( toString( message ) )
@

\subsubsection{Message\$as.list method}
\label{Message-method-aslist}

The \texttt{as.list} method converts the message to an named R list

<<>>=
message <- new( tutorial.Person, name = "foo", email = "foo@bar.com", id = 2 )
as.list( message )
@

The names of the list are the names of the declared fields of the
message type, and the content is the same as can be extracted with
the \verb|$| operator described in
section~\ref{Message-method-getfield}.

\subsubsection{Message\$update method}
\label{Message-method-update}

The \texttt{update} method can be used to update several fields
of a message at once.

<<keep.source=TRUE>>=
message <- new( tutorial.Person )
update( message,
	name = "foo",
	id = 2,
	email = "foo@bar.com" )
writeLines( message$as.character() )
@

\subsubsection{Message\$descriptor method}
\label{Message-method-descriptor}

The \texttt{descriptor} method retrieves the
descriptor of a message. See section~\ref{subsec-descriptor}
for more information about message type descriptors.

<<>>=
message <- new( tutorial.Person )
message$descriptor()
descriptor( message )
@

\subsubsection{Message\$fileDescriptor method}
\label{Message-method-fileDescriptor}

The \texttt{fileDescriptor} method retrieves the
file descriptor of the descriptor associated with a message.
See section~\ref{subsec-fileDescriptor} for more information
about file descriptors.

<<>>=
message <- new( tutorial.Person )
message$fileDescriptor()
fileDescriptor( message )
@

\subsection{message descriptors}
\label{subsec-descriptor}

Message descriptors are represented in R with the
\emph{Descriptor} S4 class. The class contains
the slots \texttt{pointer} and \texttt{type} :

\begin{table}[h]
\centering
\begin{tabular}{|cp{10cm}|}
\hline
\textbf{slot} & \textbf{description} \\
\hline
\texttt{pointer} & external pointer to the \texttt{Descriptor} object of the C++ proto library. Documentation for the
\texttt{Descriptor} class is available from the protocol buffer project page:
\url{http://code.google.com/apis/protocolbuffers/docs/reference/cpp/google.protobuf.descriptor.html#Descriptor} \\
\hline
\texttt{type} & fully qualified path of the message type. \\
\hline
\end{tabular}
\caption{\label{Descriptor-class-table}Description of slots for the \texttt{Descriptor} S4 class}
\end{table}

Similarly to messages, the \verb|$| operator can be used to extract
information from the descriptor, or invoke pseudo-methods.
Table~\ref{Descriptor-methods-table} describes the methods defined for the \texttt{Descriptor} class :

\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|ccp{8cm}|}
\hline
\textbf{Method} & \textbf{Section} & \textbf{Description} \\
\hline
\hline
\texttt{new} & \ref{Descriptor-method-new} & Creates a prototype of a message described by this descriptor.\\
\texttt{read} & \ref{Descriptor-method-read} & Reads a message from a file or binary connection.\\
\texttt{readASCII} & \ref{Descriptor-method-readASCII} & Read a message in ASCII format from a file or
text connection.\\
\hline
\texttt{name} & \ref{Descriptor-method-name} & Retrieve the name of the message type associated with
this descriptor.\\
\texttt{as.character} & \ref{Descriptor-method-ascharacter} & character representation of a descriptor\\
\texttt{toString} & \ref{Descriptor-method-tostring} & character representation of a descriptor (same as \texttt{as.character}) \\
\texttt{as.list} & \ref{Descriptor-method-aslist} & return a named
list of the field, enum, and nested descriptors included in this descriptor.\\
\texttt{asMessage} & \ref{Descriptor-method-asmessage} &
return DescriptorProto message. \\
\hline
\texttt{fileDescriptor} & \ref{Descriptor-method-filedescriptor} & Retrieve the file descriptor of this
descriptor.\\
\texttt{containing\_type} & \ref{Descriptor-method-containingtype} & Retrieve the descriptor describing the message type containing this descriptor.\\
\texttt{field\_count} & \ref{Descriptor-method-fieldcount} & Return the number of fields in this descriptor.\\
\texttt{field} & \ref{Descriptor-method-field} & Return the descriptor for the specified field in this descriptor.\\
\texttt{nested\_type\_count} & \ref{Descriptor-method-nestedtypecount}
& The number of nested types in this descriptor.\\
\texttt{nested\_type} & \ref{Descriptor-method-nestedtype} & Return the descriptor for the specified nested
type in this descriptor.\\
\texttt{enum\_type\_count} & \ref{Descriptor-method-enumtypecount} & The number of enum types in this descriptor.\\
\texttt{enum\_type} & \ref{Descriptor-method-enumtype} & Return the descriptor for the specified enum
type in this descriptor.\\
\hline
\end{tabular}
\end{small}
\caption{\label{Descriptor-methods-table}Description of methods for the \texttt{Descriptor} S4 class}
\end{table}

\subsubsection{Extracting descriptors}

The \verb|$| operator, when used on a descriptor object retrieves
descriptors that are contained in the descriptor.

This can be a field descriptor (see section~\ref{subsec-field-descriptor} ),
an enum descriptor (see section~\ref{subsec-enum-descriptor}) or a descriptor
for a nested type

<<>>=
# field descriptor
tutorial.Person$email

# enum descriptor
tutorial.Person$PhoneType

# nested type descriptor
tutorial.Person$PhoneNumber
# same as
tutorial.Person.PhoneNumber
@

\subsubsection{The new method}
\label{Descriptor-method-new}

The \texttt{new} method creates a prototype of a message
described by the descriptor.

<<>>=
tutorial.Person$new( )
new( tutorial.Person )
@

Passing additional arguments to the method allows to directlt set
the fields of the message at construction time.

<<>>=
tutorial.Person$new( email = "foo@bar.com" )

# same as
update( tutorial.Person$new( ), email = "foo@bar.com" )
@

\subsubsection{The read method}
\label{Descriptor-method-read}

The \texttt{read} method is used to read a message
from a file or a binary connection.

<<>>=
# start by serializing a message
message <- new( tutorial.Person.PhoneNumber,
	type = "HOME", number = "+33(0)...." )
tf <- tempfile()
serialize( message, tf )

# now read back the message
m <- tutorial.Person.PhoneNumber$read( tf )
writeLines( as.character( m ) )

m <- read( tutorial.Person.PhoneNumber, tf )
writeLines( as.character( m ) )
@


\subsubsection{The readASCII method}
\label{Descriptor-method-readASCII}

The \texttt{readASCII} method is used to read a message
from a text file or a character vector.

<<>>=
# start by generating the ASCII representation of a message
text <- as.character(new(tutorial.Person, id=1, name="Murray"))
text
# Then read the ascii representation in as a new message object.
msg <- tutorial.Person$readASCII(text)
@

\subsubsection{The toString method}
\label{Descriptor-method-tostring}

\texttt{toString} currently is an alias to the \texttt{as.character} function.


\subsubsection{The as.character method}
\label{Descriptor-method-ascharacter}

\texttt{as.character} prints the text representation of the descriptor
as it would be specified in the \texttt{.proto} file.

<<>>=
desc <- tutorial.Person
writeLines( desc$toString() )
writeLines( toString( desc ) )
writeLines( as.character(tutorial.Person) )
@

\subsubsection{as.list}
\label{Descriptor-method-aslist}

The \texttt{as.list} method returns a named list of the field, enum,
and nested descriptors included in this descriptor.

<<>>=
tutorial.Person$as.list()
@

\subsubsection{asMessage}
\label{Descriptor-method-asmessage}

The \texttt{asMessage} method returns a message of type
\texttt{google.protobuf.DescriptorProto} of the Descriptor.

<<>>=
tutorial.Person$asMessage()
@

\subsubsection{The fileDescriptor method}
\label{Descriptor-method-filedescriptor}

The \texttt{fileDescriptor} method retrieves the
file descriptor of the descriptor.  See section~\ref{subsec-fileDescriptor} for more information
about file descriptors.

<<>>=
desc <- tutorial.Person
desc$fileDescriptor()
fileDescriptor( desc )
@

\subsubsection{The name method}
\label{Descriptor-method-name}

The \texttt{name} method can be used to retrieve the name of the
message type associated with the descriptor.

<<>>=
# simple name
tutorial.Person$name()
# name including scope
tutorial.Person$name(full = TRUE)
@

\subsubsection{The containing\_type method}
\label{Descriptor-method-containingtype}

The \texttt{containing\_type} method retrieves the descriptor
describing the message type containing this descriptor.

<<>>=
tutorial.Person$containing_type()
tutorial.Person$PhoneNumber$containing_type()
@

\subsubsection{The field\_count method}
\label{Descriptor-method-fieldcount}

Tme \texttt{field\_count} method retrieves the number of fields in
this descriptor.

<<>>=
tutorial.Person$field_count()
@

\subsubsection{The field method}
\label{Descriptor-method-field}

The \texttt{field} method returns the descriptor for the specified
field in this descriptor.

<<>>=
tutorial.Person$field(1)
@

\subsubsection{The nested\_type\_count method}
\label{Descriptor-method-nestedtypecount}

The \texttt{nested\_type\_count} method returns the number of nested
types in this descriptor.

<<>>=
tutorial.Person$nested_type_count()
@

\subsubsection{The nested\_type method}
\label{Descriptor-method-nestedtype}

The \texttt{nested\_type} method return the descriptor for the specified nested
type in this descriptor.

<<>>=
tutorial.Person$nested_type(1)
@

\subsubsection{The enum\_type\_count method}
\label{Descriptor-method-enumtypecount}

The \texttt{enum\_type\_count} method returns the number of enum types
in this descriptor.

<<>>=
tutorial.Person$enum_type_count()
@

\subsubsection{The enum\_type method}
\label{Descriptor-method-enumtype}

The \texttt{enum\_type} method returns the descriptor for the specified enum
type in this descriptor.

<<>>=
tutorial.Person$enum_type(1)
@

\subsection{field descriptors}
\label{subsec-field-descriptor}

The class \emph{FieldDescriptor} represents field
descriptor in R. This is a wrapper S4 class around the
\texttt{google::protobuf::FieldDescriptor} C++ class.
Table~\ref{fielddescriptor-methods-table} describes the methods
defined for the \texttt{FieldDescriptor} class.

\begin{table}[h]
\centering
\begin{tabular}{|cp{10cm}|}
\hline
\textbf{slot} & \textbf{description} \\
\hline
\texttt{pointer} & External pointer to the \texttt{FieldDescriptor} C++ variable \\
\hline
\texttt{name} & simple name of the field \\
\hline
\texttt{full\_name} & fully qualified name of the field \\
\hline
\texttt{type} & name of the message type where the field is declared \\
\hline
\end{tabular}
\caption{\label{FieldDescriptor-class-table}Description of slots for the \texttt{FieldDescriptor} S4 class}
\end{table}


\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|ccp{8cm}|}
\hline
\textbf{method} & \textbf{section} & \textbf{description} \\
\hline
\hline
\texttt{as.character} & \ref{fielddescriptor-method-ascharacter} & character representation of a descriptor\\
\texttt{toString} & \ref{fielddescriptor-method-tostring} & character
representation of a descriptor (same as \texttt{as.character}) \\
\texttt{asMessage} & \ref{fielddescriptor-method-asmessage} &
return FieldDescriptorProto message. \\
\texttt{name} &  \ref{fielddescriptor-method-name} & Return the name of the field descriptor.\\
\texttt{fileDescriptor} & \ref{fielddescriptor-method-filedescriptor}
& Return the fileDescriptor where this field is defined.\\
\texttt{containing\_type} &
\ref{fielddescriptor-method-containingtype} & Return the containing descriptor of this field.\\
\texttt{is\_extension} & \ref{fielddescriptor-method-isextension} & Return TRUE if this field is an extension.\\
\texttt{number} & \ref{fielddescriptor-method-number} & Gets the declared tag number of the field.\\
\texttt{type} & \ref{fielddescriptor-method-type} & Gets the type of the field.\\
\texttt{cpp\_type} & \ref{fielddescriptor-method-cpptype} & Gets the C++ type of the field.\\
\texttt{label} & \ref{fielddescriptor-method-label} & Gets the label of a field (optional, required, or repeated).\\
\texttt{is\_repeated} & \ref{fielddescriptor-method-isrepeated} & Return TRUE if this field is repeated.\\
\texttt{is\_required} & \ref{fielddescriptor-method-isrequired} & Return TRUE if this field is required.\\
\texttt{is\_optional} & \ref{fielddescriptor-method-isoptional} & Return TRUE if this field is optional.\\
\texttt{has\_default\_value} & \ref{fielddescriptor-method-hasdefaultvalue} & Return TRUE if this field has a default value.\\
\texttt{default\_value} & \ref{fielddescriptor-method-defaultvalue} &
Return the default value.\\
\texttt{message\_type} & \ref{fielddescriptor-method-messagetype} & Return the message type if this is a message type field.\\
\texttt{enum\_type} & \ref{fielddescriptor-method-enumtype} & Return the enum type if this is an enum type field.\\
\hline
\end{tabular}
\end{small}
\caption{\label{fielddescriptor-methods-table}Description of methods for the \texttt{FieldDescriptor} S4 class}
\end{table}

\subsubsection{as.character}
\label{fielddescriptor-method-ascharacter}

The \texttt{as.character} method brings the debug string of the field descriptor.

<<>>=
writeLines( as.character( tutorial.Person$PhoneNumber ) )
@

\subsubsection{toString}
\label{fielddescriptor-method-tostring}

\texttt{toString} is an alias of \texttt{as.character}.

<<>>=
writeLines( tutorial.Person.PhoneNumber$toString() )
@

\subsubsection{asMessage}
\label{fielddescriptor-method-asmessage}

The \texttt{asMessage} method returns a message of type
\texttt{google.protobuf.FieldDescriptorProto} of the FieldDescriptor.

<<>>=
tutorial.Person$id$asMessage()
writeLines(as.character(tutorial.Person$id$asMessage()))
@

\subsubsection{name}
\label{fielddescriptor-method-name}

The \texttt{name} method can be used to retrieve the name of the
field descriptor.

<<>>=
# simple name.
name( tutorial.Person$id )
# name including scope.
name( tutorial.Person$id, full=TRUE )
@

\subsubsection{fileDescriptor}
\label{fielddescriptor-method-filedescriptor}

The \texttt{fileDescriptor} method can be used to retrieve the file
descriptor of the field descriptor.

<<>>=
fileDescriptor(tutorial.Person$id)
tutorial.Person$id$fileDescriptor()
@

\subsubsection{containing\_type}
\label{fielddescriptor-method-containingtype}

The \texttt{containing\_type} method can be used to retrieve the
descriptor for the message type that contains this descriptor.

<<>>=
containing_type(tutorial.Person$id)
tutorial.Person$id$containing_type()
@


\subsubsection{is\_extension}
\label{fielddescriptor-method-isextension}

The \texttt{is\_extension} method returns TRUE if this field is an extension.
% TODO(ms): cross reference to a to be written section on extensions.

<<>>=
is_extension( tutorial.Person$id )
tutorial.Person$id$is_extension()
@

\subsubsection{number}
\label{fielddescriptor-method-number}

The \texttt{number} method returns the declared tag number of this field.

<<>>=
number( tutorial.Person$id )
tutorial.Person$id$number()
@

\subsubsection{type}
\label{fielddescriptor-method-type}

The \texttt{type} method can be used to retrieve the type of the
field descriptor.

<<>>=
type( tutorial.Person$id )
tutorial.Person$id$type()
@
\subsubsection{cpp\_type}
\label{fielddescriptor-method-cpptype}

The \texttt{cpp\_type} method can be used to retrieve the C++ type of the
field descriptor.

<<>>=
cpp_type( tutorial.Person$id )
tutorial.Person$id$cpp_type()
@

\subsubsection{label}
\label{fielddescriptor-method-label}

Gets the label of a field (optional, required, or repeated).
The \texttt{label} method returns the label of a field (optional,
required, or repeated).  By defualt it returns a number value, but the
optional \texttt{as.string} argument can be provided to return a
human readable string representation.

<<>>=
label( tutorial.Person$id )
label( tutorial.Person$id , TRUE)
tutorial.Person$id$label(TRUE)
@
\subsubsection{is\_repeated}
\label{fielddescriptor-method-isrepeated}

The \texttt{is\_repeated} method returns TRUE if this field is repeated.

<<>>=
is_repeated( tutorial.Person$id )
tutorial.Person$id$is_repeated()
@

\subsubsection{is\_required}
\label{fielddescriptor-method-isrequired}

The \texttt{is\_required} method returns TRUE if this field is required.

<<>>=
is_required( tutorial.Person$id )
tutorial.Person$id$is_required()
@

\subsubsection{is\_optional}
\label{fielddescriptor-method-isoptional}

The \texttt{is\_optional} method returns TRUE if this field is optional.

<<>>=
is_optional( tutorial.Person$id )
tutorial.Person$id$is_optional()
@

\subsubsection{has\_default\_value}
\label{fielddescriptor-method-hasdefaultvalue}

The \texttt{has\_default\_value} method returns TRUE if this field has
a default value.

<<>>=
has_default_value(tutorial.Person$PhoneNumber$type)
has_default_value(tutorial.Person$PhoneNumber$number)
@

\subsubsection{default\_value}
\label{fielddescriptor-method-defaultvalue}

The \texttt{default\_value} method returns the default value of a field.

<<>>=
default_value( tutorial.Person$PhoneNumber$type )
default_value( tutorial.Person$PhoneNumber$number )
@

\subsubsection{message\_type}
\label{fielddescriptor-method-messagetype}

The \texttt{message\_type} method returns the message type if this is
a message type field.

<<>>=
message_type(tutorial.Person$phone)
tutorial.Person$phone$message_type()
@
\subsubsection{enum\_type}
\label{fielddescriptor-method-enumtype}

The \texttt{enum\_type} method returns the enum type if this is an enum type field.

<<>>=
enum_type(tutorial.Person$PhoneNumber$type)
@

\subsection{enum descriptors}
\label{subsec-enum-descriptor}

The class \emph{EnumDescriptor} is an R wrapper
class around the C++ class \texttt{google::protobuf::EnumDescriptor}.
Table~\ref{enumdescriptor-methods-table} describes the methods
defined for the \texttt{EnumDescriptor} class.

\begin{table}[h]
\centering
\begin{tabular}{|cp{10cm}|}
\hline
\textbf{slot} & \textbf{description} \\
\hline
\texttt{pointer} & External pointer to the \texttt{EnumDescriptor} C++ variable \\
\hline
\texttt{name} & simple name of the enum \\
\hline
\texttt{full\_name} & fully qualified name of the enum \\
\hline
\texttt{type} & name of the message type where the enum is declared \\
\hline
\end{tabular}
\caption{\label{EnumDescriptor-class-table}Description of slots for the \texttt{EnumDescriptor} S4 class}
\end{table}

\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|ccp{8cm}|}
\hline
\textbf{method} & \textbf{section} & \textbf{description} \\
\hline
\hline
\texttt{as.list} & \ref{enumdescriptor-method-aslist} & return a named
integer vector with the values of the enum and their names.\\
\texttt{as.character} & \ref{enumdescriptor-method-ascharacter} & character representation of a descriptor\\
\texttt{toString} & \ref{enumdescriptor-method-tostring} & character
representation of a descriptor (same as \texttt{as.character}) \\
\texttt{asMessage} & \ref{enumdescriptor-method-asmessage} &
return EnumDescriptorProto message. \\
\texttt{name} &  \ref{enumdescriptor-method-name} & Return the name of the enum descriptor.\\
\texttt{fileDescriptor} & \ref{enumdescriptor-method-filedescriptor}
& Return the fileDescriptor where this field is defined.\\
\texttt{containing\_type} &
\ref{enumdescriptor-method-containingtype} & Return the containing descriptor of this field.\\
\texttt{length} &
\ref{enumdescriptor-method-length} & Return the number of constants in
this enum.\\
\texttt{has} &
\ref{enumdescriptor-method-has} & Return TRUE if this enum contains
the specified named constant string.\\
\texttt{value\_count} &
\ref{enumdescriptor-method-valuecount} & Return the number of constants in
this enum (same as \texttt{length}).\\
\texttt{value} &
\ref{enumdescriptor-method-value} & Return the
EnumValueDescriptor of an enum value of specified index, name, or number.\\
\hline
\end{tabular}
\end{small}
\caption{\label{enumdescriptor-methods-table}Description of methods for the \texttt{EnumDescriptor} S4 class}
\end{table}

\subsubsection{Extracting descriptors}

The \verb|$| operator, when used on a EnumDescriptor object retrieves
EnumValueDescriptors that are contained in the descriptor.

<<>>=
tutorial.Person$PhoneType$WORK
name(tutorial.Person$PhoneType$value(number=2))
@

\subsubsection{as.list}
\label{enumdescriptor-method-aslist}

The \texttt{as.list} method creates a named R integer vector that
captures the values of the enum and their names.

<<>>=
as.list( tutorial.Person$PhoneType )
@

\subsubsection{as.character}
\label{enumdescriptor-method-ascharacter}

The \texttt{as.character} method brings the debug string of the enum type.

<<>>=
writeLines( as.character( tutorial.Person$PhoneType ) )
@

\subsubsection{toString}
\label{enumdescriptor-method-tostring}

The \texttt{toString} method brings the debug string of the enum type.

<<>>=
writeLines( toString( tutorial.Person$PhoneType ) )
@

\subsubsection{asMessage}
\label{enumdescriptor-method-asmessage}

The \texttt{asMessage} method returns a message of type
\texttt{google.protobuf.EnumDescriptorProto} of the EnumDescriptor.

<<>>=
tutorial.Person$PhoneType$asMessage()
writeLines(as.character(tutorial.Person$PhoneType$asMessage()))
@

\subsubsection{name}
\label{enumdescriptor-method-name}

The \texttt{name} method can be used to retrieve the name of the
enum descriptor.

<<>>=
# simple name.
name( tutorial.Person$PhoneType )
# name including scope.
name( tutorial.Person$PhoneType, full=TRUE )
@

\subsubsection{fileDescriptor}
\label{enumdescriptor-method-filedescriptor}

The \texttt{fileDescriptor} method can be used to retrieve the file
descriptor of the enum descriptor.

<<>>=
fileDescriptor(tutorial.Person$PhoneType)
tutorial.Person$PhoneType$fileDescriptor()
@

\subsubsection{containing\_type}
\label{enumdescriptor-method-containingtype}

The \texttt{containing\_type} method can be used to retrieve the
descriptor for the message type that contains this enum descriptor.

<<>>=
tutorial.Person$PhoneType$containing_type()
@

\subsubsection{length}
\label{enumdescriptor-method-length}

The \texttt{length} method returns the number of constants in this enum.

<<>>=
length(tutorial.Person$PhoneType)
tutorial.Person$PhoneType$length()
@

\subsubsection{has}
\label{enumdescriptor-method-has}

The \texttt{has} method returns TRUE if this enum contains the
specified named constant string.

<<>>=
tutorial.Person$PhoneType$has("WORK")
tutorial.Person$PhoneType$has("nonexistant")
@

\subsubsection{value\_count}
\label{enumdescriptor-method-valuecount}

The \texttt{value\_count} method returnst he number of constants in
this enum.
%TODO(ms): Duplicate of length, just like tostring/as.character.
<<>>=
value_count(tutorial.Person$PhoneType)
tutorial.Person$PhoneType$value_count()
@

\subsubsection{value}
\label{enumdescriptor-method-value}

The \texttt{value} method extracts an EnumValueDescriptor.  Exactly
one argument of 'index', 'number', or 'name' must be specified to
identify which constant is desired.

<<>>=
tutorial.Person$PhoneType$value(1)
tutorial.Person$PhoneType$value(name="HOME")
tutorial.Person$PhoneType$value(number=1)
@

\subsection{enum value descriptors}
\label{subsec-EnumValueDescriptor}

The class \emph{EnumValueDescriptor} is an R wrapper
class around the C++ class \texttt{google::protobuf::EnumValueDescriptor}.
Table~\ref{enumvaluedescriptor-methods-table} describes the methods
defined for the \texttt{EnumValueDescriptor} class.

\begin{table}[h]
\centering
\begin{tabular}{|cp{10cm}|}
\hline
\textbf{slot} & \textbf{description} \\
\hline
\texttt{pointer} & External pointer to the \texttt{EnumValueDescriptor} C++ variable \\
\hline
\texttt{name} & simple name of the enum value \\
\hline
\texttt{full\_name} & fully qualified name of the enum value \\
\hline
\end{tabular}
\caption{\label{EnumValueDescriptor-class-table}Description of slots for the \texttt{EnumValueDescriptor} S4 class}
\end{table}

\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|ccp{8cm}|}
\hline
\textbf{method} & \textbf{section} & \textbf{description} \\
\hline
\hline
\texttt{number} & \ref{enumvaluedescriptor-method-number} & return the
number of this EnumValueDescriptor. \\
\texttt{name} & \ref{enumvaluedescriptor-method-name} & Return the name of
the enum value descriptor.\\
\texttt{enum\_type} & \ref{enumvaluedescriptor-method-enumtype} &
return the EnumDescriptor type of this EnumValueDescriptor. \\
\texttt{as.character} & \ref{enumvaluedescriptor-method-ascharacter} & character
representation of a descriptor. \\
\texttt{toString} & \ref{enumvaluedescriptor-method-tostring} & character
representation of a descriptor (same as \texttt{as.character}). \\
\texttt{asMessage} & \ref{enumvaluedescriptor-method-asmessage} &
return EnumValueDescriptorProto message. \\
\hline
\end{tabular}
\end{small}
\caption{\label{enumvaluedescriptor-methods-table}Description of methods for the \texttt{EnumValueDescriptor} S4 class}
\end{table}

\subsubsection{number}
\label{enumvaluedescriptor-method-number}

The \texttt{number} method can be used to retrieve the number of the
enum value descriptor.

<<>>=
number( tutorial.Person$PhoneType$value(number=2) )
@

\subsubsection{name}
\label{enumvaluedescriptor-method-name}

The \texttt{name} method can be used to retrieve the name of the
enum value descriptor.

<<>>=
# simple name.
name( tutorial.Person$PhoneType$value(number=2) )
# name including scope.
name( tutorial.Person$PhoneType$value(number=2), full=TRUE )
@

\subsubsection{enum\_type}
\label{enumvaluedescriptor-method-enumtype}

The \texttt{enum\_type} method can be used to retrieve the EnumDescriptor of the
enum value descriptor.

<<>>=
enum_type( tutorial.Person$PhoneType$value(number=2) )
@

\subsubsection{as.character}
\label{enumvaluedescriptor-method-ascharacter}

The \texttt{as.character} method brings the debug string of the enum
value type.

<<>>=
writeLines( as.character( tutorial.Person$PhoneType$value(number=2) ) )
@

\subsubsection{toString}
\label{enumvaluedescriptor-method-tostring}

The \texttt{toString} method brings the debug string of the enum value
type.

<<>>=
writeLines( toString( tutorial.Person$PhoneType$value(number=2) ) )
@

\subsubsection{asMessage}
\label{enumvaluedescriptor-method-asmessage}

The \texttt{asMessage} method returns a message of type
\texttt{google.protobuf.EnumValueDescriptorProto} of the EnumValueDescriptor.

<<>>=
tutorial.Person$PhoneType$value(number=2)$asMessage()
writeLines(as.character(tutorial.Person$PhoneType$value(number=2)$asMessage()))
@

\subsection{file descriptors}
\label{subsec-fileDescriptor}

File descriptors describe a whole \texttt{.proto} file and are represented in R with the
\emph{FileDescriptor} S4 class. The class contains
the slots \texttt{pointer}, \texttt{filename}, and \texttt{package} :

\begin{table}[h]
\centering
\begin{tabular}{|cp{10cm}|}
\hline
\textbf{slot} & \textbf{description} \\
\hline
\texttt{pointer} & external pointer to the \texttt{FileDescriptor} object of the C++ proto library. Documentation for the
\texttt{FileDescriptor} class is available from the protocol buffer project page:
\url{http://developers.google.com/protocol-buffers/docs/reference/cpp/google.protobuf.descriptor.html#FileDescriptor} \\
\hline
\texttt{filename} & fully qualified pathname of the \texttt{.proto} file.\\
\hline
\texttt{package} & package name defined in this \texttt{.proto} file.\\
\hline
\end{tabular}
\caption{\label{FileDescriptor-class-table}Description of slots for the \texttt{FileDescriptor} S4 class}
\end{table}

Similarly to messages, the \verb|$| operator can be used to extract
fields from the file descriptor (in this case, types defined in the
file), or invoke pseuso-methods.
Table~\ref{filedescriptor-methods-table} describes the methods
defined for the \texttt{FileDescriptor} class.

<<>>=
f <- tutorial.Person$fileDescriptor()
f
f$Person
@
\begin{table}[h]
\centering
\begin{small}
\begin{tabular}{|ccp{8cm}|}
\hline
\textbf{method} & \textbf{section} & \textbf{description} \\
\hline
\hline
\texttt{name} & \ref{filedescriptor-method-name} & Return the filename
for this FileDescriptorProto.\\
\texttt{package} & \ref{filedescriptor-method-package} & Return the
file-level package name specified in this FileDescriptorProto.\\
\texttt{as.character} & \ref{filedescriptor-method-ascharacter} & character
representation of a descriptor. \\
\texttt{toString} & \ref{filedescriptor-method-tostring} & character
representation of a descriptor (same as \texttt{as.character}). \\
\texttt{asMessage} & \ref{filedescriptor-method-asmessage} &
return FileDescriptorProto message. \\
\texttt{as.list} & \ref{filedescriptor-method-aslist} &
return named list of descriptors defined in this file descriptor.\\
\hline
\end{tabular}
\end{small}
\caption{\label{filedescriptor-methods-table}Description of methods for the \texttt{FileDescriptor} S4 class}
\end{table}

\subsubsection{as.character}
\label{filedescriptor-method-ascharacter}
The \texttt{as.character} method brings the debug string of the file descriptor.

<<>>=
writeLines( as.character(fileDescriptor(tutorial.Person)) )
@

\subsubsection{toString}
\label{filedescriptor-method-tostring}

\texttt{toString} is an alias of \texttt{as.character}.

<<>>=
writeLines( fileDescriptor(tutorial.Person)$toString() )
@

\subsubsection{asMessage}
\label{filedescriptor-method-asmessage}

The \texttt{asMessage} method returns a protocol buffer message representation of the file descriptor.

<<>>=
asMessage(tutorial.Person$fileDescriptor())
writeLines( as.character(asMessage(tutorial.Person$fileDescriptor())) )
@

\subsubsection{as.list}
\label{filedescriptor-method-aslist}

The \texttt{as.list} method creates a named R list that contains the descriptors defined in this file descriptor.

<<>>=
as.list( tutorial.Person$fileDescriptor() )
@

\subsubsection{name}
\label{filedescriptor-method-name}

The \texttt{name} method can be used to retrieve the file name associated with the
file descriptor.  The optional boolean argument can be specified if
full pathnames are desired.

<<>>=
name( tutorial.Person$fileDescriptor() )
tutorial.Person$fileDescriptor()$name(TRUE)
@

\subsubsection{package}
\label{filedescriptor-method-package}

The \texttt{package} method can be used to retrieve the package scope
associated with this file descriptor.

<<>>=
tutorial.Person$fileDescriptor()$package()
@

\subsection{service descriptors}
\label{subsec-ServiceDescriptor}

Not fully implemented.  Needs to be connected to a concrete RPC
implementation.  The Google Protocol Buffers C++ open-source library
does not include an RPC implementation, but this can be connected
easily to others.

\subsection{method descriptors}
\label{subsec-MethodDescriptor}

Not fully implemented.  Needs to be connected to a concrete RPC
implementation.  The Google Protocol Buffers C++ open-source library
does not include an RPC implementation, but this can be connected
easily to others.

\section{Utilities}

\subsection{coercing objects to messages}

The \texttt{asMessage} function uses the standard coercion
mechanism of the \texttt{as} method, and so can be used as a shorthand :

<<>>=
# coerce a message type descriptor to a message
# asMessage( tutorial.Person )

# coerce a enum descriptor
asMessage( tutorial.Person.PhoneType )

# coerce a field descriptor
asMessage( tutorial.Person$email )

# coerce a file descriptor
asMessage( fileDescriptor( tutorial.Person ) )

@

\subsection{completion}

The \texttt{RProtoBuf} package implements the
\texttt{.DollarNames} S3 generic function
(defined in the \texttt{utils} package) for all classes.

Completion possibilities include pseudo method names for all
classes, plus :
\begin{itemize}
\item field names for messages
\item field names, enum types, nested types for message type descriptors
\item names for enum descriptors
\item names for top-level extensions
\item message names for file descriptors
\end{itemize}

In the unlikely event that there is a user-defined field of exactly
the same name as one of the pseudo methods, the user-defined field
shall take precedence for completion purposes by design, since the
method name can always be invoked directly.

\subsection{with and within}

The S3 generic \texttt{with} function is implemented for class
\texttt{Message}, allowing to evaluate an R expression in
an environment that allows to retrieve and set fields of a message
simply using their names.

<<>>=
message <- new( tutorial.Person, email = "foo@bar.com" )
with( message, {
	# set the id field
	id <- 2

	# set the name field from the email field
	name <- gsub( "[@]", " ", email )

	sprintf( "%d [%s] : %s", id, email, name )
} )
@

The difference between \texttt{with} and \texttt{within} is the value
that is returned. For \texttt{with} returns the result of the R expression,
for \texttt{within} the message is returned. In both cases, the message
is modified because \texttt{RProtoBuf} works by reference.

\subsection{identical}

The \texttt{identical} method is implemented to compare two messages.

<<>>=
m1 <- new( tutorial.Person, email = "foo@bar.com", id = 2 )
m2 <- update( new( tutorial.Person) , email = "foo@bar.com", id = 2 )
identical( m1, m2 )
@

The \verb|==| operator can be used as an alias to \texttt{identical}.

<<>>=
m1 == m2
m1 != m2
@

Alternatively, the \texttt{all.equal} function can be used, allowing
a tolerance when comparing \texttt{float} or \texttt{double} values.

\subsection{merge}

\texttt{merge} can be used to merge two messages of the same type.

<<>>=
m1 <- new( tutorial.Person, name = "foobar" )
m2 <- new( tutorial.Person, email = "foo@bar.com" )
m3 <- merge( m1, m2 )
writeLines( as.character( m3 ) )
@

\subsection{P}

The \texttt{P} function is an alternative way to retrieve
a message descriptor using its type name. It is not often used because
of the lookup mechanism described in section~\ref{sec-lookup}.

<<>>=
P("tutorial.Person")
new( P("tutorial.Person") )

# but we can do this instead
tutorial.Person
new( tutorial.Person )
@

\section{Advanced Features}
\subsection{Extensions}
\label{sec-extensions}

Extensions allow you to declare a range of field numbers in a message
that are available for extension types.  This allows others to declare
new fields for a given message type possibly in their own
\texttt{.proto} files without having to edit the original file.  See
\url{https://developers.google.com/protocol-buffers/docs/proto#extensions}.

Notice that the last line of the \texttt{Person} message schema in
\texttt{addressbook.proto} is the following line :

\begin{verbatim}
  extensions 100 to 199;
\end{verbatim}

This specifies that other users in other .proto files can use tag
numbers between 100 and 199 for extention types of this message.

For example, the following file \texttt{extend.proto}:

<<echo=FALSE,print=FALSE>>=
  extend.proto <- tempfile()
  writeLines(c(
               paste0('import "',
                      name(tutorial.Person$fileDescriptor(), TRUE), '";'),
               "package tutorial;",
               paste0("extend Person {\n  optional string nationality = 100;\n}")),
             extend.proto)
@

<<echo=FALSE>>=
writeLines(readLines(extend.proto))
@

After importing this new \texttt{.proto} file with an extension
defined, we can get or set the value of this extension in Messages of
type \texttt{Person}.

<<>>=
library(RProtoBuf)
readProtoFiles(extend.proto)
person <- new(tutorial.Person, id=1, name="Murray")
person
person$setExtension(P("tutorial.nationality"), "USA")
cat(as.character(person))
person$getExtension(P("tutorial.nationality"))
@

The character output of this message places the extension type in
brackets to differentiate it from other fields in the message, and so
we know we need to use the \texttt{getExtension} method rather than a
'\$' field extractor to get the value.

\subsection{Descriptor lookup}
\label{sec-lookup}

The \texttt{RProtoBuf} package uses the user defined tables framework
that is defined as part of the \texttt{RObjectTables} package available
from the OmegaHat project.

The feature allows \texttt{RProtoBuf} to install the
special environment \emph{RProtoBuf:DescriptorPool} in the R search path.
The environment is special in that, instead of being associated with a
static hash table, it is dynamically queried by R as part of R's usual
variable lookup. In other words, it means that when the R interpreter
looks for a binding to a symbol (foo) in its search path,
it asks to our package if it knows the binding "foo", this is then
implemented by the \texttt{RProtoBuf} package by calling an internal
method of the \texttt{protobuf} C++ library.

\subsection{64-bit integer issues}
\label{sec:int64}

R does not have native 64-bit integer support.  Instead, R treats
large integers as doubles which have limited precision.  For example,
it loses the ability to distinguish some distinct integers:

<<>>=
2^53 == (2^53 + 1)
@

Protocol Buffers are frequently used to pass data between different
systems, however, and most other systems these days have support for
64-bit integers.  To work around this, RProtoBuf allows users to get
and set 64-bit integer types by treating them as characters when
running on a platform with a 64-bit long long type available.

<<echo=FALSE,print=FALSE>>=
if (!exists("protobuf_unittest.TestAllTypes",
            "RProtoBuf:DescriptorPool")) {
    unittest.proto.file <- system.file("unitTests", "data",
                                       "unittest.proto",
                                       package="RProtoBuf")
    readProtoFiles(file=unittest.proto.file)
}
@

If we try to set an int64 field in R to double values, we lose
precision:

% We comment out the raw Sweave here because it doesn't work on 32-bit
% platforms but it is important to illustrate what the user is missing
% out on so we hard code the output from a platform with 64-bit long
% long type.
%
% <<>>=
%  test <- new(protobuf_unittest.TestAllTypes)
%  test$repeated_int64 <- c(2^53, 2^53+1)
%  length(unique(test$repeated_int64))
% @

\begin{Schunk}
\begin{Sinput}
>   test <- new(protobuf_unittest.TestAllTypes)
>   test$repeated_int64 <- c(2^53, 2^53+1)
>   length(unique(test$repeated_int64))
\end{Sinput}
\begin{Soutput}
[1] 1
\end{Soutput}
\end{Schunk}

However, we can specify the values as character strings so that the
C++ library on which RProtoBuf is based can store a true 64-bit
integer representation of the data.

% <<>>=
%   test$repeated_int64 <- c("9007199254740992", "9007199254740993")
% @
\begin{Schunk}
\begin{Sinput}
>   test$repeated_int64 <- c("9007199254740992", "9007199254740993")
\end{Sinput}
\end{Schunk}

When reading the value back into R, numeric types are returned by
default, but when the full precision is required a character value
will be returned if the \texttt{RProtoBuf.int64AsString} option is set
to \texttt{TRUE}.

% TODO: the if statement prevents us from seeing line by line
% output inside the block (E.g. the output of test$repeated_int64).
% This reduces the usefulness of this example when we are on a 64-bit
% platform.  E.g. only the final ``2'' is returned.
% <<echo=TRUE,print=TRUE>>=
%  options("RProtoBuf.int64AsString" = FALSE)
%  test$repeated_int64
%  length(unique(test$repeated_int64))
%  options("RProtoBuf.int64AsString" = TRUE)
%  test$repeated_int64
%  length(unique(test$repeated_int64))
%@
\begin{Schunk}
\begin{Sinput}
>   options("RProtoBuf.int64AsString" = FALSE)
\end{Sinput}
\begin{Soutput}
$RProtoBuf.int64AsString
[1] FALSE
\end{Soutput}
\begin{Sinput}
>   test$repeated_int64
\end{Sinput}
\begin{Soutput}
[1] 9.007199e+15 9.007199e+15
\end{Soutput}
\begin{Sinput}
>   length(unique(test$repeated_int64))
\end{Sinput}
\begin{Soutput}
[1] 1
\end{Soutput}
\begin{Sinput}
>   options("RProtoBuf.int64AsString" = TRUE)
\end{Sinput}
\begin{Soutput}
$RProtoBuf.int64AsString
[1] FALSE
\end{Soutput}
\begin{Sinput}
>   test$repeated_int64
\end{Sinput}
\begin{Soutput}
[1] "9007199254740992" "9007199254740993"
\end{Soutput}
\begin{Sinput}
>   length(unique(test$repeated_int64))
\end{Sinput}
\begin{Soutput}
[1] 2
\end{Soutput}
\end{Schunk}

<<echo=FALSE,print=FALSE>>=
options("RProtoBuf.int64AsString" = FALSE)
@

\subsection{Deprecated Feature: Protocol Buffer Groups}
\label{sec:groups}

Groups are a deprecated feature that offered another way to nest
information in message definitions.  For example, the
\texttt{TestAllTypes} message type in \texttt{unittest.proto} includes
an OptionalGroup type:

\begin{verbatim}
  optional group OptionalGroup = 16 {
    optional int32 a = 17;
  }
\end{verbatim}

And although the feature is deprecated, it can be used with RProtoBuf:

<<echo=TRUE>>=
test <- new(protobuf_unittest.TestAllTypes)
test$optionalgroup$a <- 3
test$optionalgroup$a
cat(as.character(test))
@

Note that groups simply combine a nested message type and a field into
a single declaration.  The field type is OptionalGroup in this
example, and the field name is converted to lower-case 'optionalgroup'
so as not to conflict with the type name.

\section{Other approaches}

Saptarshi Guha wrote another package that deals with integration
of protocol buffer messages with R, taking a different angle :
serializing any R object as a message, based on a single catch-all
\texttt{proto} file.  Saptarshi's package is available at
\url{http://ml.stat.purdue.edu/rhipe/doc/html/ProtoBuffers.html}.

Jeroen Ooms took a similar approach influenced by Saptarshi in his
\texttt{RProtoBufUtils} package.  Unlike Saptarshi's package,
RProtoBufUtils depends on RProtoBuf for underlying message operations.
This package is available at
\url{https://github.com/jeroenooms/RProtoBufUtils}.

% Phillip Yelland wrote another implementation, currently proprietary,
% that has significant speed advantages when querying fields from a
% large number of protocol buffers, but is less user friendly for the
% basic cases documented here.

\section{Plans for future releases}
Protocol buffers have a mechanism for remote procedure calls (rpc)
that is not yet used by \texttt{RProtoBuf}, but we may one day
take advantage of this by writing a protocol buffer message R server,
and client code as well, probably based on the functionality
of the \texttt{Rserve} package.

Extensions have been implemented in RProtoBuf and have been
extensively used and tested, but they are not currently described in
this vignette.  Additional examples and documentation are needed for
extensions.

% \section{Troubleshouting}

\section{Acknowledgments}

% protobuf ?
% saptarshi ?

Some of the design of the package is based on the design of the
\texttt{rJava} package by Simon Urbanek (dispatch on new, S4 class
structures using external pointers, etc ...). We'd like to thank
Simon for his indirect involvment on \texttt{RProtoBuf}.

The user defined table mechanism, implemented by Duncan Temple Lang
for the purpose of the \texttt{RObjectTables} package allowed the
dynamic symbol lookup (see section~\ref{sec-lookup}). Many thanks
to Duncan for this amazing feature.


\end{document}

\documentclass[10pt]{article}
%\VignetteIndexEntry{RProtoBuf-unitTests}
%\VignetteKeywords{RProtoBuf, Protocol Buffers, package}
%\VignetteDepends{RProtoBuf}

\usepackage{vmargin}
\setmargrb{0.75in}{0.75in}{0.75in}{0.75in}

<<echo=FALSE,print=FALSE>>=
require(RProtoBuf)
prettyVersion <- packageDescription("RProtoBuf")$Version
prettyDate <- format(Sys.Date(), "%B %e, %Y")
library(RUnit)
@

\usepackage{microtype}                     %% cf http://www.khirevich.com/latex/microtype/
\usepackage[T1]{fontenc}		   %% cf http://www.khirevich.com/latex/font/
\usepackage[bitstream-charter]{mathdesign} %% cf http://www.khirevich.com/latex/font/

\usepackage[colorlinks]{hyperref}
\author{Romain Fran\c{c}ois \and Dirk Eddelbuettel \and Murray Stokely}
\title{RProtoBuf : Unit testing results}
\date{RProtoBuf version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}

\begin{document}
\maketitle

\section*{Test Execution}

<<unitTesting,echo=FALSE,print=FALSE>>=
pkg <- "RProtoBuf"

if (file.exists("unitTests-results")) unlink("unitTests-results", recursive = TRUE)
dir.create("unitTests-results")
pathRcppTests <<- system.file("unitTests", package = pkg)
path <- system.file("unitTests", package=pkg)
testSuite <- defineTestSuite(name=paste(pkg, "unit testing"), dirs=path)
tests <- runTestSuite(testSuite)
err <- getErrors(tests)
if (err$nFail > 0) cat(sprintf("unit test problems: %d failures", err$nFail))
if (err$nErr > 0) cat(sprintf("unit test problems: %d errors", err$nErr))
printHTMLProtocol(tests, fileName=sprintf("unitTests-results/%s-unitTests.html", pkg))
printTextProtocol(tests, fileName=sprintf("unitTests-results/%s-unitTests.txt" , pkg))

#if (file.exists("/tmp")) {
#    invisible(sapply(c("txt", "html"), function(ext) {
#        fname <- sprintf("unitTests-results/%s-unitTests.%s", pkg, ext)
#        file.copy(fname, "/tmp", overwrite=TRUE)
#    }))
#}
@

\section*{Test Results}

\begin{verbatim}
<<importResults,echo=FALSE,results=tex>>=
results <- sprintf("unitTests-results/%s-unitTests.txt", pkg)
if (file.exists(results)) {
    writeLines(readLines(results))
} else{
    writeLines("Unit test results not available")
}
@

\end{verbatim}

\end{document}

\name{ArrayOutputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{ArrayOutputStream-class}

\title{Class "ArrayOutputStream"  }
\description{A \linkS4class{ZeroCopyOutputStream} backed by an in-memory array of bytes}
\section{Objects from the Class}{
Objects can be created by the \link{ArrayOutputStream} function
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{ External pointer to the \code{google::protobuf::io::ArrayOutputStream} C++ object}
  }
}
\section{Extends}{
Class \code{"\linkS4class{ZeroCopyOutputStream}"}, directly.
}
\section{Methods}{
See \linkS4class{ZeroCopyOutputStream}
}
\references{ The \code{ArrayOutputStream} class from the protobuf C++ library.}
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\linkS4class{ZeroCopyOutputStream} for methods
}
\keyword{classes}

\name{SetCloseOnDelete-methods}
\docType{methods}
\alias{SetCloseOnDelete}
\alias{SetCloseOnDelete-methods}
\title{set the close on delete behavior}
\description{
By default, the file descriptor is not closed when a stream is destroyed, 
use \code{SetCloseOnDelete( stream, TRUE ) } to change that.
}
\section{Methods}{
	See classes \linkS4class{FileInputStream} and 
	\linkS4class{FileOutputStream} for implementations.
}
\keyword{methods}

\name{sizegets}
\docType{methods}
\alias{size<-}
\alias{size<--methods}
\alias{size<-,Message-method}
\title{Set the size of a field}
\description{
	Sets the size of a repeated field.
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{ sets the size of a message field }
}}
\keyword{methods}

\name{size-methods}
\docType{methods}
\alias{size}
\alias{size-methods}
\alias{size,Message-method}
\title{Size of a message field}
\description{
The number of object currently in a given field of a protocol buffer
message. 

For non repeated fields, the size is 1 if the message has the field, 0 otherwise. 

For repeated fields, the size is the number of objects in the array.

For repeated fields, the size can also be assigned to in order to shrink
or grow the vector.  Numeric types are given a default value of 0 when
the new size is greater than the existing size.  Character types are
given a default value of "".  Growing a repeated field in this way is
not supported for message, group, and enum types.
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{Number of objects in a message field}
}}
\examples{
unitest.proto.file <- system.file("tinytest", "data", "unittest.proto",
				  package = "RProtoBuf" )
readProtoFiles(file = unitest.proto.file)

test <- new(protobuf_unittest.TestAllTypes)
test$size("optional_int32")

test$add("repeated_int32", 1:10)
test$size("repeated_int32")
test$repeated_int32

size(test, "repeated_int32") <- 5
test$repeated_int32

size(test, "repeated_int32") <- 15
test$repeated_int32
}
\keyword{methods}

\name{FileOutputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{FileOutputStream-class}
\alias{close,FileOutputStream-method}
\alias{flush,FileOutputStream-method}
\alias{GetErrno,FileOutputStream-method}
\alias{SetCloseOnDelete,FileOutputStream-method}

\title{Class "FileOutputStream"  }
\description{A \linkS4class{ZeroCopyOutputStream} reading from a file}
\section{Objects from the Class}{
Objects can be created by the \link{FileOutputStream} function
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{ External pointer to the \code{google::protobuf::io::FileOutputStream} C++ object}
  }
}
\section{Extends}{
Class \code{"\linkS4class{ZeroCopyOutputStream}"}, directly.
}
\section{Methods}{
	\describe{
		\item{close}{\code{signature(con="FileOutputStream")}:
			Flushes any buffers and closes the underlying file.
			Returns false if an error occurs during the process; use \code{GetErrno} to examine the error
		}
		\item{flush}{\code{signature(con="FileOutputStream")}:
			Flushes FileOutputStream's buffers but does not close the underlying file
		}
		\item{GetErrno}{\code{signature(object="FileInputStream")}:
			If an I/O error has occurred on this file descriptor, this is the errno from that error.
			Otherwise, this is zero. Once an error occurs, the stream is broken and all subsequent operations will fail.
		}
		\item{SetCloseOnDelete}{\code{signature(object="FileOutputStream")}:
			set the close on delete behavior.
		}
	}
	See \linkS4class{ZeroCopyOutputStream} for inherited methods
}
\references{ The \code{FileOutputStream} class from the protobuf C++ library.}
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\linkS4class{ZeroCopyOutputStream} for methods
}
\keyword{classes}

\name{FileInputStream-methods}
\docType{methods}
\alias{FileInputStream}
\alias{FileInputStream-methods}
\alias{FileInputStream,character,integer,logical-method}
\title{Creates an FileInputStream}
\description{
Constructor for \linkS4class{FileInputStream} objects
}
\section{Methods}{
\describe{
\item{\code{signature(filename = "character", block_size = "logical", close.on.delete = "logical" )}}{Creates a \linkS4class{FileInputStream} reading from the given file.}
}}
\keyword{methods}

\name{ArrayInputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{ArrayInputStream-class}

\title{Class "ArrayInputStream"  }
\description{A \linkS4class{ZeroCopyInputStream} backed by an in-memory array of bytes}
\section{Objects from the Class}{
Objects can be created by the \link{ArrayInputStream} function
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{ External pointer to the \code{google::protobuf::io::ArrayInputStream} C++ object}
  }
}
\section{Extends}{
Class \code{"\linkS4class{ZeroCopyInputStream}"}, directly.
}
\section{Methods}{
See \linkS4class{ZeroCopyInputStream}
}
\references{ The \code{ArrayInputStream} class from the protobuf C++ library.}
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\linkS4class{ZeroCopyInputStream} for methods
}
\examples{
stream <- ArrayInputStream(as.raw(0:10))
stream$ReadRaw(5)

stringsstream <- ArrayInputStream(as.raw(c(0x74, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x67)))
stringsstream$ReadString(7)

intstream <- ArrayInputStream(as.raw(c(0x9e, 0xa7, 0x05)))
intstream$ReadVarint32()
}
% local({
% 	stream <- ArrayInputStream( as.raw(0:10) )
% 	stopifnot( identical( stream$Next(), as.raw(0:10) ) )
% 	stream$BackUp( 5 )
% 	stopifnot( identical( stream$Next(), as.raw(6:10) ) )
%
% 	# limiting the block size
% 	stream <- ArrayInputStream( as.raw(0:10), 5 )
% 	stopifnot( identical(length( stream$Next() ), 5L ) )
% })
\keyword{classes}

\name{with.Message}
\alias{with.Message}
\alias{within.Message}
\title{
with and within methods for protocol buffer messages
}
\description{
Convenience wrapper that allow getting and setting 
fields of protocol buffer messages from within the object
}
\usage{
\S3method{with}{Message}(data, expr, ...)
\S3method{within}{Message}(data, expr, ...)
}
\arguments{
	\item{data}{A protocol buffer message, instance of \linkS4class{Message} }
	\item{expr}{R expression to evaluate}
	\item{\dots}{ignored}
}
\details{
The expression is evaluated in an environment 
that allows to set and get fields of the message

The fields of the message are mapped to active bindings
(see \link{makeActiveBinding}) so that they can be accessed
and modified from within the environment. 
}
\value{
	\code{with} returns the value of the expression and 
	\code{within} returns the \code{data} argument.
}
\author{
Romain Francois <francoisromain@free.fr>
}
\examples{
\dontrun{
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" ) 
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P( "tutorial.Person") }
romain <- within( new( Person ), {
	email <- "francoisromain@free.fr"
	id <- 10L
} )
}
\keyword{ classes }

\name{EnumValueDescriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{EnumValueDescriptor-class}
\alias{show,EnumValueDescriptor-method}
\alias{as.character,EnumValueDescriptor-method}
\alias{toString,EnumValueDescriptor-method}
\alias{$,EnumValueDescriptor-method}
\alias{name,EnumValueDescriptor-method}
\alias{number,EnumValueDescriptor-method}
\alias{enum_type,EnumValueDescriptor,missing,missing-method}

\title{Class "EnumValueDescriptor"  }
\description{ R representation of an enum value descriptor. This
  is a thin wrapper around the \code{EnumValueDescriptor} c++ class. }
\section{Objects from the Class}{
  Objects of this class are typically retrieved with the
  \code{value} method of the \linkS4class{EnumDescriptor} class
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to the \code{EnumValueDescriptor} instance}
  	  \item{\code{name}:}{ simple name of the enum }
  	  \item{\code{full_name}:}{fully qualified name }
  }
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "EnumValueDescriptor")}: small information }
    \item{as.character}{\code{signature(x = "EnumValueDescriptor")}:
      returns the debug string of the enum descriptor.
      This is retrieved by a call to the \code{DebugString}
      method of the EnumDescriptor object.  }
    \item{toString}{\code{signature(x = "EnumValueDescriptor")}: same as \code{as.character} }
    \item{$}{\code{signature(x = "EnumValueDescriptor")}: invoke pseudo
      methods }
    \item{name}{\code{signature(object = "EnumValueDescriptor", full = "logical")}:
      return the name of this enum constant.}
    \item{number}{\code{signature(object = "EnumValueDescriptor")}:
      return the numeric value of this enum constant.}
    \item{enum_type}{\code{signature(object = "EnumDescriptor")} : retrieves the \linkS4class{EnumDescriptor} related to this value descriptor.}
  }

}
\author{ Romain Francois <francoisromain@free.fr> }
\examples{
\dontrun{
# example proto file supplied with this package
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )
# reading a proto file and creating the descriptor
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P( "tutorial.Person" ) }
# enum type
Person$PhoneType

# enum value type
value(Person$PhoneType, 1)

name(value(Person$PhoneType, 1))
name(value(Person$PhoneType, 1), TRUE)

number(value(Person$PhoneType, number=1))

enum_type(value(Person$PhoneType, number=1))
}
\keyword{classes}

\name{isInitialized-methods}
\docType{methods}
\alias{isInitialized}
\alias{isInitialized-methods}
\alias{isInitialized,Message-method}
\title{Indicates if a protocol buffer message is initialized}
\description{
	Indicates if a \linkS4class{Message} is 
	initialized. A message is initialized if all its 
	required fields are set.
}
\section{Methods}{
\describe{
\item{\code{ signature(object = "Message")}}{ is the message initialized}
}}
\examples{
message <- new( tutorial.Person, name = "" )
isInitialized( message ) # FALSE (id is not set)
message$isInitialized()  # FALSE

message <- new( tutorial.Person, name = "", id = 2 )
isInitialized( message ) # TRUE
message$isInitialized()  # TRUE

}
\keyword{methods}

\name{ServiceDescriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{ServiceDescriptor-class}
\alias{as.character,ServiceDescriptor-method}
\alias{toString,ServiceDescriptor-method}
\alias{show,ServiceDescriptor-method}
\alias{$,ServiceDescriptor-method}
\alias{[[,ServiceDescriptor-method}
\alias{length,ServiceDescriptor-method}

\alias{method_count}
\alias{method_count-methods}
\alias{method_count,ServiceDescriptor-method}

\alias{method}
\alias{method-methods}
\alias{method,ServiceDescriptor-method}

\title{Class "ServiceDescriptor"}
\description{R representation of Service Descriptors}
\section{Objects from the Class}{
TODO
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{External pointer to a \code{google::protobuf::ServiceDescriptor} C++ object }
  	  \item{\code{name}:}{fully qualified name of the service }
  }
}
\section{Methods}{
\describe{
    \item{as.character}{\code{signature(x = "ServiceDescriptor")}: debug string of the service }
    \item{toString}{\code{signature(x = "ServiceDescriptor")}: debug string of the service}
    \item{show}{\code{signature(x = "ServiceDescriptor")}: ... }
    \item{$}{\code{signature(x = "ServiceDescriptor")}: invoke pseudo methods or retrieve
    	method descriptors contained in this service descriptor. }
	\item{[[}{\code{signature(x = "ServiceDescriptor")}: extracts methods
	descriptors contained in this service descriptor}
	\item{length}{\code{signature(x = "ServiceDescriptor")}: number of \linkS4class{MethodDescriptor} }
	\item{method_count}{\code{signature(x = "ServiceDescriptor")}: number of \linkS4class{MethodDescriptor} }
	\item{method}{\code{signature(x = "ServiceDescriptor")}: retrieves a \linkS4class{MethodDescriptor} }
	}
}
\author{ Romain Francois <francoisromain@free.fr> }
\keyword{classes}

\name{Next-methods}
\docType{methods}
\alias{Next}
\alias{Next-methods}
\title{Obtains a chunk of data from the stream}
\description{
Obtains a chunk of data from the stream
}
\seealso{
	\linkS4class{ZeroCopyInputStream} implements Next.
}
\keyword{methods}

\name{name}
\docType{methods}
\alias{name}
\alias{name-methods}
\alias{name,Descriptor-method}
\alias{name,FieldDescriptor-method}
\alias{name,EnumDescriptor-method}
\alias{name,ServiceDescriptor-method}
\alias{name,MethodDescriptor-method}

\title{Name or full name of a descriptor}
\description{
name or full name of a descriptor
}
\section{Methods}{
\describe{
	\item{\code{signature(object = "Descriptor")}}{ ... }
	\item{\code{signature(object = "FieldDescriptor")}}{ ... }
	\item{\code{signature(object = "EnumDescriptor")}}{ ... }
	\item{\code{signature(object = "ServiceDescriptor")}}{ ... }
	\item{\code{signature(object = "MethodDescriptor")}}{ ... }
}}
\keyword{methods}

\name{readJSON-methods}
\docType{methods}
\alias{readJSON}
\alias{readJSON-methods}
\alias{readJSON,Descriptor,ANY-method}
\alias{readJSON,Descriptor,character-method}
\title{read a message in JSON format}
\description{
 Method to read a Message in JSON format
}
\section{Methods}{
\describe{

\item{\code{signature(descriptor = "Descriptor", input = "ANY")}}{
Read the message from a connection (file, etc ...)
}

\item{\code{signature(descriptor = "Descriptor", input = "character")}}{
Read the message directly from the character string
}
}}
\keyword{methods}
\examples{
\dontrun{
# example file that contains a "tutorial.AddressBook" message
book <- system.file( "examples", "addressbook.pb", package = "RProtoBuf" )

# read the message
message <- read( tutorial.AddressBook, book )

# Output in text format to a temporary file
out.file <- tempfile()
writeLines( message$toJSON(), file(out.file))

# Verify that we can read back in the message from a text file.
message2 <- readJSON( tutorial.AddressBook, file(out.file, "rb"))

# Verify that we can read back in the message from an unopened file.
message3 <- readJSON( tutorial.AddressBook, file(out.file))

\dontshow{
stopifnot( identical( message, message2) )
}
}
}

\name{FileOutputStream-methods}
\docType{methods}
\alias{FileOutputStream}
\alias{FileOutputStream-methods}
\alias{FileOutputStream,character,integer,logical-method}
\title{Creates an FileOutputStream}
\description{
Constructor for \linkS4class{FileOutputStream} objects
}
\section{Methods}{
\describe{
\item{\code{signature(filename = "character", block_size = "logical", close.on.delete = "logical" )}}{Creates a \linkS4class{FileOutputStream} writing to the given file.}
}}
\keyword{methods}

\name{FieldDescriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{FieldDescriptor-class}
\alias{show,FieldDescriptor-method}
\alias{$,FieldDescriptor-method}
\alias{as.character,FieldDescriptor-method}
\alias{toString,FieldDescriptor-method}
\alias{containing_type,FieldDescriptor-method}
\alias{is_extension,FieldDescriptor-method}
\alias{number,FieldDescriptor-method}
\alias{type,FieldDescriptor-method}
\alias{cpp_type,FieldDescriptor-method}
\alias{label,FieldDescriptor-method}

\alias{is_required}
\alias{is_optional}
\alias{is_repeated}
\alias{is_required-methods}
\alias{is_optional-methods}
\alias{is_repeated-methods}
\alias{is_required,FieldDescriptor-method}
\alias{is_optional,FieldDescriptor-method}
\alias{is_repeated,FieldDescriptor-method}

\alias{has_default_value}
\alias{has_default_value-methods}
\alias{has_default_value,FieldDescriptor-method}

\alias{default_value}
\alias{default_value-methods}
\alias{default_value,FieldDescriptor-method}

\alias{message_type}
\alias{message_type-methods}
\alias{message_type,FieldDescriptor-method}

\alias{enum_type,FieldDescriptor,missing,missing-method}


\title{Class "FieldDescriptor"}
\description{R representation of message type field descriptor. 
This is a thin wrapper around the C++ class \code{FieldDescriptor} }
\section{Objects from the Class}{
Objects typically are retrieved from \linkS4class{FieldDescriptor}
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to the \code{FieldDescriptor} c++ object }
  	  \item{\code{name}:}{ name of the field within the message type }
  	  \item{\code{full_name}:}{Fully qualified name of the field}
  	  \item{\code{type}:}{Fully qualified name of the type that contains this field }
  }
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "FieldDescriptor")}: small description }
    \item{as.character}{\code{signature(x = "FieldDescriptor")}: 
  	  	  returns the debug string of the field descriptor. 
  	  	  This is retrieved by a call to the \code{DebugString}
  	      method of the FieldDescriptor object.  }
  	\item{toString}{\code{signature(x = "FieldDescriptor")}: same as \code{as.character} }
  	\item{$}{\code{signature(x = "FieldDescriptor")}: used to invoke pseudo methods }
    \item{containing_type}{\code{signature(object = "FieldDescriptor")} : returns a \linkS4class{Descriptor} of the message type that contains this field descriptor.}
    \item{is_extension}{\code{signature(object = "FieldDescriptor")} : indicates if this is an extension.}
    \item{number}{\code{signature(object = "FieldDescriptor")} : gets the declared tag number of this field.}
	\item{type}{\code{signature(object = "FieldDescriptor")} : type of this field.}
	\item{cpp_type}{\code{signature(object = "FieldDescriptor")} : c++ type of this field.}
	\item{label}{\code{signature(object = "FieldDescriptor")} : label of this field.}
	\item{is_required}{\code{signature(object = "FieldDescriptor")} : is this field required.}
	\item{is_optional}{\code{signature(object = "FieldDescriptor")} : is this field optional.}
	\item{is_repeated}{\code{signature(object = "FieldDescriptor")} : is this field repeated.}
	\item{has_default_value}{\code{signature(object = "FieldDescriptor")} : indicates if this field has a default value.}
	\item{default_value}{\code{signature(object = "FieldDescriptor")} : the default value of this field.}
	\item{message_type}{\code{signature(object = "FieldDescriptor")} : the \linkS4class{Descriptor} for the associated message type. Generates an error if this field is not a message type field.}
	\item{enum_type}{\code{signature(object = "FieldDescriptor")} : the \linkS4class{EnumDescriptor} for the associated enum type.Generates an error if this field is not an enum type field }
  	}
}
\references{ The \code{FieldDescriptor} C++ class }
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\linkS4class{Descriptor}
}
\examples{
\dontrun{
# example proto file supplied with this package
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" ) 

# reading a proto file and creating the descriptor
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P("tutorial.Person") }

# field descriptor object
Person$email

# debug string
as.character( Person$email )

# or as a pseudo method
Person$email$as.character()

Person$email$is_required()
Person$email$is_optional()
Person$email$is_repeated()

Person$email$has_default_value()
Person$email$default_value()

Person$email$is_extension()

# Get the default values
has_default_value(Person$id)
has_default_value(Person$email)
has_default_value(Person$phone)
default_value(Person$id)
default_value(Person$email)
default_value(Person$phone)

# Get the types of field descriptors
type(Person$id)
type(Person$id, as.string=TRUE)
cpp_type(Person$email)
cpp_type(Person$email, TRUE)

# Get the label of a field descriptor
label(Person$id)
label(Person$email)
label(Person$phone)
label(Person$id, TRUE)
label(Person$email, TRUE)
label(Person$phone, TRUE)
LABEL_OPTIONAL
LABEL_REQUIRED
LABEL_REPEATED

# Test if a field is optional
is_optional(Person$id)
is_optional(Person$email)
is_optional(Person$phone)

# Test if a field is repeated
is_repeated(Person$id)
is_repeated(Person$email)
is_repeated(Person$phone)

# Test if a field is required
is_required(Person$id)
is_required(Person$email)
is_required(Person$phone)

# Return the class of a message field
message_type(Person$phone)
}
\keyword{classes}

\name{ConnectionInputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{ConnectionInputStream-class}

\title{Class "ConnectionInputStream"  }
\description{A \linkS4class{ZeroCopyInputStream} reading from a binary R connection}
\section{Objects from the Class}{
Objects can be created by the \link{ConnectionInputStream} function 
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{ External pointer to the \code{rprotobuf::ConnectionInputStream} C++ object}
  }
}
\section{Extends}{
Class \code{"\linkS4class{ZeroCopyInputStream}"}, directly.
}
\section{Methods}{
See \linkS4class{ZeroCopyInputStream}
}
\references{ The internal C++ class \code{ConnectionInputStream} }
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\linkS4class{ZeroCopyInputStream} for methods
}
\keyword{classes}

\name{Message-class}
\Rdversion{1.1}
\docType{class}
\alias{Message-class}
\alias{as.character,Message-method}
\alias{$<-,Message-method}
\alias{$,Message-method}
\alias{[[,Message-method}
\alias{[[<-,Message-method}
\alias{serialize,Message-method}
\alias{show,Message-method}
\alias{update,Message-method}
\alias{length,Message-method}
\alias{names,Message-method}
\alias{str,Message-method}
\alias{toString,Message-method}
\alias{toJSON}
\alias{toJSON,Message-method}
\alias{identical,Message,Message-method}
\alias{==,Message,Message-method}
\alias{!=,Message,Message-method}
\alias{all.equal,Message,Message-method}
\alias{getExtension,Message-method}
\alias{setExtension,Message-method}
\alias{getExtension}
\alias{setExtension}

\title{Class "Message" }
\description{
R representation of protocol buffer messages. This is a thin wrapper
around the \code{Message} c++ class that holds the actual message
as an external pointer.
}
\section{Objects from the Class}{
	Objects are typically created by the \code{new} function invoked
	on a \linkS4class{Descriptor} object.
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to the c++ \code{Message} object}
  	  \item{\code{type}:}{fully qualified name of the message type}
  }
}
\section{Methods}{
  \describe{
    \item{as.character}{\code{signature(x = "Message")}: returns the debug string of the message.
    This is built from a call to the \code{DebugString} method of the \code{Message} object}
    \item{toString}{\code{signature(x = "Message")}: same as \code{as.character} }
    \item{toTextFormat}{\code{signature(x = "Message")}: returns the TextFormat of the message.
    	This is built from a call to \code{TextFormat::PrintToString} with the \code{Message} object}
    \item{toDebugString}{\code{signature(x = "Message")}: same as \code{as.character} }
    \item{toJSON}{\code{signature(x = "Message")}: returns the JSON representation of the message.
    This is built from a call to the
    \code{google::protobuf::util::MessageToJsonString} method and
    accepts two arguments \code{preserve_proto_field_names} - if FALSE (the default) convert field names to camelCase
    \code{always_print_primitive_fields} - whether to return the default
    value for missing primitive fields (default false)}
    \item{$<-}{\code{signature(x = "Message")}: set the value of a field of the message. }
    \item{$}{\code{signature(x = "Message")}: gets the value of a field.
    Primitive types are brought back to R as R objects of the closest matching R type.
    Messages are brought back as instances of the \code{Message} class.}
    \item{[[}{\code{signature(x = "Message")}: extracts a field identified by its name or declared tag number }
    \item{[[<-}{\code{signature(x = "Message")}: replace the value of a field identified by its name or declared tag number }
	\item{serialize}{\code{signature(object = "Message")}: serialize a message. If the
    	"connection" argument is \code{NULL}, the payload of the message is returned as a raw vector,
    	if the "connection" argument is a binary writable connection, the payload is written into the
       connection. If "connection" is a character vector, the message is sent to
       the file (in binary format). }
    \item{show}{\code{signature(object = "Message")}: displays a short text about the message }
    \item{update}{\code{signature(object = "Message")}: set several fields of the message at once }
    \item{length}{\code{signature(x = "Message")}: The number of fields actually contained in the message.
    A field counts in these two situations: the field is repeated and the field size is greater than 0,
    the field is not repeated and the message has the field.}
    \item{setExtension}{\code{signature(object = "Message")}: set an
      extension field of the Message.}
    \item{getExtension}{\code{signature(object = "Message")}: get the
      value of an extension field of the Message.}
    \item{str}{\code{signature(object = "Message")}: displays the structure of the message }
    \item{identical}{\code{signature(x = "Message", y = "Message")}: Test if two messages are exactly identical }
    \item{==}{\code{signature(e1 = "Message", e2 = "Message")}: Same as \code{identical} }
    \item{!=}{\code{signature(e1 = "Message", e2 = "Message")}: Negation of \code{identical} }
    \item{all.equal}{\code{signature(e1 = "Message", e2 = "Message")}: Test near equality }
    \item{names}{\code{signature(x = "Message")}: extracts the names of the message. }

  }
}
\references{The \code{Message} class from the C++ proto library.}
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\code{\link{P}} creates objects of class \linkS4class{Descriptor} that
	can be used to create messages.
}
\examples{
\dontrun{
# example proto file supplied with this package
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )

# reading a proto file and creating the descriptor
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{ Person <- P("tutorial.Person") }

PhoneNumber <- P( "tutorial.Person.PhoneNumber" )

# creating a prototype message from the descriptor
p <- new( Person )
p$email # not set, returns default value
p$id    # not set, returns default value
as.character( p ) # empty
has( p, "email" ) # is the "email" field set
has( p, "phone" ) # is the "email" field set
length( p )       # number of fields actually set

# update several fields at once
romain <- update( new( Person ),
	email = "francoisromain@free.fr",
	id = 1,
	name = "Romain Francois",
	phone = new( PhoneNumber , number = "+33(0)...", type = "MOBILE" )
	)

# supply parameters to the constructor
dirk <- new( Person,
	email = "edd@debian.org",
	id = 2,
	name = "Dirk Eddelbuettel" )
# update the phone repeated field with a list of PhoneNumber messages
dirk$phone <- list(
	new( PhoneNumber , number = "+01...", type = "MOBILE" ),
	new( PhoneNumber , number = "+01...", type = "HOME" ) )

# with/within style
saptarshi <- within( new(Person), {
	id <- 3
	name <- "Saptarshi Guha"
	email <- "saptarshi.guha@gmail.com"
} )

# make an addressbook
book <- new( tutorial.AddressBook, person = list( romain, dirk, saptarshi ) )

# serialize the message to a file
tf <- tempfile( )
serialize( book, tf )

# the payload of the message
serialize( book, NULL )

# read the file into a new message
m <- tutorial.AddressBook$read( tf )
writeLines( as.character( m ) )
sapply( m$person, function(p) p$name )
}
\keyword{classes}

\name{EnumDescriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{EnumDescriptor-class}
\alias{show,EnumDescriptor-method}
\alias{has,EnumDescriptor-method}
\alias{as.character,EnumDescriptor-method}
\alias{toString,EnumDescriptor-method}
\alias{$,EnumDescriptor-method}
\alias{containing_type,EnumDescriptor-method}
\alias{length,EnumDescriptor-method}

\alias{value_count}
\alias{value_count-methods}
\alias{value_count,EnumDescriptor-method}

\alias{value}
\alias{value-methods}
\alias{value,EnumDescriptor-method}

\alias{[[,EnumDescriptor-method}
\alias{names,EnumDescriptor-method}

\title{Class "EnumDescriptor"  }
\description{ R representation of an enum descriptor. This 
is a thin wrapper around the \code{EnumDescriptor} c++ class. }
\section{Objects from the Class}{
Objects of this class are typically retrieved as members of 
\linkS4class{Descriptor} objects
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to the \code{EnumDescriptor} instance}
  	  \item{\code{name}:}{ simple name of the enum }
  	  \item{\code{full_name}:}{fully qualified name }
  	  \item{\code{type}:}{fully qualified name of the type that contains this enumeration}
  }
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "EnumDescriptor")}: small information }
    \item{as.character}{\code{signature(x = "EnumDescriptor")}: 
      returns the debug string of the enum descriptor. 
      This is retrieved by a call to the \code{DebugString}
      method of the EnumDescriptor object.  }
    \item{toString}{\code{signature(x = "EnumDescriptor")}: same as \code{as.character} }
    \item{$}{\code{signature(x = "EnumDescriptor")}: get the 
      number associated with the name}
    \item{has}{\code{signature(object = "EnumDescriptor")}: indicate if
      the given name is a constant present in this enum.}
    \item{containing_type}{\code{signature(object = "EnumDescriptor")} : returns a \linkS4class{Descriptor} of the message type that contains this enum descriptor, or NULL if this is a top level enum descriptor.}
    \item{length}{\code{signature(x = "EnumDescriptor")} : number of constants in this enum.}
    \item{value_count}{\code{signature(object = "EnumDescriptor")} : number of constants in this enum.}
    \item{value}{\code{signature(object = "EnumDescriptor")} : extracts an \linkS4class{EnumValueDescriptor}.
      Exactly one argument of \code{index}, \code{number} or \code{name} has to be 
      used.
      If \code{index} is used, the enum value descriptor is retrieved
      by position, using the \code{value} method of the C++ class. 
      If \code{number} is used, the enum value descriptor is retrieved
      using the value of the constant, using the \code{FindValueByNumber}
      C++ method. 
      If \code{name} is used, the enum value descriptor is retrieved
      using the name of the constant, using the \code{FindValueByName}
      C++ method.
    }
    \item{[[}{\code{signature(x = "EnumDescriptor")}: extracts field identified by its name or declared tag number}
    \item{names}{\code{signature(x = "EnumDescriptor")} : extracts names of this enum}
  }
  	
}
\references{ The \code{EnumDescriptor} C++ class }
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{The \linkS4class{Descriptor} class}
\examples{
\dontrun{
# example proto file supplied with this package
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" ) 

# reading a proto file and creating the descriptor
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P("tutorial.Person") }
# enum type
Person$PhoneType

has(Person$PhoneType, "MOBILE")
has(Person$PhoneType, "HOME")
has(Person$PhoneType, "WORK")

has(Person$PhoneType, "FOOBAR")

length(Person$PhoneType)
}
\keyword{classes}

\name{merge-methods}
\alias{merge,Message,Message-method}
\title{Merge two messages of the same type}
\description{
	Merge two \linkS4class{Message} objects
	of the same type.
}
\section{Methods}{
\describe{
	\item{\code{signature(x = "Message", y = "Message")}}{ merge two messages of the same type }
}}
\section{Errors}{
	An error of class "IncompatibleType" is thrown if 
	the two messages are not of the same message type.
}
\keyword{methods}
\examples{

m1 <- new( tutorial.Person, email = "francoisromain@free.fr" )
m2 <- new( tutorial.Person, id = 5 )
m3 <- merge( m1, m2 )
writeLines( as.character( m1 ) )
writeLines( as.character( m2 ) )
writeLines( as.character( m3 ) )

}

\name{ArrayOutputStream-methods}
\docType{methods}
\alias{ArrayOutputStream}
\alias{ArrayOutputStream-methods}
\alias{ArrayOutputStream,integer,missing-method}
\alias{ArrayOutputStream,integer,integer-method}
\alias{ArrayOutputStream,integer,numeric-method}
\alias{ArrayOutputStream,numeric,missing-method}
\alias{ArrayOutputStream,numeric,integer-method}
\alias{ArrayOutputStream,numeric,numeric-method}
\title{Creates an ArrayOutputStream}
\description{
Constructor for \linkS4class{ArrayOutputStream} objects
}
\section{Methods}{
\describe{
\item{\code{signature(size = "integer", block_size = "missing" )}}{Creates a \linkS4class{ArrayOutputStream} using of the given size}
\item{\code{signature(size = "integer", block_size = "integer" )}}{Creates a \linkS4class{ArrayOutputStream} ... same with block size. }
\item{\code{signature(size = "integer", block_size = "numeric" )}}{Creates a \linkS4class{ArrayOutputStream} ... same with block size. }
\item{\code{signature(size = "numeric", block_size = "missing" )}}{Creates a \linkS4class{ArrayOutputStream} using of the given size}
\item{\code{signature(size = "numeric", block_size = "integer" )}}{Creates a \linkS4class{ArrayOutputStream} ... same with block size. }
\item{\code{signature(size = "numeric", block_size = "numeric" )}}{Creates a \linkS4class{ArrayOutputStream} ... same with block size. }
}}
\keyword{methods}

\name{Descriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{Descriptor-class}
\alias{as.character,Descriptor-method}
\alias{$<-,Descriptor-method}
\alias{$,Descriptor-method}
\alias{new,Descriptor-method}
\alias{show,Descriptor-method}
\alias{toString,Descriptor-method}
\alias{containing_type,Descriptor-method}
\alias{field_count,Descriptor-method}
\alias{nested_type_count,Descriptor-method}
\alias{enum_type_count,Descriptor-method}
\alias{field,Descriptor-method}
\alias{nested_type,Descriptor-method}
\alias{enum_type,Descriptor,ANY,ANY-method}
\alias{[[,Descriptor-method}
\alias{names,Descriptor-method}
\alias{length,Descriptor-method}

\title{Class "Descriptor" }
\description{ full descriptive information about a protocol buffer
message type. This is a thin wrapper around the
C++ class \code{Descriptor}}
\section{Objects from the Class}{
	Objects are usually created by calls to the \code{\link{P}}
	function.
}
\section{Slots}{
  \describe{
  	\item{\code{pointer}:}{external pointer holding a \code{Descriptor} object }
    \item{\code{type}:}{full name of the corresponding message type}
  }
}
\section{Methods}{
  \describe{
  	  \item{as.character}{\code{signature(x = "Descriptor")}:
  	  	  returns the debug string of the descriptor.
  	  	  This is retrieved by a call to the \code{DebugString}
  	      method of the Descriptor object.  }
  	\item{toString}{\code{signature(x = "Descriptor")}: same as \code{as.character} }
    \item{$}{\code{signature(x = "Descriptor")}: retrieves a
    descriptor for a member of the message type. This can
    either be another "Descriptor" instance describing
    a nested type, or a \linkS4class{EnumDescriptor} object
    describing an enum type, or a \linkS4class{FieldDescriptor}
    object describing a field of the message}
    \item{new}{\code{signature(Class = "Descriptor")}: creates
    a prototype message (\linkS4class{Message}) of this
    descriptor}
    \item{show}{\code{signature(object = "Descriptor")}:
    simple information}
    \item{containing_type}{\code{signature(object = "Descriptor")} : returns a descriptor of the message type that contains this message descriptor, or \code{NULL} if this is a top-level message type.}
	\item{field_count}{\code{signature(object = "Descriptor")} : The number of fields of this message type. }
	\item{nested_type_count}{\code{signature(object = "Descriptor")} : The number of nested types of this message type. }
	\item{enum_type_count}{\code{signature(object = "Descriptor")} : The number of enum types of this message type. }
	\item{field}{\code{signature(object = "Descriptor")} : extract a field descriptor from a descriptor.
		Exactly one argument of \code{index}, \code{number} or \code{name} has to be used.
		If \code{index} is used, the field descriptor is retrieved by
		position, using the \code{field} method of the \code{google::protobuf::Descriptor} C++ class.
		If \code{number} is used, the field descriptor is retrieved using
		the tag number, with the \code{FindFieldByNumber} C++ method.
		If \code{name} is used, the field descriptor is retrieved by name
		using the \code{FindFieldByName}
	}
	\item{nested_type}{\code{signature(object = "Descriptor")} :
			extracts a message type descriptor that is nested
			in this descriptor.
			Exactly one argument of \code{index} of \code{name} has to be used.
			If \code{index} is used, the nested type will be retrieved
			using its position with the \code{nested_type} method
			of the \code{google::protobuf::Descriptor} C++ class.
			If \code{name} is used, the nested type will be retrieved
			using its name, with the \code{FindNestedTypeByName} C++ method
	}
	\item{enum_type}{\code{signature(object = "Descriptor")} :
			extracts an enum type descriptor that is contained
			in this descriptor.
			Exactly one argument of \code{index} of \code{name} has to be used.
			If \code{index} is used, the enum type will be retrieved
			using its position with the \code{enum_type} method
			of the \code{google::protobuf::Descriptor} C++ class.
			If \code{name} is used, the enum type will be retrieved
			using its name, with the \code{FindEnumTypeByName} C++ method
	}
    \item{[[}{\code{signature(x = "Descriptor")}: extracts a field identified by its name or declared tag number}
    \item{names}{\code{signature(x = "Descriptor")} : extracts names of this descriptor}
    \item{length}{\code{signature(x = "Descriptor")} : extracts length of this descriptor}

  }
}
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	the \code{\link{P}} function creates "Descriptor" messages.
}
\examples{
\dontrun{
# example proto file supplied with this package
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )
# reading a proto file and creating the descriptor
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P( "tutorial.Person" ) }

# enum type
Person$PhoneType

# nested type
Person$PhoneNumber

# field
Person$email

# use this descriptor to create a message
new( Person )
}
\keyword{classes}

\name{ZeroCopyOutputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{ZeroCopyOutputStream-class}

\alias{ByteCount,ZeroCopyOutputStream-method}
\alias{BackUp,ZeroCopyOutputStream-method}
\alias{Next,ZeroCopyOutputStream,raw-method}
\alias{$,ZeroCopyOutputStream-method}

\alias{WriteRaw}
\alias{WriteRaw-methods}
\alias{WriteRaw,ZeroCopyOutputStream,raw-method}

\alias{WriteString}
\alias{WriteString-methods}
\alias{WriteString,ZeroCopyOutputStream,character-method}

\alias{WriteLittleEndian32}
\alias{WriteLittleEndian32-methods}
\alias{WriteLittleEndian32,ZeroCopyOutputStream,integer-method}
\alias{WriteLittleEndian32,ZeroCopyOutputStream,numeric-method}
\alias{WriteLittleEndian32,ZeroCopyOutputStream,raw-method}

\alias{WriteLittleEndian64}
\alias{WriteLittleEndian64-methods}
\alias{WriteLittleEndian64,ZeroCopyOutputStream,integer-method}
\alias{WriteLittleEndian64,ZeroCopyOutputStream,numeric-method}
\alias{WriteLittleEndian64,ZeroCopyOutputStream,raw-method}

\alias{WriteVarint32}
\alias{WriteVarint32-methods}
\alias{WriteVarint32,ZeroCopyOutputStream,integer-method}
\alias{WriteVarint32,ZeroCopyOutputStream,numeric-method}
\alias{WriteVarint32,ZeroCopyOutputStream,raw-method}

\alias{WriteVarint64}
\alias{WriteVarint64-methods}
\alias{WriteVarint64,ZeroCopyOutputStream,integer-method}
\alias{WriteVarint64,ZeroCopyOutputStream,numeric-method}
\alias{WriteVarint64,ZeroCopyOutputStream,raw-method}


\title{Virtual Class "ZeroCopyOutputStream" }
\description{R wrapper for the ZeroCopyOutputStream c++ class}
\section{Objects from the Class}{
This is a virtual class
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to the \code{google::protobuf::io::ZeroCopyOutputStream} object}
  }
}
\section{Methods}{
\describe{
	\item{$}{\code{signature(x="ZeroCopyOutputStream")}: invokes a method}
	\item{Next}{\code{signature(object="ZeroCopyOutputStream", payload = "raw" )}: push the raw vector into the stream. Returns the number of bytes actually written.}
	\item{BackUp}{\code{signature(object="ZeroCopyOutputStream")}: Backs up a number of bytes, so that the end of the last buffer returned by \code{Next} is not actually written.}
	\item{ByteCount}{\code{signature(object="ZeroCopyOutputStream")}: Returns the total number of bytes written since this object was created. }
	\item{WriteRaw}{\code{signature(object="ZeroCopyOuputStream")}, payload = "raw": write the raw bytes to the stream}
}
}

\author{ Romain Francois <francoisromain@free.fr> }
\references{The \code{google::protobuf::io::ZeroCopyOutputStream} C++ class.}
\seealso{TODO: add classes that extend}
\keyword{classes}

\name{is_extension-methods}
\alias{is_extension}
\alias{is_extension-methods}
\title{Indicates if a field descriptor is an extension}
\description{
  Indicates if a field descriptor is an extension
}
\seealso{
  The method is implemented for the \linkS4class{FieldDescriptor} class
}
\keyword{methods}
\examples{
Person <- P( "tutorial.Person" )
is_extension(Person$id)
}

\name{has-methods}
\docType{methods}
\alias{has}
\alias{has-methods}
\alias{has,Message-method}

\title{Indicates if an object has the given field set}
\description{
This generic method, currently implemented
for \linkS4class{Message} and \linkS4class{EnumDescriptor} indicates if the
message or enum descriptor has the given field set. 

For messages and non-repeated fields, a call to the \code{HasField}
method of the corresponding \code{Message} is issued.

For messages and repeated fields, a call to the \code{FieldSize}
method is issued, and the message is declared to have 
the field if the size is greater than 0.

\code{NULL} is returned if the descriptor for the message does not
contain the given field at all.

For EnumDescriptors, a boolean value indicates if the given name is
present in the enum definition.
}
\section{Methods}{
	\describe{
		\item{has}{\code{signature(object = "Message")}:
		  Indicates if the message has a given field.}
                \item{has}{\code{signature(object = "EnumDescriptor")}:
		  Indicates if the EnumDescriptor has a given named element.}
	}
}
\examples{
unitest.proto.file <- system.file("tinytest", "data", "unittest.proto",
				  package = "RProtoBuf" )
readProtoFiles(file = unitest.proto.file)

test <- new(protobuf_unittest.TestAllTypes)
test$has("optional_int32")
# FALSE
test$add("repeated_int32", 1:10)
test$has("repeated_int32")
# TRUE
test$has("nonexistant")
# NULL

has(protobuf_unittest.TestAllTypes$NestedEnum, "FOO")
has(protobuf_unittest.TestAllTypes$NestedEnum, "BAR")
has(protobuf_unittest.TestAllTypes$NestedEnum, "XXX")
}
\keyword{methods}

\name{fetch-methods}
\docType{methods}
\alias{fetch}
\alias{fetch-methods}
\alias{fetch,Message-method}
\title{Fetch content of a repeated field}
\description{
Fetch content of a repeated field of a message
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{Fetch content of a message repeated field}
}}
\keyword{methods}

\name{P}
\alias{P}
\title{Protocol Buffer descriptor importer}
\description{
The \code{P} function searches for a protocol message
descriptor in the descriptor pool.
}
\usage{
P(type, file)
}
\arguments{
  \item{type}{Fully qualified type name of the protocol buffer or extension}
  \item{file}{optional proto file. If given, the definition
  contained in the file is first registered with the 
  pool of message descriptors}
}
\value{
An object of class \linkS4class{Descriptor} for message types or
\linkS4class{FieldDescriptor} for extensions.  An error is generated otherwise.
}
\author{
Romain Francois <francoisromain@free.fr>
}
\examples{
\dontrun{
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" ) 
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{ Person <- P("tutorial.Person") }

cat(as.character( Person ))
}
\keyword{ interface }

\name{field-methods}
\alias{field}
\alias{field-methods}
\title{Extract a field descriptor}
\description{
	Extract a \linkS4class{FieldDescriptor} from a 
	\linkS4class{Descriptor}
}
\seealso{
The method is implemented for the \linkS4class{Descriptor} class
}
\keyword{methods}

\name{FileDescriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{FileDescriptor-class}
\alias{as.character,FileDescriptor-method}
\alias{toString,FileDescriptor-method}
\alias{$,FileDescriptor-method}
\alias{show,FileDescriptor-method}
\alias{name,FileDescriptor-method}

\title{Class "FileDescriptor"}
\description{Class "FileDescriptor"}
\section{Objects from the Class}{
Objects are usually created using the \code{\link{fileDescriptor}} method
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to a \code{google::protobuf::FileDescriptor} C++ object }
	  \item{\code{package}:}{the package name defined in the file, e.g. 'tutorial'.}
	  \item{\code{filename}:}{the filename of this FileDescriptor}
  }
}
\section{Methods}{
	\describe{
		\item{$}{\code{signature(x = "FileDescriptor")}: used to invoke a pseudo method of the
		file descriptor or get a top level message, enum or service descriptor}
		\item{toString}{\code{signature(x = "FileDescriptor" )} : gets the debug string}
		\item{as.character}{\code{signature(x = "FileDescriptor" )} : gets the debug string}
		\item{show}{\code{signature(x = "FileDescriptor" )} : prints small text}
		\item{name}{\code{signature(object = "FileDescriptor" )} : name of the file}
	}
}
\author{ Romain Francois <francoisromain@free.fr> }
\keyword{classes}
\seealso{
	\linkS4class{Descriptor}
}
\examples{
# example proto file supplied with this package
desc <- P("tutorial.Person")
person <- new(desc)

person$fileDescriptor()
name(person$fileDescriptor())
# [1] "addressbook.proto"
as.character(person$fileDescriptor())
}

\name{RProtoBuf-package}
\alias{RProtoBuf-package}
\alias{RProtoBuf}
\docType{package}
\title{R Interface to the Protocol Buffers API}
\description{
Protocol Buffers are a way of encoding structured data in an
efficient yet extensible format. Google uses Protocol Buffers for almost all
of its internal RPC protocols and file formats.

This package provides R API to create, manipulate, parse and serialize
protocol buffer messages from R
}
\author{Romain Francois, Dirk Eddelbuettel, Murray Stokely and Jeroen Ooms.}
\references{\url{https://github.com/eddelbuettel/rprotobuf}}
\keyword{package}
\seealso{\linkS4class{Message} for some examples}

\examples{
\dontrun{
# an example proto file
system.file( "proto", "addressbook.proto", package = "RProtoBuf" )

# create a message of type AddressBook, defined in the example proto file
demo( "addressbook", package = "RProtoBuf" )

# using R binary connections and files to read and write messages
demo( "io", package = "RProtoBuf" )

# more documentation in the vignette
vignette( "RProtoBuf", package = "RProtoBuf" )
}
}

\name{clone-methods}
\docType{methods}
\alias{clone}
\alias{clone-methods}
\alias{clone,Message-method}
\title{Clone protocol buffer messages}
\description{
Generic "clone" function and associated 
method for \linkS4class{Message} objects 
}
\section{Methods}{
\describe{
	\item{\code{signature(object = "Message")}}{ clone the message }
}}
\examples{
\dontrun{
# example proto file supplied with this package
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )

# reading a proto file and creating the descriptor
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P("tutorial.Person")}

# creating a prototype message from the descriptor
sheep <- new( Person, email = "francoisromain@free.fr", id = 2 )

# cloning the sheep
newsheep <- clone( sheep )

# clone and update at once
newsheep <- clone( sheep, id = 3 )

# this can also be used as a pseudo method
sheep$clone()
sheep$clone( id = 3 )
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/serialize_pb.R
\name{serialize_pb}
\alias{can_serialize_pb}
\alias{serialize_pb}
\alias{unserialize_pb}
\title{Serialize R object to Protocol Buffer Message.}
\usage{
serialize_pb(object, connection, ...)
}
\arguments{
\item{object}{R object to serialize}

\item{connection}{passed on to  \code{\link{serialize}}}

\item{...}{additional arguments passed on to  \code{\link{serialize}}}
}
\description{
Serializes R objects to a general purpose protobuf message using the same
\code{rexp.proto} descriptor and mapping between R objects and protobuf
mesages as RHIPE.
}
\details{
Clients need both the message and the \code{rexp.proto} descriptor to parse
serialized R objects. The latter is included in the the package installation
\code{proto} directory: \code{system.file(package="RProtoBuf", "proto/rexp.proto")}

The following storage types are natively supported by the descriptor:
\code{character}, \code{raw}, \code{double}, \code{complex}, \code{integer},
\code{list}, and \code{NULL}. Objects with other storage types, such as
functions, environments, S4 classes, etc, are serialized using base R
\code{\link{serialize}} and stored in the proto \code{native} type.
Missing values, attributes and numeric precision will be preserved.
}
\examples{
msg <- tempfile();
serialize_pb(iris, msg);
obj <- unserialize_pb(msg);
identical(iris, obj);

}

\name{clear-methods}
\docType{methods}
\alias{clear}
\alias{clear-methods}
\alias{clear,Message,character-method}
\alias{clear,Message,integer-method}
\alias{clear,Message,missing-method}
\alias{clear,Message,numeric-method}
\alias{clear,Message,raw-method}
\title{Clear a field or all fields of the message and set them to their default values}
\description{
Clear one field or all fields of the message and set them to their default values
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message", field = "missing")}}{Clear all fields of the message and set them to their default values }
\item{\code{signature(object = "Message", field = "character")}}{ Clear the field identified by its name }
\item{\code{signature(object = "Message", field = "integer")}}{ Clear the field identified by its tag number }
\item{\code{signature(object = "Message", field = "numeric")}}{ Clear the field identified by its tag number }
\item{\code{signature(object = "Message", field = "raw")}}{ Clear the field identified by its tag number}

}}
\keyword{methods}
\examples{
	message <- new( tutorial.Person, name = "dddd", email = "eeeeeee", id = 1 )
	writeLines( as.character( message ) )
	clear( message )
	# clear works also as a pseudo method :
	message$clear() 
	
	writeLines( as.character( message ) )
	
	# clear single fields
	message <- new( tutorial.Person, name = "dddd", email = "eeeeeee", id = 1 )
	message$clear( "name" )
	writeLines( as.character( message ) )
	
}

\name{enum_type-methods}
\alias{enum_type}
\alias{enum_type-methods}
\title{Extract an enum type descriptor for a nested type}
\description{
	Extract a \linkS4class{EnumDescriptor} contained in a 
	\linkS4class{Descriptor}
}
\seealso{
The method is implemented for the \linkS4class{Descriptor} class
}
\keyword{methods}

\name{completion}
\alias{.DollarNames.Message}
\alias{.DollarNames.Descriptor}
\alias{.DollarNames.EnumDescriptor}
\alias{.DollarNames.FieldDescriptor}
\alias{.DollarNames.FileDescriptor}
\alias{.DollarNames.ServiceDescriptor}
\alias{.DollarNames.MethodDescriptor}
\alias{.DollarNames.ZeroCopyInputStream}
\alias{.DollarNames.ZeroCopyOutputStream}
\title{
Completion support for protocol buffer messages and descriptors
}
\description{
These functions support completion of protocol buffer
messages and descriptors.
}
\usage{
\S3method{.DollarNames}{Message}(x, pattern = "")
\S3method{.DollarNames}{Descriptor}(x, pattern = "")
\S3method{.DollarNames}{EnumDescriptor}(x, pattern = "")
\S3method{.DollarNames}{FieldDescriptor}(x, pattern = "")
\S3method{.DollarNames}{FileDescriptor}(x, pattern = "")
\S3method{.DollarNames}{ServiceDescriptor}(x, pattern = "")
\S3method{.DollarNames}{MethodDescriptor}(x, pattern = "")
\S3method{.DollarNames}{ZeroCopyInputStream}(x, pattern = "")
\S3method{.DollarNames}{ZeroCopyOutputStream}(x, pattern = "")
}
\arguments{
\item{x}{message (\linkS4class{Message}) or descriptor (\linkS4class{Descriptor}) }
\item{pattern}{filter}
}
\value{
Character vector containing potential completions. 

For \linkS4class{Message} objects, completions are 
the fields of the message and a set of pseudo methods ("has")

For \linkS4class{EnumDescriptor} objects, completions
are the names of the possible constants

For \linkS4class{Descriptor} objects, completions
are the names of the fields, enum types and nested message types
defined in the associated message type. 

For \linkS4class{FileDescriptor} objects, completions
are the names of the top-level descriptors (message, enum or service)
contained in the associated file, or pseudo methods.

}
\author{
Romain Francois <francoisromain@free.fr>
}
\examples{
# creating a prototype message from the descriptor
p <- new( tutorial.Person )

.DollarNames( p )
.DollarNames( tutorial.Person )
# but this is usually used with the <TAB> expansion on the command line
# <TAB> means "press the TAB key"
# p$<TAB>
# Person$<TAB>

}
\keyword{ programming }

\name{field_count-methods}
\alias{field_count}
\alias{field_count-methods}
\title{The number of fields}
\description{
The number of fields 
}
\seealso{
The method is implemented for the \linkS4class{Descriptor} class
}
\keyword{methods}

\name{asMessage}
\alias{asMessage}
\title{
coerce an object to a protobuf message
}
\description{
	coerce an object to the \linkS4class{Message} class. 
	This is a short-hand to the \link{as} method with the 
	\code{Class} argument set to "Message"
}
\usage{
asMessage(x, ...)
}
\arguments{
  \item{x}{object to coerce to a protobuf message}
  \item{\dots}{Passed to \code{\link{as}} }
}
\value{
	a \linkS4class{Message} object
}
\author{
Romain Francois <francoisromain@free.fr>
}
\examples{
# coerce a message type descriptor to a message
asMessage( tutorial.Person )

# coerce a enum descriptor
asMessage( tutorial.Person.PhoneType )

# coerce a field descriptor
asMessage( tutorial.Person$email )

# coerce a file descriptor 
asMessage( fileDescriptor( tutorial.Person ) )
}
\keyword{ programming }

\name{fileDescriptor-methods}
\docType{methods}
\alias{fileDescriptor}
\alias{fileDescriptor-methods}
\alias{fileDescriptor,Descriptor-method}
\alias{fileDescriptor,Message-method}
\alias{fileDescriptor,EnumDescriptor-method}
\alias{fileDescriptor,FieldDescriptor-method}
\alias{fileDescriptor,ServiceDescriptor-method}
\alias{fileDescriptor,MethodDescriptor-method}

\title{gets the file descriptor of an object}
\description{
Gets the file descriptor of an object
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Descriptor")}}{ retrieves the file descriptor associated with this descriptor}
\item{\code{signature(object = "Message")}}{ retrieves the file descriptor associated with the descriptor of this message }
\item{\code{signature(object = "EnumDescriptor")}}{ retrieves the file descriptor associated with the enum descriptor }
\item{\code{signature(object = "FieldDescriptor")}}{ retrieves the file descriptor associated with the field descriptor }
\item{\code{signature(object = "ServiceDescriptor")}}{ retrieves the file descriptor associated with the service descriptor }
\item{\code{signature(object = "MethodDescriptor")}}{ retrieves the file descriptor associated with the method descriptor }
}}
\keyword{methods}

\name{FileInputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{FileInputStream-class}
\alias{close,FileInputStream-method}
\alias{GetErrno,FileInputStream-method}
\alias{SetCloseOnDelete,FileInputStream-method}

\title{Class "FileInputStream"  }
\description{A \linkS4class{ZeroCopyInputStream} reading from a file}
\section{Objects from the Class}{
Objects can be created by the \link{FileInputStream} function
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{ External pointer to the \code{google::protobuf::io::FileInputStream} C++ object}
  }
}
\section{Extends}{
Class \code{"\linkS4class{ZeroCopyInputStream}"}, directly.
}
\section{Methods}{
	\describe{
		\item{close}{\code{signature(con="FileInputStream")}:
			Flushes any buffers and closes the underlying file.
			Returns false if an error occurs during the process; use \code{GetErrno} to examine the error
		}
		\item{GetErrno}{\code{signature(object="FileInputStream")}:
			If an I/O error has occurred on this file descriptor, this is the errno from that error.
			Otherwise, this is zero. Once an error occurs, the stream is broken and all subsequent operations will fail.
		}
		\item{SetCloseOnDelete}{\code{signature(object="FileInputStream")}:
			set the close on delete behavior.
		}
	}
	See \linkS4class{ZeroCopyInputStream} for inherited methods
}
\references{ The \code{FileInputStream} class from the protobuf C++ library.}
\author{ Romain Francois <francoisromain@free.fr> }
\seealso{
	\linkS4class{ZeroCopyInputStream} for methods
}
\keyword{classes}

\name{ConnectionInputStream-methods}
\docType{methods}
\alias{ConnectionInputStream}
\alias{ConnectionInputStream-methods}
\alias{ConnectionInputStream,connection-method}
\title{Creates an ConnectionInputStream}
\description{
Constructor for \linkS4class{ConnectionInputStream} objects
}
\section{Methods}{
\describe{
\item{\code{signature(object="connection")}}{Creates a \linkS4class{ConnectionInputStream} reading from the given R binary connection.}
}}
\keyword{methods}

\name{number-methods}
\alias{number}
\alias{number-methods}
\title{Gets the declared tag number of a field}
\description{
Gets the declared tag number of a field
}
\seealso{
The method is implemented for \linkS4class{FieldDescriptor} and
\linkS4class{EnumValueDescriptor} classes.
}
\keyword{methods}
\examples{
\dontrun{
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{ Person <- P("tutorial.Person") }

number(Person$id)
number(Person$email)
as.character(Person)

number(value(tutorial.Person$PhoneType, name="HOME"))
}

\name{descriptor-methods}
\docType{methods}

\alias{descriptor}
\alias{descriptor-methods}
\alias{descriptor,Message-method}
\title{Get the descriptor of a message}
\description{
	Get the \linkS4class{Descriptor} associated
	with a \linkS4class{Message}
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{ Get the descriptor of the message, as a \linkS4class{Descriptor} instance }
}}
\keyword{methods}

\name{bytesize-methods}
\docType{methods}
\alias{bytesize}
\alias{bytesize-methods}
\alias{bytesize,Message-method}
\title{The number of bytes taken by a message }
\description{
The number of bytes taken by a \linkS4class{Message}
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{ The number of bytes the message would take when serialized }
}}
\keyword{methods}
\examples{
	message <- new( tutorial.Person, name = "dddd", email = "eeeeeee", id = 1 )
	bytesize( message )
	\dontshow{
		stopifnot( all.equal( bytesize(message), length(serialize(message, NULL ) ) ) )
	}
}

\name{ArrayInputStream-methods}
\docType{methods}
\alias{ArrayInputStream}
\alias{ArrayInputStream-methods}
\alias{ArrayInputStream,raw,missing-method}
\alias{ArrayInputStream,raw,integer-method}
\alias{ArrayInputStream,raw,numeric-method}
\title{Creates an ArrayInputStream}
\description{
Constructor for \linkS4class{ArrayInputStream} objects
}
\section{Methods}{
\describe{
\item{\code{signature(payload = "raw", block_size = "missing" )}}{Creates a \linkS4class{ArrayInputStream} using the raw vector as the payload of the stream}
\item{\code{signature(payload = "raw", block_size = "integer" )}}{Creates a \linkS4class{ArrayInputStream} ... same with block size. }
\item{\code{signature(payload = "raw", block_size = "numeric" )}}{Creates a \linkS4class{ArrayInputStream} ... same with block size. }
}}
\keyword{methods}

\name{ZeroCopyInputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{ZeroCopyInputStream-class}

\alias{Skip,ZeroCopyInputStream-method}
\alias{ByteCount,ZeroCopyInputStream-method}
\alias{BackUp,ZeroCopyInputStream-method}
\alias{Next,ZeroCopyInputStream,missing-method}
\alias{$,ZeroCopyInputStream-method}

\alias{ReadRaw}
\alias{ReadRaw-methods}
\alias{ReadRaw,ZeroCopyInputStream,integer-method}
\alias{ReadRaw,ZeroCopyInputStream,numeric-method}

\alias{ReadString}
\alias{ReadString-methods}
\alias{ReadString,ZeroCopyInputStream,integer-method}
\alias{ReadString,ZeroCopyInputStream,numeric-method}

\alias{ReadVarint32}
\alias{ReadVarint32-methods}
\alias{ReadVarint32,ZeroCopyInputStream-method}

\alias{ReadLittleEndian32}
\alias{ReadLittleEndian32-methods}
\alias{ReadLittleEndian32,ZeroCopyInputStream-method}

\alias{ReadLittleEndian64}
\alias{ReadLittleEndian64-methods}
\alias{ReadLittleEndian64,ZeroCopyInputStream-method}

\alias{ReadVarint64}
\alias{ReadVarint64-methods}
\alias{ReadVarint64,ZeroCopyInputStream-method}


\title{Virtual Class "ZeroCopyInputStream" }
\description{R wrapper for the ZeroCopyInputStream c++ class}
\section{Objects from the Class}{
This is a virtual class
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{external pointer to the \code{google::protobuf::io::ZeroCopyInputStream} object}
  }
}
\section{Methods}{
\describe{
	\item{$}{\code{signature(x="ZeroCopyInputStream")}: invokes a method}
	\item{Next}{\code{signature(object="ZeroCopyInputStream")}: Get a number of bytes from the stream as a raw vector. }
	\item{Skip}{\code{signature(object="ZeroCopyInputStream")}: skip a number of bytes}
	\item{BackUp}{\code{signature(object="ZeroCopyInputStream")}: Backs up a number of bytes, so that the next call to \code{Next} returns data again that was already returned by the last call to \code{Next}.}
	\item{ByteCount}{\code{signature(object="ZeroCopyInputStream")}: Returns the total number of bytes read since this object was created. }
	\item{ReadRaw}{\code{signature(object="ZeroCopyInputStream", size = "integer")}: read raw bytes from the stream}
	\item{ReadRaw}{\code{signature(object="ZeroCopyInputStream", size = "numeric")}: read raw bytes from the stream}
	\item{ReadString}{\code{signature(object="ZeroCopyInputStream", size = "integer")}: same as \code{ReadRaw} but formats the result as a string}
	\item{ReadString}{\code{signature(object="ZeroCopyInputStream", size = "numeric")}: same as \code{ReadRaw} but formats the result as a string}
	\item{ReadVarint32}{\code{signature(object="ZeroCopyInputStream")}: Read an unsigned integer with Varint encoding, truncating to 32 bits. }
	\item{ReadLittleEndian32}{\code{signature(object="ZeroCopyInputStream")}: Read a 32-bit little-endian integer.  }
	\item{ReadLittleEndian64}{\code{signature(object="ZeroCopyInputStream")}: Read a 64-bit little-endian integer. In R the value is stored as a \code{double} which looses some precision (no other way) }
	\item{ReadVarint64}{\code{signature(object="ZeroCopyInputStream")}: Read a 64-bit integer with varint encoding. In R the value is stored as a \code{double} which looses some precision (no other way) }
}
}

\author{ Romain Francois <francoisromain@free.fr> }
\references{The \code{google::protobuf::io::ZeroCopyInputStream} C++ class.}
\seealso{TODO: add classes that extend}
\keyword{classes}

\name{BackUp-methods}
\docType{methods}
\alias{BackUp}
\alias{BackUp-methods}
\title{Backs up a number of bytes from a stream}
\description{
Backs up a number of bytes from a stream
}
\seealso{
	\linkS4class{ZeroCopyInputStream} implements BackUp.
}
\keyword{methods}

\name{as.list.Message}
\alias{as.list.Message}
\alias{as.list.Descriptor}
\alias{as.list.EnumDescriptor}
\alias{as.list.FileDescriptor}
\alias{as.list.ServiceDescriptor}
\title{
Grab the protocol buffer message as an R list
}
\description{
Utility to grab the protocol buffer message as 
an R list, with one item per field. 
}
\usage{
\S3method{as.list}{Message}(x, ...)
\S3method{as.list}{Descriptor}(x, ...)
\S3method{as.list}{EnumDescriptor}(x, ...)
\S3method{as.list}{FileDescriptor}(x, ...)
\S3method{as.list}{ServiceDescriptor}(x, ...)
}
\arguments{
	\item{x}{A protocol buffer message, instance of \linkS4class{Message}, 
	or a protocol message descriptor, instance of \linkS4class{Descriptor} }
	\item{\dots}{ignored}
}
\value{
For messages, a list of the content of the fields is 
returned.

For message type descriptors, a list containing
nested type descriptors (\linkS4class{Descriptor} objects),
enum type descriptors (\linkS4class{EnumDescriptor} objects),
then field descriptors (\linkS4class{FieldDescriptor} objects) in that order.

For enum descriptors, a named list of the enumerated values.

For file descriptors, a named list of descriptors defined in the
specified file descriptor.

For service descriptors, ...

}
\author{
Romain Francois <francoisromain@free.fr>
}
\examples{
Person <- P( "tutorial.Person" )
romain <- new( Person, email = "francoisromain@free.fr", id = 1 )
as.list( romain )
as.list( Person )
as.list( Person$PhoneType)
}
\keyword{ programming }

\name{ByteCount-methods}
\docType{methods}
\alias{ByteCount}
\alias{ByteCount-methods}
\title{The number of bytes read/written since the object was created}
\description{
The number of bytes read/written since the object was created
}
\seealso{
	\linkS4class{ZeroCopyInputStream} implements ByteCount.
}
\keyword{methods}

\name{type-methods}

\alias{type}
\alias{type-methods}

\alias{cpp_type}
\alias{cpp_type-methods}

\alias{TYPE_DOUBLE}
\alias{TYPE_FLOAT}
\alias{TYPE_INT64}
\alias{TYPE_UINT64}
\alias{TYPE_INT32}
\alias{TYPE_FIXED64}
\alias{TYPE_FIXED32}
\alias{TYPE_BOOL}
\alias{TYPE_STRING}
\alias{TYPE_GROUP}
\alias{TYPE_MESSAGE}
\alias{TYPE_BYTES}
\alias{TYPE_UINT32}
\alias{TYPE_ENUM}
\alias{TYPE_SFIXED32}
\alias{TYPE_SFIXED64}
\alias{TYPE_SINT32}
\alias{TYPE_SINT64}

\alias{CPPTYPE_INT32}
\alias{CPPTYPE_INT64}
\alias{CPPTYPE_UINT32}
\alias{CPPTYPE_UINT64}
\alias{CPPTYPE_DOUBLE}
\alias{CPPTYPE_FLOAT}
\alias{CPPTYPE_BOOL}
\alias{CPPTYPE_ENUM}
\alias{CPPTYPE_STRING}
\alias{CPPTYPE_MESSAGE}

\title{Gets the type or the C++ type of a field}
\description{
Gets the type or the C++ type of a field
}
\arguments{
  \item{object}{A \linkS4class{FieldDescriptor} object.}
  \item{as.string}{If true, print a string representation of the type.}
}
\seealso{
The method is implemented for the \linkS4class{FieldDescriptor} class
}
\keyword{methods}
\examples{
\dontrun{
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{Person <- P( "tutorial.Person" )}
type(Person$id)
type(Person$id, as.string=TRUE)
cpp_type(Person$email)
cpp_type(Person$email, TRUE)
}

\name{GetErrno-methods}
\docType{methods}
\alias{GetErrno}
\alias{GetErrno-methods}
\title{Get the error number for an I/O error}
\description{
If an I/O error has occurred on this file descriptor, 
this is the errno from that error
}
\section{Methods}{
	See classes \linkS4class{FileInputStream} and 
	\linkS4class{FileOutputStream} for implementations.
}
\keyword{methods}

\name{swap-methods}
\docType{methods}
\alias{swap}
\alias{swap-methods}
\alias{swap,Message-method}
\title{swap elements of a repeated field of a message}
\description{
swap elements of a repeated field of a message.
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{swap elements of a repeated field of a message}
}}
\references{See the \code{SwapElements} of the \code{Reflection} class, part of the protobuf library.}
\keyword{methods}

\name{containing_type-methods}
\alias{containing_type}
\alias{containing_type-methods}
\title{Gets the message type descriptor that contains a descriptor}
\description{
Gets a \linkS4class{Descriptor} describing the message type 
that contains the descriptor. 
}
\seealso{
The method is implemented for these classes : 
\linkS4class{Descriptor}, \linkS4class{EnumDescriptor}, 
\linkS4class{FieldDescriptor}
}
\examples{
# Containing type of a field is the message descriptor
tutorial.Person$id$containing_type()

# No containing type for the top-level message descriptor.
tutorial.Person$containing_type()
}
\keyword{methods}

\name{set-methods}
\docType{methods}
\alias{set}
\alias{set-methods}
\alias{set,Message-method}
\title{set a subset of values of a repeated field of a message}
\description{
set a subset of values of a repeated field of a message
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{set a subset of values of a repeated field of a message}
}}
\keyword{methods}

\name{read-methods}
\docType{methods}
\alias{read}
\alias{read-methods}
\alias{read,Descriptor,character-method}
\alias{read,Descriptor,raw-method}
\alias{read,Descriptor,ANY-method}
\title{Read a protocol buffer message from a connection}
\description{
	Read a \linkS4class{Message} from a 
	connection using its associated \linkS4class{Descriptor}
}
\section{Methods}{
\describe{
	\item{\code{signature(descriptor = "Descriptor", input = "character")}}{ Read the message from a file }
	\item{\code{signature(descriptor = "Descriptor")}}{ Read from a binary connection.  }
	\item{\code{signature(descriptor = "Descriptor", input = "raw")}}{ Read the message from a raw vector }
}}
\keyword{methods}
\examples{
# example file that contains a "tutorial.AddressBook" message
book <- system.file( "examples", "addressbook.pb", package = "RProtoBuf" )

# read the message
message <- read( tutorial.AddressBook, book )

# or using the pseudo method 
message <- tutorial.AddressBook$read( book )

# write its debug string
writeLines( as.character( message ) )

# grab the name of each person
sapply( message$person, function(p) p$name )

# read from a binary file connection
f <- file( book, open = "rb" )
message2 <- read( tutorial.AddressBook, f )
close( f )

# read from a message payload (raw vector)
payload <- readBin( book, raw(0), 5000 )
message3 <- tutorial.AddressBook$read( payload )

\dontshow{
stopifnot( identical( message, message2) )
stopifnot( identical( message, message3) )
}

}

\name{ConnectionOutputStream-methods}
\docType{methods}
\alias{ConnectionOutputStream}
\alias{ConnectionOutputStream-methods}
\alias{ConnectionOutputStream,connection-method}
\title{Creates an ConnectionOutputStream}
\description{
Constructor for \linkS4class{ConnectionOutputStream} objects
}
\section{Methods}{
\describe{
\item{\code{signature(object="connection")}}{
	Creates a \linkS4class{ConnectionOutputStream} writing to the given R binary connection.}
}}
\keyword{methods}

\name{nested_type_count-methods}
\alias{nested_type_count}
\alias{nested_type_count-methods}
\title{The number of fields}
\description{
The number of fields 
}
\seealso{
The method is implemented for the \linkS4class{Descriptor} class
}
\keyword{methods}

\name{readProtoFiles}
\alias{readProtoFiles}
\alias{readProtoFiles2}
\alias{resetDescriptorPool}
\title{protocol buffer descriptor importer}
\description{
  Imports proto files into the descriptor pool that 
  is then used by the \code{P} function to resolve
  message type names.
}
\usage{
  readProtoFiles(files, dir, package="RProtoBuf", pattern="\\\\.proto$", lib.loc=NULL)
  readProtoFiles2(files, dir=".", pattern="\\\\.proto$", recursive=FALSE, protoPath=getwd())
  resetDescriptorPool()
}
\arguments{
  \item{files}{Proto files}
  \item{dir}{Directory. If \code{files} is not specified, files
  with the "proto" extension in the \code{dir} directory 
  are imported}
  \item{package}{R package name. If \code{files} and \code{dir} are
  missing, "proto" files in the "proto" directory of the 
  package tree are imported.}
  \item{pattern}{A filename pattern to match proto files when using \code{dir}.}
  \item{recursive}{Whether to descend recursively into \code{dir}.}
  \item{lib.loc}{Library location.}
  \item{protoPath}{Search path for proto file imports.}
}
\details{
  \code{readProtoFiles2} is different from \code{readProtoFiles} to be
  consistent with the behavior of \code{protoc} command line tool in
  being explicit about the search path for proto import statements. In addition,
  we also require that both \code{files} and \code{dir} arguments are
  interpreted relative to \code{protoPath}, so that there is consistency in
  future imports of the same files through import statements of other proto
  files.

  \code{resetDescriptorPool} clears all imported proto definitions.
}
\value{\code{NULL}, invisibly.}
\author{Romain Francois <francoisromain@free.fr>}
\seealso{\code{\link{P}}}
\examples{
\dontrun{
# from a package
readProtoFiles(package = "RProtoBuf")

# from a directory
proto.dir <- system.file("proto", package = "RProtoBuf")
readProtoFiles(dir = proto.dir)

# set of files
proto.files <- list.files(proto.dir, full.names = TRUE)
readProtoFiles(proto.files)
}
}
\keyword{programming}

\name{add-methods}
\docType{methods}
\alias{add}
\alias{add-methods}
\alias{add,Message-method}
\title{add elements of a repeated field of a message}
\description{
Add elements to a repeated field of a message.
}
\section{Methods}{
\describe{
\item{\code{signature(object = "Message")}}{add elements to a repeated field of a message}
}}
\examples{
unitest.proto.file <- system.file("tinytest", "data", "unittest.proto",
				  package = "RProtoBuf" )
readProtoFiles(file = unitest.proto.file)

test <- new(protobuf_unittest.TestAllTypes)
test$add("repeated_int32", 1)
test$add("repeated_int32", 2:10)
test$repeated_int32
}
\keyword{methods}

\name{label-methods}

\alias{label}
\alias{label-methods}

\alias{LABEL_OPTIONAL}
\alias{LABEL_REQUIRED}
\alias{LABEL_REPEATED}

\title{Gets the label of a field}
\description{
Gets the label of a field (optional, required, or repeated).
}
\arguments{
  \item{object}{A \linkS4class{FieldDescriptor} object.}
  \item{as.string}{If true, print a string representation of the type.}
}
\seealso{
The method is implemented for the \linkS4class{FieldDescriptor} class
}
\keyword{methods}
\examples{
\dontrun{
proto.file <- system.file( "proto", "addressbook.proto", package = "RProtoBuf" )
Person <- P( "tutorial.Person", file = proto.file )
}
\dontshow{ Person <- P("tutorial.Person") }

label(Person$id)
label(Person$email)
label(Person$phone)
label(Person$id, TRUE)
label(Person$email, TRUE)
label(Person$phone, TRUE)
LABEL_OPTIONAL
LABEL_REQUIRED
LABEL_REPEATED
}

\name{MethodDescriptor-class}
\Rdversion{1.1}
\docType{class}
\alias{MethodDescriptor-class}
\alias{as.character,MethodDescriptor-method}
\alias{toString,MethodDescriptor-method}
\alias{$,MethodDescriptor-method}
\alias{$<-,MethodDescriptor-method}

\alias{input_type}
\alias{input_type-methods}
\alias{input_type,MethodDescriptor-method}

\alias{output_type}
\alias{output_type-methods}
\alias{output_type,MethodDescriptor-method}

\title{Class "MethodDescriptor"}
\description{R representation of Service Descriptors}
\section{Objects from the Class}{
TODO
}
\section{Slots}{
  \describe{
  	  \item{\code{pointer}:}{External pointer to a \code{google::protobuf::MethodDescriptor} C++ object }
  	  \item{\code{name}:}{fully qualified name of the method}
  	  \item{\code{service}:}{fully qualified name of the service that defines this method}
  }
}
\section{Methods}{
  \describe{
    \item{as.character}{\code{signature(x = "MethodDescriptor")}: debug string of the method }
    \item{toString}{\code{signature(x = "MethodDescriptor")}: debug string of the method}
	\item{$}{\code{signature(x = "MethodDescriptor")}: ... }
	\item{$<-}{\code{signature(x = "MethodDescriptor")}: ... }
	\item{input_type}{\code{signature(object = "MethodDescriptor")}: the \linkS4class{Descriptor} of the input type of the method}
	\item{output_type}{\code{signature(object = "MethodDescriptor")}: the \linkS4class{Descriptor} of the output type of the method }
  }
}
\author{ Romain Francois <francoisromain@free.fr> }
\keyword{classes}
